# Accounts.pm - The Object Class that provides a Accounts Object
#$Id: Accounts.pm,v 1.11 2005/01/24 23:26:55 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.

package Portal::Accounting::Accounts;
use strict;
use Portal::Accounting::Main;
use Portal::Accounting::Objects::Account;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.11 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Chart of Accounts",
  new => "New Account",
  save => "Save Account",
  edit => "Edit Account",
  delete => "Delete Account",
  assign => "Assign Accounts",
  );

=head1 NAME

Accounts - Object used to work with accounts

=head1 SYNOPSIS

  use Portal::Accounting::Accounts;
  my $obj = Portal::Accounting::Accounts->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Accounts is the Accounts Menu from which every other part of
the systems will be accessed.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Accounts module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the Accounts State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{commands} = \%commands;

  # do validation
  if (!$self->Portal::Accounting::Accounts::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  my $l = $self->{langObj};
  $self->{metaTypes} = { 1 => $l->map("Asset"),
    2 => $l->map("Liability"),
    3 => $l->map("Equity"),
    4 => $l->map("Revenue"),
    5 => $l->map("Expense"),
  };
  return $self;
}

=item % getCommands(void)

  This method returns the commands hash and is only to be
  used by the registerApp method for creating the permissions,
  if using the AppStateCommandSecurity implementation.

=cut
sub getCommands
{
  return %commands;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject c_display()

=cut
sub c_display
{
  my $self = shift;
  my %args = ( command => "display", @_ );
  my $command = $args{command};
  my $doc = $self->prepDoc;
  my $siteDir = $self->{configObj}->{siteDir};
  my $url = $self->{baseURL};
  my $refreshUrl = $url . "?$self->{getInfo}&command=display";
  my $l = $self->{langObj};
  my $message;
  my $lc;
  if (defined $self->input->{AccountCode}) { $lc = $self->input->{AccountCode}; }
  if (defined $self->input->{message}) { $lc = $self->input->{message}; }

  my $chart = $self->load(dbHandle => $self->{dbHandle}, langObj => $self->{langObj}, AccountCode => $lc);
  if ($chart == 0 ) { $doc->print($self->errorMessage); return $doc;  }

  $doc->print($self->printHTML);
  return $doc;
}

sub c_new
{
  my $self = shift;
  return $self->c_edit(@_);
}

sub c_save
{
  my $self = shift;
  return $self->c_edit(@_);
}

=item HTMLObject c_edit()

  this routine is to create/edit a new account

=cut
sub c_edit
{
  my $self = shift;
  my $doc = $self->prepDoc;
  my %args = ( @_ );
  my $command = $self->{input}->{command};
  my $l = $self->{langObj};
  my $lc = "";
  if (defined $self->{input}->{ledgerCode}) { $lc = $self->{input}->{ledgerCode}; }
  my $account = Portal::Accounting::Objects::Account->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, meta => $self->{input}->{meta}, ledgerCode => $lc);
  if ($account->error) { $doc->print($account->errorMessage); return $doc;  }
  #how do i know if this is edit or new?
  if ($self->{input}->{command} eq "save")
  { #validate user input then react
    if ($account->validateForm(input => $self->{input}), formName => "main")
    {
      my $result = $account->populate(%{$self->{input}});
      if (not defined $result) { $self->error($account->errorMessage);  return undef;  }
      if ($result) #populate worked
      {
        if ($account->isNew) #insert Account obj
        {
          $account->insert();
        }
        else
        {
          $account->update();
        }
        if ($account->error)
        {
          $doc->print("Error".$account->errorMessage());
        }
        else
        {
          $doc->setFocus("body");
          $doc->print("Success");
          my $ChartOfAccounts = $l->map("Chart of Accounts");
          $doc->print(<<"ENDOF");
  <form name="main">
    <input type="submit" name="submit" value="$ChartOfAccounts">
    <input type="hidden" name="app" value="Accounting">
    <input type="hidden" name="state" value="Accounts">
    <input type="hidden" name="command" value="display">
    <input type="hidden" name="ledgerCode" value="$lc">
  </form>
ENDOF
          if ($account->debug) { $doc->print($account->debugString); };
          return $doc;
        }
      }
      else { $doc->print($account->genErrorString("all"));  }
    }
    #invalid so let form redisplay
    else
    {
      $doc->print($account->errorMessage);
      if ($account->debug) { $doc->print($account->debugString); };
    }
  }
  elsif ($command eq "edit")
  {
    $account->populate(id => $self->{input}->{id});
    if (!$account->select())
    {
      $doc->print("Error".$account->errorMessage());
      if ($account->debug) { $doc->print($account->debugString); };
    }
  }
  #display form
  $account->resetError;
  my %form = $account->printForm(formName => "main");
  if ($account->error) {  $doc->print($account->errorMessage); }
  else {  $doc->print(%form);  }

  if ($account->debug) { $doc->print($account->debugString); };
  return $doc;
}

sub load
{
  my $self = shift;
  my %args = ( dbHandle => undef, ledgerCode => "", app => "Accounting", state => "Accounts", @_ );

  $self->{ledgerCode} = $args{ledgerCode};
  $self->{dbHandle} = $args{dbHandle};
  $self->{app} = $args{app};
  $self->{state} = $args{state};
  # instantiate anything unique to this module
  my $sql = "SELECT name, number AS code, number, type, active, taxable";
  $sql .= ", (SELECT first_digit FROM acct_type_tb WHERE num = type) AS first_digit";
  $sql .= " FROM account_tb ORDER BY first_digit, name";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  while (my $data = $result->fetchrow_hashref)
  {
    my $assigned = "SELECT ledger_code FROM local_accounts_tb WHERE acct_num = '$data->{number}'";
    my $adb = $self->{dbHandle}->read($assigned);
    if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
    $data->{assigned} = { };
    while (my $a = $adb->fetchrow_hashref())
    {
      $data->{assigned}->{$a->{ledger_code}} = $a->{ledger_code};
    }
    push (@{$self->{data}}, $data);
  }
  my $sql = "SELECT name, num FROM acct_type_tb";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  while (my $data = $result->fetchrow_hashref)
  {
    $self->{types}->{$data->{num}} = $self->langObj->map($data->{name});
  }
  # do anything else you might need to do.
  return 1;
}

#take the approach that this chart of accounts is mainly for assignement and to link to editing of accounts or new.
#if i do this on the client side i can simply
#
#1 send down raw data
#2 have client display data with markup etc.
#3 client can enable, disable assignment button as user changes ledger
sub printHTML
{
  my $self = shift;
  my $l = $self->langObj;
  my $click4new = $l->map("Create"). " ". $l->map("Account");
  my $click2edit = $l->map("Edit"). " ". $l->map("Account");
  my $meta = "";
  my $Name = $l->map("Account");
  my $Code = $l->map("Code");
  my $Type = $l->map("Type");
  my $Active = $l->map("Active");
  my $Taxable = $l->map("Taxable");
  my $Assigned = $l->map("Assign");
  my $showActive = $l->map("Show InActive");
  my $hideActive = $l->map("Hide InActive");
  my $click = $l->map("Click here to Assign");
  my $lc = "";
  if (defined $self->{input}->{ledgerCode}) { $lc = $self->{input}->{ledgerCode}; }

  my %return;
  $return{css} = <<"ENDOF";
  .chartOfAccounts { width: 70%; }
  .heading  { text-align: left; padding: 5; }
  .name     { text-align: left; }
  .code     { text-align: right; padding-right: 5;}
  .type     { text-align: left; }
  .active   { text-align: center; }
  .taxable  { text-align: center; }
  .assigned { text-align: center; }
  .heading { text-align: left; padding:5; font-weight: bold; background-color: #ffffff; }
  .active_yes   { }
  .active_no { display: none; }
ENDOF
  $return{javascript} = <<"ENDOF";
function showInActive()
{
  var rules = document.styleSheets[0].cssRules;
  var index = -1;
  for (i = 0; i < rules.length; i++)
  {
    if (rules[i].cssText.match(/active_no/) != null)
    {
      index = i;
      break;
    }
  }
  document.styleSheets[0].cssRules[index].style.display = 'table-row';
}

function hideInActive()
{
  var rules = document.styleSheets[0].cssRules;
  var index = -1;
  for (i = 0; i < rules.length; i++)
  {
    if (rules[i].cssText.match(/active_no/) != null)
    {
      index = i;
      break;
    }
  }
  document.styleSheets[0].cssRules[index].style.display = 'none';
}

function toggleInActive()
{
  var me = document.mainForm;
  if (me.toggle.value == "show")
  {
    me.toggle.value = "hide";
    return hideInActive();
  }
  else
  {
    me.toggle.value = "show";
    return showInActive();
  }
}

function toggleAssign(id)
{
  eval("if (document.mainForm.assign_"+id+".checked == 1) { assignAccount("+id+");  } else { unAssignAccount("+id+");  }");
}

function assignAccount(id)
{
  eval("var un = document.mainForm.unAssign.value.replace(/"+id+":/g, '');");
  document.mainForm.unAssign.value = un;
  document.mainForm.assign.value = document.mainForm.assign.value + id + ':';
}

function unAssignAccount(id)
{
  eval("var a = document.mainForm.assign.value.replace(/"+id+":/g, '');");
  document.mainForm.assign.value = a;
  document.mainForm.unAssign.value = document.mainForm.unAssign.value + id + ':';
}

ENDOF
  $return{body} .= <<"ENDOF";
<form name="mainForm">
<input type="hidden" name="app" value="$self->{app}">
<input type="hidden" name="state" value="$self->{state}">
<input type="hidden" name="command" value="assign">
<input type="hidden" name="ledgerCode" value="$lc">
<input type="hidden" name="toggle" value="hide">
<input type="hidden" name="assign" value="">
<input type="hidden" name="unAssign" value="">
<table class="edit" width="100%">
 <tr>
  <th class="name">$Name</th>
  <th class="code">$Code</th>
  <th class="type">$Type</th>
  <th class="active"><a href="#" title="$showActive" onclick="toggleInActive(); return false;">$Active</a></th>
  <th class="taxable">$Taxable</th>
  <th class="assigned"><input type="submit" name="submit" value="$Assigned" title="$click"></th>
 </tr>
ENDOF
  foreach my $d (@{$self->{data}})
  {
    if ($meta ne $d->{first_digit})
    {
      $return{body} .= <<"ENDOF";
 <tr class="heading">
  <td class="name" colspan="6"><br><a title="$click4new" href="$self->{baseURL}?app=$self->{app}&state=$self->{state}&command=new&ledgerCode=$lc&meta=$d->{first_digit}">$self->{metaTypes}->{$d->{first_digit}}</a></td>
 </tr>
ENDOF
    }
    $meta = $d->{first_digit}; #set to next group so that heading will only be output on changes
    my $class = "active_yes";
    if ($d->{active} == 0) { $class = "active_no";  }
    $return{body} .= " <tr class=\"$class\">\n";
    $return{body} .= "  <td class=\"name\"><a href=\"$self->{baseURL}?app=$self->{app}&state=$self->{state}&command=edit&ledgerCode=$lc&meta=$d->{first_digit}&id=$d->{number}\" title=\"$click2edit\">$d->{name}</a></td>\n";
    $return{body} .= "  <td class=\"code\">$d->{code}</td>\n";
    $return{body} .= "  <td class=\"type\">$self->{types}->{$d->{type}}</td>\n";
    if ($d->{active} == 0) { $return{body} .= "  <td class=\"active\">[ &nbsp;]</td>\n" }
    else { $return{body} .= "  <td class=\"active\">[X]</td>\n" }
    if ($d->{taxable} == 0) { $return{body} .= "  <td class=\"taxable\">[ &nbsp;]</td>\n" }
    else { $return{body} .= "  <td class=\"taxable\">[X]</td>\n" }
    #figure assigned
    if ($self->{input}->{ledgerCode} ne "")
    {
      if (exists $d->{assigned}->{$self->{input}->{ledgerCode}})
      {  #check the assigned button on
        $return{body} .= "  <td class=\"assigned\"><input type=\"checkbox\" name=\"assign_$d->{number}\" onclick=\"toggleAssign($d->{number});\" checked=\"1\" /></td>\n";
      }
      else
      {  #check the assigned button off
        $return{body} .= "  <td class=\"assigned\"><input type=\"checkbox\" name=\"assign_$d->{number}\" onclick=\"toggleAssign($d->{number});\" /></td>\n";
      }
    }
    else
    {
      #disalbe assign button
      $return{body} .= "  <td class=\"assigned\"><input type=\"checkbox\" name=\"assign_$d->{number}\" onclick=\"toggleAssign($d->{number});\" disabled=\"1\" /></td>\n";
    }
    $return{body} .= " </tr>\n";
  }
  my $Asset = $l->map("Asset");
  my $Liability = $l->map("Liability");
  my $Equity = $l->map("Equity");
  my $Revenue = $l->map("Revenue");
  my $Expense = $l->map("Expense");
  my $help = $l->map("Help4ChartOfAccounts");
  $return{body} .= <<"ENDO";
 <tr>
  <th class="name">$Name</th>
  <th class="code">$Code</th>
  <th class="type">$Type</th>
  <th class="active"><a href="#" title="$showActive" onclick="toggleInActive(this.form); return false;">$Active</a></th>
  <th class="taxable">$Taxable</th>
  <th class="assigned"><input type="submit" name="submit" value="$Assigned" title="$click"></th>
 </tr>
 </table>
</form>
  <p>$help
  </p>
  <p><form name="new">
  <input type="hidden" name="app" value="$self->{app}">
  <input type="hidden" name="state" value="$self->{state}">
  <input type="hidden" name="command" value="new">
  <input type="hidden" name="ledgerCode" value="$lc">
  <input type="radio" name="meta" value="1" checked>$Asset&nbsp;&nbsp;&nbsp;
  <input type="radio" name="meta" value="2">$Liability&nbsp;&nbsp;&nbsp;
  <input type="radio" name="meta" value="3">$Equity&nbsp;&nbsp;&nbsp;
  <input type="radio" name="meta" value="4">$Revenue&nbsp;&nbsp;&nbsp;
  <input type="radio" name="meta" value="5">$Expense&nbsp;&nbsp;&nbsp;
  <input type="submit" name="submit" value="$click4new">
  </p>
ENDO
  return %return;
}

sub c_assign
{
  my $self = shift;
  my $doc = $self->prepDoc();
  #walk assign unassign vars and build up arrays
  my $assign = $self->{input}->{assign};
  my $unassign = $self->{input}->{unAssign};
  $assign =~ s/:$//;
  $unassign =~ s/:$//;
  my @assign = split(":", $assign);
  my @unassign = split(":", $unassign);
  #call assign
  if (@assign > 0)
  {
    $assign = $self->assignAccounts(ledgerCode => $self->{input}->{ledgerCode}, accounts => \@assign );
    if (! defined $assign) { $doc->print($self->errorMessage);  }
  }
  #call unassign
  if (@unassign > 0)
  {
    $unassign = $self->unAssignAccounts(ledgerCode => $self->{input}->{ledgerCode}, accounts => \@unassign );
    if (! defined $unassign) { $doc->print($self->errorMessage);  }
  }
  #output # records and link back to chart of accounts
  my $r = $self->{langObj}->map("records modified");
  my $chart = $self->{langObj}->map("Chart of Accounts");
  $doc->print($assign + $unassign . " $r.<br>\n");
  $doc->print(<<"ENDOF");
  <form name="mainForm">
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="command" value="display">
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
    <input type="submit" name="submit" value="$chart">
  </form>
ENDOF
  return $doc;
}

=item integer assignAccounts(ledgerCode => , accounts => \@accounts)

  This routine will assign the given accounts to the given ledger.
  ledgerCode - string
  accounts - an arrayref to array of account numbers

=cut
sub assignAccounts
{
  my $self = shift;
  my %args = ( ledgerCode => "", accounts => undef, @_ );
  my $l = $args{ledgerCode};
  my $a = $args{accounts};
  my $records = 0;
  if ($l eq "") { $self->error("Invalid ledgerCode in assignAccounts<br>\n");  return undef;  }
  if (! defined $a) { $self->error("No accounts given in assignAccounts<br>\n");  return undef;  }
  foreach my $account (@{$a})
  {
    if ($account !~ /^\d+$/) { $self->error("Invalid account given in assignAccounts: '$account'<br>\n");  return undef;  }
    my $sql = "DELETE FROM local_accounts_tb WHERE ledger_code = '$l' AND acct_num = '$account'; ";
    $sql .= "INSERT INTO local_accounts_tb (acct_num, ledger_code, taxable) VALUES (?, ?, ";
    $sql .= " (SELECT taxable FROM account_tb WHERE number = ?)";
    $sql .= ")";
    my @plug = ( $account, $l, $account );
    my $result = $self->{dbHandle}->write(sql => $sql, plug => \@plug);
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{dbHandle}->errorMessage);
      $self->{dbHandle}->rollback;
      return undef;
    }
    $records++;
  }
  $self->{dbHandle}->commit;
  return $records;
}

=item integer unAssignAccounts(ledgerCode => , accounts => \@accounts)

  This routine will unassign the given accounts from the given ledger.
  ledgerCode - string
  accounts - an arrayref to array of account numbers

=cut
sub unAssignAccounts
{
  my $self = shift;
  my %args = ( ledgerCode => "", accounts => undef, @_ );
  my $l = $args{ledgerCode};
  my $a = $args{accounts};
  my $records = 0;
  if ($l eq "") { $self->error("Invalid ledgerCode in assignAccounts<br>\n");  return undef;  }
  if (! defined $a) { $self->error("No accounts given in assignAccounts<br>\n");  return undef;  }
  foreach my $account (@{$a})
  {
    if ($account !~ /^\d+$/) { $self->error("Invalid account given in assignAccounts: '$account'<br>\n");  return undef;  }
    my $sql = "DELETE FROM local_accounts_tb WHERE ledger_code = '$l' AND acct_num = '$account'; ";
    my $result = $self->{dbHandle}->write(sql => $sql);
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{dbHandle}->errorMessage);
      $self->{dbHandle}->rollback;
      return undef;
    }
    $records++;
  }
  $self->{dbHandle}->commit;
  return $records;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
