# Main.pm - The Object Class that provides a Main Object
#$Id: Base.pm,v 1.22 2005/01/24 22:25:54 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by James Pattie & Jason A. Pattie, 11/07/2000.

package Portal::Accounting::Base;
use strict;
use HTMLObject::Base;
use HTMLObject::Normal;
use HTMLObject::FrameSet;
use Portal::AppState;
use Portal::Data::Variables;
use Portal::Accounting::Data::Variables;
use Date::Calc qw(:all);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::AppState Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.22 $';

# global variables.

=head1 NAME

Main - Object used to build a Main Menu for accounting package

=head1 SYNOPSIS

  use Portal::Accounting::Main;
  my $obj = Portal::Accounting::Main->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Main is the Main Menu from which every other part of
the systems will be accessed.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Main module.
 See Portal::AppState(3) for a listing of required arguments.

 Instantiates an instance of the Main State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( release => "", DBVersion => "", @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{state} = (split /::/, ref $self)[2];
  $self->{release} = $args{release};
  $self->{DBVersion} = $args{DBVersion};
  $self->{codeVersion} = $args{codeVersion};
  $self->{portalVariables} = Portal::Data::Variables->new(langObj => $self->{langObj});
  if ($self->{portalVariables}->didErrorOccur)
  {
    $self->setError($self->{portalVariables}->errorMessage);
    return $self;
  }
  $self->{variables} = Portal::Accounting::Data::Variables->new(langObj => $self->{langObj});
  if ($self->{variables}->didErrorOccur)
  {
    $self->prefixError($self->{variables}->errorMessage);
    return $self;
  }
  $self->{jsDir}         = $self->{methods}->createBaseURL(type => "App",    linkType => "js",       appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{imageDir}      = $self->{methods}->createBaseURL(type => "App",    linkType => "image",    appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{cssDir}        = $self->{methods}->createBaseURL(type => "App",    linkType => "css",      appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{dbDump}        = $self->{methods}->createBaseURL(type => "App",    linkType => "db_dump",  appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{oldCGIDir}     = $self->{methods}->createBaseURL(type => "App",    linkType => "js",       appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} )."accounting/cgi-bin";
  $self->{portalCGIDir}  = $self->{methods}->createBaseURL(type => "Portal", linkType => "cgi",      appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{baseURL}       = $self->{methods}->createBaseURL(type => "App",    linkType => "cgi",      appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});
  $self->{output} = { app => $self->{app}, state => $self->{state}, command => $self->{input}->{command} };

  #the state is not set until runtime
  $self->{postInfo} .= "<input type=\"hidden\" name=\"app\" value=\"$self->{app}\">\n";
  $self->{postInfo} .= "<input type=\"hidden\" name=\"state\" value=\"$self->{state}\">\n";
  $self->{getInfo} .= "app=$self->{app}&state=$self->{state}";

  # do validation
  if (!$self->Portal::Accounting::Base::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  return $self;
}

sub prepDoc
{
  my $self = shift;
  my %foo;
  my $doc = HTMLObject::Normal->new;
  my $colorScheme = $self->{applicationObj}->getCurrentColorScheme(userId => $self->{userObj}->{id});
  if ($self->{applicationObj}->error)
  {
    $self->prefixError($self->{applicationObj}->errorMessage);
    return undef;
  }
  if (!defined $colorScheme)
  {
    $self->error("No valid colorScheme returned!<br>\n");
    return undef;
  }
  my %cssEntries = $self->{methods}->generateColorSchemeCSS(colorScheme => $colorScheme, appName => $self->{app});
  if ($self->{methods}->error)
  {
    $self->error($self->{methods}->errorMessage);
    return undef;
  }
  $doc->print(%cssEntries);

  my $title = $self->{langObj}->map($self->{commands}->{$self->{input}->{command}});
  $doc->setTitle( $title);
  $doc->setJavascriptInclude(file => $self->{jsDir}."Accounting.js");
  $foo{body} = "<center><h3>$title</h3></center>";
  $doc->print( %foo );
  return $doc;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (length $self->{release} == 0)
  {
    $self->invalid("release", $self->{release});
  }
  if (length $self->{DBVersion} == 0)
  {
    $self->invalid("DBVersion", $self->{DBVersion});
  }
  if (length $self->{codeVersion} == 0)
  {
    $self->invalid("codeVersion", $self->{codeVersion});
  }
  if (not defined $self->{methods})
  { $self->missing("methods");  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
