# StockPriceUpdate.pm - The Object Class that provides a StockPriceUpdate Object
# Created by James Pattie & Jason A. Pattie, 03/06/2001.

#$Id: StockPriceUpdate.pm,v 1.11 2005/01/05 22:31:49 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::DB::StockPriceUpdate;
use Portal::Accounting::Data::StockQuote;
use Portal::Accounting::DB::Main;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::DB::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.04';

=head1 NAME

StockPriceUpdate - Object used to build a stock price update Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::DB::StockPriceUpdate;
  my $obj = Portal::Accounting::DB::StockPriceUpdate->new(langObj);
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

StockPriceUpdate is a class that provides the ability to obtain updated (delayed)
quotes and historical quote information from the Internet by calling a helper
object that takes care of retrieving the information.  This object then takes
that information and updates the stock price table with the data.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::Accounting::DB::StockPriceUpdate object.
 See Portal::Accounting::DB::Main(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item array updateStockPrices(tickerList, startDate, endDate, type, interval)

 Summary: this function retrieves data for the sent in list of tickers from the Internet by calling the StockQuote
           object methods to gather the data; it then inserts the data into the database in the stock_price_tb table
 Required: tickerList, [startDate, endDate] (if type = historical)
 Optional: type = current/historical (defaults to current price), interval (if type = historical)
  Returns: array where index 0 is bool (1 on success, 0 on failure)
           index 1 is array ref to goodTickers
           index 2 is array ref to badTickers

=cut

sub updateStockPrices
{
  my $self = shift;
  my %args = ( @_ );
  my $errorString = "";
  my $tickerList = $args{'tickerList'};
  my $startDate = $args{'startDate'};
  my $endDate = $args{'endDate'};
  my $type = $args{'type'};
  my $manualPrice = $args{'manualPrice'};
  my $interval = $args{'interval'};
  my @goodTickers = ();
  my @badTickers = ();

  if (not defined $args{'tickerList'})
  {
    $errorString = "tickerList must be specified.";
    $self->setError(errorString => $errorString);
    return (0, \@goodTickers, \@badTickers);
  }
  if (defined $type && $type ne "historical" && $type ne "current" && $type ne "manual")
  {
    $errorString = "Invalid type ($type).  Must be 'historical' or 'current' or 'manual'.";
    $self->setError(errorString => $errorString);
    return (0, \@goodTickers, \@badTickers);
  }
  if (defined $type && $type eq "manual" && ((not defined $args{'manualPrice'} || $manualPrice eq "") && (not defined $args{'startDate'} || $startDate eq "")))
  {
    $errorString = "'manualPrice' and 'startDate' must be specified when type => 'manual'.";
    $self->setError(errorString => $errorString);
    return (0, \@goodTickers, \@badTickers);
  }

  if ((not defined $type) || $type eq "current" || ($type ne "current" && $type ne "historical" && $type ne "manual"))
  {
    # get the prices for the list of stocks
    (my $result, my $currentPrices) = Portal::Accounting::Data::StockQuote::getStockQuotes(tickerList => $tickerList);
    if ($result ne "")
    {
      $errorString = "Looking up current prices failed.<br>" . $result;
      $self->setError(errorString => $errorString);
      return (0, \@goodTickers, \@badTickers);
    }
    my %priceHash = %{$currentPrices};
    # insert the prices into the database
    foreach my $ticker (keys %priceHash)
    {
      if ($ticker eq "") { next;  }
      push @goodTickers, $ticker;
      my $stockObj = $self->getStockInfo(ticker => $ticker);
      if ($self->didErrorOccur())
      {
        $errorString = "Failed to lookup stock information for ticker ($ticker).<br>" . $self->errorMessage;
        $self->setError(errorString => $errorString);
        return (0, \@goodTickers, \@badTickers);
      }
      my @tempArray = keys %{$priceHash{$ticker}};
      my $date = $tempArray[0];
      my $sth = $self->{dbHandle}->write(sql => "DELETE FROM stock_price_tb WHERE sub_acct = ? AND date_of = ?", plug => [ $stockObj->{number}, $date ]);
      if ($self->{dbHandle}->didErrorOccur())
      {
        $errorString = "Problem occurred while deleting the stock prices for sub-account ($stockObj->{number}), ticker ($ticker) on date ($date).<br>" . $self->{dbHandle}->errorMessage;
        $self->setError(errorString => $errorString);
        $self->{dbHandle}->rollback;
        return (0, \@goodTickers, \@badTickers);
      }
      $sth = $self->{dbHandle}->write(sql => "INSERT INTO stock_price_tb (date_of, price, sub_acct) VALUES (?, ?, ?)", plug => [ $date, $priceHash{$ticker}{$date}, $stockObj->{number} ]);
      if ($self->{dbHandle}->didErrorOccur())
      {
        $errorString = "Problem occurred while inserting the stock price for sub-account ($stockObj->{number}), ticker ($ticker) on date ($date).<br>" . $self->{dbHandle}->errorMessage;
        $self->setError(errorString => $errorString);
        $self->{dbHandle}->rollback;
        return (0, \@goodTickers, \@badTickers);
      }
    }
    foreach my $ticker (@{$tickerList})
    {
      if ($ticker eq "") {  next;  }
      if (not grep /^$ticker$/, @goodTickers)
      {
        push @badTickers, $ticker;
      }
    }
  }
  elsif ($type eq "manual")
  {
    # insert the prices into the database
    foreach my $ticker (@{$tickerList})
    {
      if ($ticker eq "") {  next;  }
      push @goodTickers, $ticker;
      my $stockObj = $self->getStockInfo(ticker => $ticker);
      if ($self->didErrorOccur())
      {
        $errorString = "Failed to lookup stock information for ticker ($ticker).<br>" . $self->errorMessage;
        $self->setError(errorString => $errorString);
        return (0, \@goodTickers, \@badTickers);
      }
      my $sth = $self->{dbHandle}->write(sql => "DELETE FROM stock_price_tb WHERE sub_acct = ? AND date_of = ?", plug => [ $stockObj->{number}, $startDate ]);
      if ($self->{dbHandle}->didErrorOccur())
      {
        $errorString = "Problem occurred while deleting the stock prices for sub-account ($stockObj->{number}), ticker ($ticker) on date ($startDate).<br>" . $self->{dbHandle}->errorMessage;
        $self->setError(errorString => $errorString);
        $self->{dbHandle}->rollback;
        return (0, \@goodTickers, \@badTickers);
      }
      $sth = $self->{dbHandle}->write(sql => "INSERT INTO stock_price_tb (date_of, price, sub_acct) VALUES (?, ?, ?)", plug => [ $startDate, $manualPrice, $stockObj->{number} ]);
      if ($self->{dbHandle}->didErrorOccur())
      {
        $errorString = "Problem occurred while inserting the stock price for sub-account ($stockObj->{number}), ticker ($ticker) on date ($startDate).<br>" . $self->{dbHandle}->errorMessage;
        $self->setError(errorString => $errorString);
        $self->{dbHandle}->rollback;
        return (0, \@goodTickers, \@badTickers);
      }
    }
  }
  elsif ($type eq "historical")
  {
    my @dateResult = $self->{methods}->fixupDate(date => $startDate);
    if (!$dateResult[0])
    {
      $self->setError("startDate = '$startDate' is invalid!<br>$dateResult[2]\n");
      return (0, \@goodTickers, \@badTickers);
    }
    else
    {
      # make sure the date is "fixed".
      $startDate = $dateResult[1];
    }
    @dateResult = $self->{methods}->fixupDate(date => $endDate);
    if (!$dateResult[0])
    {
      $self->setError("endDate = '$endDate' is invalid!<br>$dateResult[2]\n");
      return (0, \@goodTickers, \@badTickers);
    }
    else
    {
      # make sure the date is "fixed".
      $endDate = $dateResult[1];
    }
    if ($interval !~ /^(d|w|m)/i)
    {
      $interval = "d"; # daily prices
    }
    else
    {
      $interval =~ /^(.)/;
      $interval = $1;
    }

    $startDate =~ /^(\d{4})-(\d{2})-(\d{2})$/;
    my $startMonth = $2;
    my $startDay = $3;
    my $startYear = $1;
    $endDate =~ /^(\d{4})-(\d{2})-(\d{2})$/;
    my $endMonth = $2;
    my $endDay = $3;
    my $endYear = $1;

    # get the prices for the list of stocks
    foreach my $ticker (@{$tickerList})
    {
      if ($ticker eq "") {  next;  }
      my $stockObj = $self->getStockInfo(ticker => $ticker);
      if ($stockObj->market eq "None") {  next;  }
      if ($self->didErrorOccur())
      {
        $errorString = "Failed to lookup stock information for ticker ($ticker).<br>" . $self->errorMessage;
        $self->setError(errorString => $errorString);
        return (0, \@goodTickers, \@badTickers);
      }
      (my $result, my $historicalPrices) = Portal::Accounting::Data::StockQuote::getHistoricalQuotes(ticker => $ticker, interval => $interval, startDay => $startDay, startMonth => $startMonth, startYear => $startYear, endDay => $endDay, endMonth => $endMonth, endYear => $endYear);
      if ($result ne "")
      {
        $errorString = "Looking up historical prices failed on $ticker:$stockObj->{market}:$stockObj->{name}<br>" . $result;
        $self->setError(errorString => $errorString);
        return (0, \@goodTickers, \@badTickers);
      }
      my %priceHash = %{$historicalPrices};
      if (keys %priceHash == 0)
      {
        push @badTickers, $ticker;
      }
      else
      {
        push @goodTickers, $ticker;
      }
      foreach my $date (keys %priceHash)
      {
        my $sth = $self->{dbHandle}->write(sql => "DELETE FROM stock_price_tb WHERE sub_acct = ? AND date_of = ?", plug => [ $stockObj->{number}, $date ]);
        if ($self->{dbHandle}->didErrorOccur())
        {
          $errorString = "Problem occurred while deleting the stock prices for sub-account ($stockObj->{number}), ticker ($ticker) on date ($date).<br>" . $self->{dbHandle}->errorMessage;
          $self->setError(errorString => $errorString);
          $self->{dbHandle}->rollback;
          return (0, \@goodTickers, \@badTickers);
        }
        $sth = $self->{dbHandle}->write(sql => "INSERT INTO stock_price_tb (date_of, price, sub_acct) VALUES (?, ?, ?)", plug => [ $date, $priceHash{$date}, $stockObj->{number} ]);
        if ($self->{dbHandle}->didErrorOccur())
        {
          $errorString = "Problem occurred while inserting the stock price for sub-account ($stockObj->{number}), ticker ($ticker) on date ($date).<br>" . $self->{dbHandle}->errorMessage;
          $self->setError(errorString => $errorString);
          $self->{dbHandle}->rollback;
          return (0, \@goodTickers, \@badTickers);
        }
      }
    }
  }

  # everything looks good
  $self->{dbHandle}->commit;
  if ($self->{dbHandle}->didErrorOccur())
  {
    $errorString = "Problem committing work.<br>" . $self->{dbHandle}->errorMessage;
    $self->setError(errorString => $errorString);
    $self->{dbHandle}->rollback;
    return (0, \@goodTickers, \@badTickers);
  }

  return (1, \@goodTickers, \@badTickers);
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting::DB::Main(3)

=cut
