# Account.pm - The Object Class that provides a Account Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: Account.pm,v 1.7 2005/01/24 23:27:22 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Account;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Id: Account.pm,v 1.7 2005/01/24 23:27:22 moreejt Exp $';

=head1 NAME

Account - Object used to create, edit, save an Account.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Account;
  my $accountObj = Portal::Accounting::Objects::Account->new();
  $populate(
    name => "test", number => "1234", type => 1,
    code => "1234", active => "true", taxable => "true");
  if ($didErrorOccur())
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

Account is a Account class.  This represents an account in the
accounting system.  Accounts are grouped by their type and must have
unique names and numbers.  Right now number and code are the same but
eventually code will be user defined and number will be internally derived.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>)

 Creates a new instance of the Portal::Accounting::Objects::Account module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( meta => "", ledgerCode => "", @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  $self->{vars}  = { foo => "string" };
  $self->{usingForms} = 1;
  my $l = $self->{langObj};
  my $Submit = $l->map("Save");
  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Accounts" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode} },
    "command" => { -Type => "hidden", -Value => "save" },
    "new" => { -Type => "hidden", -Value => "1" },
    "name" => { -Label => $l->map("Name"), -Type => "text", size => 15 },
    "code" => { -Label => $l->map("Code"), -Type => "text", size => 10 , -Value => $self->suggestCode($args{meta}) },
    "id" => { -Label => $l->map("Id"), -Type => "hidden", -Value => "-1" },
    "taxable" => { -Label => $l->map("Taxable"), -Type => "checkbox", -Boolean => 1 , -onload => "if(document.main.meta.value > 0 && document.main.meta.value < 4) { this.disabled = true; }"},
    "active" => { -Label => $l->map("Active"), -Type => "checkbox",  -Boolean => 1, checked => "1" },
    "type" => { -Label => $l->map("Type"), -Type => "select", -Options => $self->getAccountTypes4Select($args{meta}), },
    "meta" => { -Value => $args{meta}, -Type => "hidden" },
  };
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  #FIELD=id#
  #FIELD=new#
  #FIELD=meta#
  <table class="main">
    <tr>
      <td>#LABEL=name#</td><td>#FIELD=name#</td>
    </tr><tr>
      <td>#LABEL=code#</td><td>#FIELD=code#</td>
    </tr><tr>
      <td>#LABEL=type#</td><td>#FIELD=type#</td>
    </tr><tr>
      <td>#LABEL=taxable#</td><td>#FIELD=taxable#
    </tr><tr>
      </td>
      <td>#LABEL=active#</td><td>#FIELD=active#</td>
    </tr>
  </table>
  <input type="submit" name="Submit" value="$Submit">
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
function taxCheck
{

}
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( name code id active taxable) ],
    optional => [  ],
    constraints => {
      id => qr/^\d\d+$/,
    },
  };
#   $self->{formObj} = HTMLObject::Form->new();
  $self->{defaultForm} = "main";
#   $self->{action} = "";
#   $self->{method} = "post";
  ##db infrastructure
  $self->{id} = "id";
  $self->{INSERTorder}[0] = [ qw( name code type active taxable ) ] ;  #insert order for db column names
  $self->{newOnly}[0] = { number => "" };
  $self->{db2data}[0] = { number => 'id' };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = reverse %{$self->{db2data}[0]};

  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM account_tb WHERE number = #FIELD=id#";
  $self->{INSERT}[0] = "INSERT INTO account_tb (number, ##COLUMNS##) VALUES ( ";
  $self->{INSERT}[0] .= "  next_account( ";
  $self->{INSERT}[0] .= "    (SELECT MAX(account_tb.number) FROM account_tb, acct_type_tb WHERE account_tb.type = acct_type_tb.num AND acct_type_tb.first_digit = '#FIELD=meta#'))";
  $self->{INSERT}[0] .= ", ##VALUES##)";
  $self->{UPDATE}[0] = "UPDATE account_tb SET ##COLUMNS## WHERE number = #FIELD=id#";
  #remember to use single quotes around ##foo## if needed
  $self->{DELETE}[0] = "DELETE FROM account_tb WHERE number = #FIELD=id#";  #is 'id' a hardcoded form element?
  #find out ifis this being referenced from other tables before deleting
  $self->{preDELETE}[0] = "SELECT debitcredit_tb.parent_num FROM debitcredit_tb WHERE debitcredit_tb.account_num = #FIELD=id#";
  #what should we output as the value of the thing just deleted?
  $self->{postDELETE}[0] = "SELET name||', '||code||', '||number FROM account_tb WHERE number = '#FIELD=id#'";

  return $self;
}

=item HTMLObject::Form::Options getAccountType4Select() or getAccountType4Select(meta)

  This routine selects the account types from the database and creates select options to put on an HTMLObject::Form select element.
  1 - 5 will limit to Assets (1), Liabilities(2), etc.

=cut
sub getAccountTypes4Select
{
  my $self = shift;
  my $l = $self->{langObj};
  my $meta = "";
  if (scalar @_ == 1) { $meta = @_[0]; }
  if ($meta ne "")
  {
    if ($meta =~ /^[1-5]$/)
    {
      $meta = " WHERE first_digit = '$meta' ";
    }
    else
    {
      $self->error("Invalid meta for getAccountTypes4Select<br>\n");
      return undef;
    }
  }
  my $sql = "SELECT name, num, first_digit FROM acct_type_tb $meta ORDER BY first_digit, name";
  my $db = $self->{dbHandle};
  my $result = $db->read($sql);
  if ($db->error) { $self->error($db->errorMessage); return undef; }
  #walk this data and translate $l->map($d->name)
  #pull out first_digit
  my @foo;
  while (my $d = $result->fetchrow_arrayref)
  {
    my $pre = "";
    if ($d->[2] == 1) {  $pre = $l->map("Asset");  }
    elsif ($d->[2] == 2) {  $pre = $l->map("Liability");  }
    elsif ($d->[2] == 3) {  $pre = $l->map("Equity");  }
    elsif ($d->[2] == 4) {  $pre = $l->map("Revenue");  }
    elsif ($d->[2] == 5) {  $pre = $l->map("Expense");  }
    else {  $self->error("Invalid first_digit returned from record: ".$d->[0].", ".$d->[1].", ".$d->[2]."\n");  return undef;  }
    $pre .= ":";
    $d->[0] = $pre . $l->map($d->[0]);
    push (@foo, [$d->[0], $d->[1]]);
    $self->{firstDigit}->{$d->[1]} = $d->[2];
  }
  return $self->{formObj}->createSelectOptions(\@foo) ;
}

=item string suggestCode(code)

  This routine suggests a code for the account by putting them into a javascript array
  Optional:  code - 1-5 to return the next value for Asset (1), Liability(2), and so on

=cut
sub suggestCode
{
  my $self = shift;
  my $code = shift;
  if ($code !~ /^[1-5]$/) { $code = ""; }
  my $sql = "SELECT acct_type_tb.first_digit AS f, next_account(MAX(account_tb.number)) AS n WHERE account_tb.type = acct_type_tb.num GROUP BY acct_type_tb.first_digit";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef; }
  while (my $data = $result->fetchrow_hashref)
  {
    #if we roll to the next first_digit range increment by a factor of 10
    my $next = $data->{n};
    #use this if ever making an advanced create account that shows all types
    $self->{javascript}->{main} .= << "ENDOF";
  var firstDigits;
  firstDigits[$data->{f}] = $next;
function setCode(myForm)
{
  var type = myForm.type[myForm.type.selectedIndex].value;
  myForm.code.value = firstDigits[type];
}
ENDOF
    if ($code == $data->{f}) { $code = $next;  }
  }
  return $code;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub firstDigit
{
  my $self = shift;
  my $type = shift;
  my $f = "";
  my $sql = "SELECT first_digit FROM acct_type_tb WHERE num = '$type'";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef; }
  if (my $data = $result->fetchrow_hashref)
  {
    $f = $data->{first_digit};
  }
  else { $self->error("Invalid type. no first_digit returned for: $type");  return undef;  }
  return $f;
}

=item %form printForm(data => \%data, template => $template,
                   name => "mainForm", action => "",
                   method => "post", profile => \%profile)

=cut

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Base(3)

=cut
