# Base.pm - The Object Class that provides a Base Object
#$Id: Base.pm,v 1.17 2005/01/26 22:35:13 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Base;
use strict;
use Portal::Objects::Base;
use Portal::Data::Variables;
use Portal::Accounting::Data::Variables;
#use HTMLObject::Form;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT %Elements);

require Exporter;

@ISA = qw(Portal::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Id: Base.pm,v 1.17 2005/01/26 22:35:13 moreejt Exp $';
%Elements = ();

=head1 NAME

BaseObject - Base Object for deriving other objects.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Base;
  @ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);

=head1 DESCRIPTION

BaseObject is a Base class for Accounting Objects such as
Objects::Ledger and Objects::Account.  It contains methods for
handling database select, insert, update, delete.  Also contains
methods for handling forms which are defined in the child objects.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::Base.
    See Portal::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( debug => 0, dbHandle => undef, @_ );
  $self->{dbHandle} = $args{dbHandle};
  $self->{debug} = $args{debug};
  $self->{debugString} = "";
  $self->{action} = "";
  $self->{method} = "post";
  ##db infrastructure
  $self->{SELECT} = undef;
  $self->{UPDATE} = undef;
  $self->{INSERT} = undef;
  $self->{DELETE} = undef;
  $self->{preDELETE} = undef;
  $self->{postDELETE} = undef;
  $self->{INSERTorder} = undef;  #insert order for db column names
  $self->{newOnly} = undef;
  $self->{db2data} = undef;  #map database columns to ->data hash, if not specified we assume same name in db and object
  $self->{data2db} = undef;
  $self->{data}->{main} = { };
  $self->{data}->{main}->{new} = { -Value => 1, -Type => "hidden" } ;
  $self->{template}->{main} = { };
  $self->{profile}->{main} = { };
  $self->{javascript}->{main} = { };
  $self->{css}->{main} = { };

  #$self->{SELECT} = "SELECT ##COLUMNS## FROM foo_tb WHERE id = ##id##";
  #$self->{INSERT} = "INSERT INTO foo_tb (##COLUMNS##) VALUES (##VALUES##)";
  #$self->{UPDATE} = "UPDATE foo_tb SET (##COLUMNS##) WHERE id = ##id##";
  #remember to use single quotes around ##foo## if needed
  #$self->{DELETE} = "DELETE FROM foo_tb WHERE id = ##id##";  #is 'id' a hardcoded form element?
  # find out ifis this being referenced from other tables before deleting
  #$self->{preDELETE} = "SELECT bar_tb.id FROM bar_tb WHERE bar_tb.foo = ##id##";
  # what should we output as the value of the thing just deleted?
  #$self->{postDELETE} = "SELET name FROM foo_tb WHERE id = '##id##'";

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  eval { $self->{portalVariables} = Portal::Data::Variables->new(langObj => $self->{langObj}); };
  if ($@)
  {
    $self->setError($@);
    return $self;
  }
  if ($self->{portalVariables}->didErrorOccur)
  {
    $self->setError($self->{portalVariables}->errorMessage);
    return $self;
  }
  eval { $self->{variables} = Portal::Accounting::Data::Variables->new(langObj => $self->{langObj}); };
  if ($@)
  {
    $self->setError($@);
    return $self;
  }
  if ($self->{variables}->didErrorOccur)
  {
    $self->setError($self->{variables}->errorMessage);
    return $self;
  }

  # do validation
#   if (!$self->Portal::Accounting::Objects::Base::isValid)
#   {
#     # the error is set in the isValid() method.
#     return $self;
#   }

  # do anything else you might need to do.
  return $self;
}

=item bool isValid(void)

 Returns 0 for invalid or 1 for valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (not defined $self->{portalVariables})
  {
    $self->setError("portalVariables is not defined!<br>\n");
    return 0;
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean isNew

  1 for new, 0 for not

=cut
sub isNew
{
  my $self = shift;
  return $self->{data}->{main}->{new}->{-Value};
}

=item string dbRequired("db|data")

  This routine returns all of the data which is required by the select statements.
  optional:  db - to return the db column names, data - to return the data element names

=cut
sub dbRequired
{
  my $self = shift;
  my $basis = shift;
  if (! defined $basis)  {  $basis = "db";}
  else  {  $basis = "data";}
  my $return = "";
  #walk each select looking for #FIELD=#
  my @sqls = @{$self->{SELECT}};
  for (my $index = 0; $index < @sqls; $index++)
  {
    my $sql = $sqls[$index];
    while ($sql =~ /#FIELD=([^#]+)#/)
    {
      my $found = $1;
      my $map = $self->db2data(var => $found, index => $index);
      my $value = "foo";
      $sql =~ s/#FIELD=$found#/$value/g;
      if ($basis eq "db")  {  $return .= "$found:";  }
      else  {  $return .= "$map:";  }
    }
  }
  return $return;
}

=item boolean select()

  Use:  $obj->populate(id => $foo, other => $bar);
        $obj->select()

  This routine uses the $self->{INSERTorder} to automatically load the data
  from the sql call into the current object's ->{data} hash
  required:  must be popluated with critical data before calling select on the object
      To find out which data is critical call $obj->dbRequired
  optional:
  returns: undef on error, 1 for success, 0 for no records found

=cut
sub select
{
  my $self = shift;
  if (! defined $self->{SELECT} || ! defined $self->{SELECT}[0] || $self->{SELECT}[0] eq "") { $self->error("Must define sql select statement<br>\n"); return undef;  }
  #subst on sql in SELECT ARRAYS
  my @sqls = @{$self->{SELECT}};
  for (my $index = 0; $index < @sqls; $index++)
  {
    my $sql = $sqls[$index];
    my $columns = "";
    my $first = 1;
    foreach my $key (@{$self->{INSERTorder}[$index]})
    {
      if (!$first) {  $columns .= ",";  } else {  $first = 0; }
      $columns .= " $key ";
  #    if (defined $self->{db2data}->{$key}) { $columns .= " AS $self->{db2data}->{$key} ";  }
  #  cant do this bc SQL Is case INsensitive
    }
    #subsitute ##foo##
    if ($self->debug) {  $self->debug($sql);  }
    $sql =~ s/##COLUMNS##/$columns/g;
    while ($sql =~ /#FIELD=([^#]+)#/)
    {
      my $found = $1;
      my $value = $self->{data}->{main}->{$found}->{-Value};
      if ($self->debug) {  $self->debug("<br>s/#FIELD=$found#/$value/g<br>\n");  }
      $sql =~ s/#FIELD=$found#/$value/g;
    }
    if ($self->debug) {  $self->debug($sql);  }
    my $result = $self->{dbHandle}->read($sql);
    if ($self->{dbHandle}->error)
    {
      $self->error("<br>".$self->{dbHandle}->errorMessage."<br>\n\n$sql");
      return undef;
    }
    if (my $DB = $result->fetchrow_hashref)
    {
      foreach my $form (keys %{$self->{data}})
      {
        foreach my $key (keys %{$DB})
        {
          my $mapKey = $self->db2data(var => $key, index => $index);
          if ($self->{data}->{$form}->{$mapKey}->{-Type} eq "checkbox")
          {
            #data could be NULL/something or boolean TRUE/FALSE
            if ($DB->{$key} eq "t" || $DB->{$key} eq "true" || $DB->{$key} == 1)
            {
              $self->{data}->{$form}->{$mapKey}->{checked} = 1;
            }
            elsif ($DB->{$key} eq "f" || $DB->{$key} eq "false" || $DB->{$key} == 0)
            {
              $self->{data}->{$form}->{$mapKey}->{checked} = 0;
            }
            else #NULL OR empty or something non boolean
            {
              if ($DB->{$key} eq "") { $self->{data}->{$form}->{$mapKey}->{checked} = 0;  }
              else { $self->{data}->{$form}->{$mapKey}->{checked} = 1;  }
            }
          }
          else
          {
            $self->{data}->{$form}->{$mapKey}->{-Value} = $DB->{$key};
          }
          if (exists $self->{newOnly}[$index]->{$key})
          {
            $self->{data}->{$form}->{$mapKey}->{-ReadOnly} = 1;
          }
        }
        $self->{data}->{$form}->{new}->{-Value} = 0;
      }

    }
    else { return 0; }
  }
  return 1;
}

=item boolean insert(string)

  Routine that tries to take data in object and
  inserts it into the database from the main data hash
  optional: string - name of a different data hash to take data from
  returns: undef for error, 0 for failed, or 1 for success

=cut
sub insert
{
  my $self = shift;
  my $form = shift;
  if (not defined $form) { $form = "main";  }

  if (! defined $self->{INSERT} || ! defined $self->{INSERT}[0] || $self->{INSERT}[0] eq "") { $self->error("Must define sql INSERT statement<br>\n"); return undef;  }
  #subst on sql in INSERT ARRAYS
  my @sqls = @{$self->{INSERT}};
  for (my $index = 0; $index < @sqls; $index++)
  {
    my $sql = $sqls[$index];
    my $columns = "";
    my $values = "";
    my $first = 1;
    my @plug;
    foreach my $key (@{$self->{INSERTorder}[$index]})
    {
      if (!$first) {  $columns .= ","; $values .= ",";  } else {  $first = 0; }
      $columns .= " $key ";
      $values .= " ? ";
      my $map = $self->db2data(var => $key, index => $index);
      if ($self->{data}->{$form}->{$map}->{-Type} eq "checkbox")
      {
        if ($self->{data}->{$form}->{$map}->{checked} == 1)
        {
          if ($self->{data}->{$form}->{$map}->{-Boolean} )  { push @plug, 'true';  }
          elsif (defined $self->{data}->{$form}->{$map}->{-Value})
          {
            push @plug, $self->{data}->{$form}->{$map}->{-Value};
          }
          else { $self->error("Cannot use non boolean checkbox without a value set<br>\n"); return undef; }
        }
        else
        {
          if ($self->{data}->{$form}->{$map}->{-Boolean} )  { push @plug, 'false';  }
          elsif (defined $self->{data}->{$form}->{$map}->{-Value})
          {
            push @plug, undef;
          }
          else { $self->error("Cannot use non boolean checkbox without a value set<br>\n"); return undef; }
        }
      }
      else
      {
        if (defined $self->{data}->{$form}->{$map}->{-Value})
        {
          push @plug, $self->{data}->{$form}->{$map}->{-Value};
        }
        else {  push @plug, "NULL";  }
      }
    }
    #subsitute ##foo##
    $sql =~ s/##COLUMNS##/$columns/g;
    $sql =~ s/##VALUES##/$values/g;
    while ($sql =~ /#FIELD=([^#]+)#/)
    {
      my $found = $1;
      my $value = $self->{data}->{$form}->{$found}->{-Value};
      $sql =~ s/#FIELD=$found#/$value/g;
    }
    if ($self->debug)
    {
      my $foo = "";
      foreach my $p (@plug) { $foo .= $p.", "; }
      $self->debug($sql." ". $foo);
    }
    my $result = $self->{dbHandle}->write(sql => $sql, plug => \@plug);
    if ($self->{dbHandle}->error)
    {
      $self->error("<br>".$self->{dbHandle}->errorMessage."<br>\n\n$sql");
      return undef;
    }
  }
  $self->{dbHandle}->commit;
  return 1;
}

=item boolean update("string")

  Routine that tries to take data from internal object and
  update the database
  returns: undef for error, 0 for failed, or 1 for success

=cut
sub update
{
  my $self = shift;
  my $form = shift;
  if (! defined $form) { $form = "main";  }
  if (! defined $self->{UPDATE} || ! defined $self->{UPDATE}[0] || $self->{UPDATE}[0] eq "") { $self->error("Must define sql UPDATE statement<br>\n"); return undef;  }
  #subst on sql in UPDATE ARRAYS
  my @sqls = @{$self->{UPDATE}};
  for (my $index = 0; $index < @sqls; $index++)
  {
    my $sql = $sqls[$index];
    my $columns = "";
    my $first = 1;
    my @plug;
    foreach my $key (@{$self->{INSERTorder}[$index]})
    {
      if (exists $self->{newOnly}[$index]->{$key}) { next; }
      if (!$first) {  $columns .= ",";  } else {  $first = 0; }
      $columns .= " $key = ? ";
      my $map = $self->db2data(var => $key, index => $index);
      if ($self->{data}->{$form}->{$map}->{-Type} eq "checkbox")
      {
        if ($self->{data}->{$form}->{$map}->{checked} == 1)
        {
          if ($self->{data}->{$form}->{$map}->{-Boolean} )  { push @plug, 'true';  }
          elsif (defined $self->{data}->{$form}->{$map}->{-Value})
          {
            push @plug, $self->{data}->{$form}->{$map}->{-Value};
          }
          else { $self->error("Cannot use non boolean checkbox without a value set<br>\n"); return undef; }
        }
        else
        {
          if ($self->{data}->{$form}->{$map}->{-Boolean} )  { push @plug, 'false';  }
          elsif (defined $self->{data}->{$form}->{$map}->{-Value})
          {
            push @plug, 'NULL';
          }
          else { $self->error("Cannot use non boolean checkbox without a value set<br>\n"); return undef; }
        }
      }
      else
      {
        if (defined $self->{data}->{$form}->{$map}->{-Value})
        {
          push @plug, $self->{data}->{$form}->{$map}->{-Value};
        }
        else {  push @plug, "NULL";  }
      }
    }
    #subsitute ##foo##
    #"UPDATE foo_tb SET (##COLUMNS##) WHERE id = ##id##";
    $sql =~ s/##COLUMNS##/$columns/g;
    # search for ##foo## in $sql
    # subst on each one with db2data->{foo} -> {-Value}
    while ($sql =~ /#FIELD=([^#]+)#/)
    {
      my $found = $1;
      my $value = $self->{data}->{$form}->{$found}->{-Value};
      $sql =~ s/#FIELD=$found#/$value/g;
    }
    my $result = $self->{dbHandle}->write(sql => $sql, plug => \@plug);
    if ($self->{dbHandle}->error)
    {
      $self->error("<br>".$self->{dbHandle}->errorMessage."<br>\n\n$sql");
      return undef;
    }
  }
  $self->{dbHandle}->commit;
  return 1;
}

=item (integer, string) delete("string")

  This routine will attempt to delete the record for the object internal id.
  optional:  string - name of the data hash to use.  defaults to main
  returns: undef on error, array of (integer, string) on success
    integer: -1 if preDELETE finds a record or number of records if DELETE sql stmt is run
    string:  the DB record found if preDELETE stops the delete from happening,
             or the result of postDELETE if delete was successful

=cut
sub delete
{
  my $self = shift;
  my $form = shift;
  my $r = 0;
  if (! defined $form) { $form = "main"; }
  if (! defined $self->{DELETE} || ! defined $self->{DELETE}[0] || $self->{DELETE}[0] eq "") { $self->error("Must define sql DELETE statement<br>\n"); return undef;  }
  #subst on sql in DELETE ARRAYS
  my $check = $self->preDELETE($form);
  my $post = $self->postDELETE($form);
  if (! defined $check) { $self->error("Error in delete when calling preDELETE"); return undef; }
  if (! defined $post) { $self->error("Error in delete when calling postDELETE"); return undef; }
  if ($check ne "") {  return (-1, $check);  }

  my @sqls = @{$self->{DELETE}};
  for (my $index = 0; $index < @sqls; $index++)
  {
    my $sql = $sqls[$index];
    while ($sql =~ /#FIELD=([^#]+)#/)
    {
      my $found = $1;
      my $value = $self->{data}->{$form}->{$found}->{-Value};
      $sql =~ s/#FIELD=$found#/$value/g;
    }
    my $result = $self->{dbHandle}->write(sql => $sql);
    if ($self->{dbHandle}->error)
    {
      $self->error("<br>".$self->{dbHandle}->errorMessage."<br>\n\n$sql");
      $self->{dbHandle}->rollback;
      return undef;
    }
    $r += $result;
    if ($self->debug) { $self->debug("<br>rs:$result:$r<br>$sql<br>\n"); }
  }
  if ($r > 0)  {  $self->{dbHandle}->commit;  }
  else  {  $self->{dbHandle}->rollback;  }
  return ($r, $post);
}

#returns undef on error
# or empty if nothing found
# or record found
sub preDELETE
{
  my $self = shift;
  my $form = shift;
  my $return = "";
  if (! defined $form) { $form = "main"; }
  if (! defined $self->{preDELETE} || ! defined $self->{preDELETE}[0] || $self->{preDELETE}[0] eq "") { return "";  }
  #subst on sql in preDELETE ARRAYS
  my @sqls = @{$self->{preDELETE}};
  for (my $index = 0; $index < @sqls; $index++)
  {
    my $sql = $sqls[$index];
    if (!defined $sql || $sql eq "") { next; }
    #subst on sql
    while ($sql =~ /#FIELD=([^#]+)#/)
    {
      my $found = $1;
      my $value = $self->{data}->{$form}->{$found}->{-Value};
      $sql =~ s/#FIELD=$found#/$value/g;
    }
    #run the check code
    if ($self->debug) { $self->debug("<br>".$sql."<br>");  }
    my $result = $self->{dbHandle}->read(sql => $sql);
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{dbHandle}->errorMessage."<br>\n\n$sql<br>");
      return undef;
    }
    #if any rows are returned then return it
    while (my @data = $result->fetchrow_array)
    {
      $return .= $data[0]."\n";
    }
  }
  return $return;
}

#retuns undef on error
# or value from sql statement for post delete
sub postDELETE
{
  my $self = shift;
  my $form = shift;
  my $return = "";
  if (! defined $form) { $form = "main"; }
  if (! defined $self->{postDELETE} || ! defined $self->{postDELETE}[0] || $self->{postDELETE}[0] eq "") { return "";  }
  #subst on sql in postDELETE ARRAYS
  my @sqls = @{$self->{postDELETE}};
  for (my $index = 0; $index < @sqls; $index++)
  {
    my $sql = $sqls[$index];
    if (!defined $sql || $sql eq "") { next; }
    #subst on sql
    while ($sql =~ /#FIELD=([^#]+)#/)
    {
      my $found = $1;
      my $value = $self->{data}->{$form}->{$found}->{-Value};
      $sql =~ s/#FIELD=$found#/$value/g;
    }
    #run the check code
    if ($self->debug) { $self->debug("<br".$sql."<br>");  }
    my $result = $self->{dbHandle}->read(sql => $sql);
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{dbHandle}->errorMessage."<br>\n\n$sql<br>");
      return undef;
    }
    #if any rows are returned then return it
    while (my @data = $result->fetchrow_array)
    {
      $return .= $data[0]."\n";
    }
  }
  return $return;
}

=item string db2data("var") or db2data(var =>, index =>)

summary:  Routine looks in db2data hash for the given database column name.  If found
  it returns the corresponding data element name.  If not found the data mapping is
  assumed to be the same as the given name and that is returned.
requires:  if only 1 argument it is the variable to map.  Will assume the index to be 0
optional:  index - corresponds to the arrays in the internal data structures
returns:  The corresponding data element name for the given database column name.

=cut
sub db2data
{
  my $self = shift;
  my $map;
  my $index = 0;
  if (@_ == 1)  {  $map = shift;  }
  else
  {
    my %args = (index => $index, @_);
    $map = $args{var};
    $index = $args{index};
  }
  if (defined $self->{db2data}[$index]->{$map}) { return $self->{db2data}[$index]->{$map};  }
  return $map;
}

=item string data2db("var") or data2db(var =>, index =>)

summary:  Routine looks in data2db hash for the given data element name.  If found
  it returns the corresponding database column name.  If not found the data mapping
  is assumed to be the same as the given name and that is returned.
requires:  if only 1 argument it is the variable to map.  Will assume the index to be 0
optional:  index - corresponds to the arrays in the internal data structures
returns:  The corresponding database column name for the given data element.

=cut
sub data2db
{
  my $self = shift;
  my $map;
  my $index = 0;
  if (@_ == 1)  {  $map = shift;  }
  else
  {
    my %args = (index => $index, @_);
    $map = $args{var};
    $index = $args{index};
  }
  if (defined $self->{data2db}[$index]->{$map}) { return $self->{data2db}[$index]->{$map};  }
  return $map;
}

=item bool populate()

   Populates the form data structures with given data.

   Returns the true/false result from isValid().

 requires:

 returns: true/false indicating isValid() result.

=cut
sub populate
{
  my $self = shift;
  my %args = ( @_ );

  foreach my $form (keys %{$self->{data}})
  {
    # assign to $self and update the data profiles.
    foreach my $name (keys %{$self->{data}->{$form}})
    {
      if ($name eq "app" || $name eq "state" || $name eq "command")  {  next;  }
      if ($self->{data}->{$form}->{$name}->{-Type} eq "checkbox")
      {
        if (exists $args{$name})
        {
          $self->{data}->{$form}->{$name}->{checked} = 1;
        }
        else
        {
          $self->{data}->{$form}->{$name}->{checked} = 0;
        }
      }
      elsif (exists $args{$name})
      {
       if ($self->debug) {  $self->debug("Setting: $form -> $name -> $args{$name}<br>\n");  }
        $self->{data}->{$form}->{$name}->{-Value} = $args{$name};
      }
    }
  }
  # do validation
  return $self->isValid;
}

=item boolean debug("string") or boolean debug()

  similar to the error routine this routine will set a debug
  statement and turn on debug if called.  access throught
  $obj->debugString

=cut
sub debug
{
  my $self = shift;
  my @callerArgs = caller(1);
  (my $subName = $callerArgs[3]) =~ s/^(.+)(::)([^:]+)$/$1->$3/;
  my $callerErrStr = "$subName";

  if (scalar @_ > 0)
  {
    # we are setting an error condition.
    if (scalar @_ == 1)
    {
      $self->{debugString} .= $callerErrStr . @_[0]."<br>";
    }
    else
    {
      if (@_[0] eq "debugString")
      {
        my %args = ( @_ );
        if (!exists $args{debugString})  # make sure we get the debugString argument!
        {
          $self->debug($callerErrStr . "<b>debugString</b> is missing!<br>\n");
          return;
        }
        else
        {
          $self->{debugString} .= $callerErrStr . $args{debugString};
        }
      }
      else
      {
        # handle the sprintf case.
        if (@_[0] =~ /(?<!\\)%[^%]/)
        {
          # build up the string to eval for the sprintf.
          my $str = "\"@_[0]\"";
          for (my $i=1; $i < scalar @_; $i++)
          {
            $str .= ", \"@_[$i]\"";
          }
          $self->{debugString} .= $callerErrStr;
          eval "\$self->{debugString} .= sprintf($str);";
          if ($@)
          {
            $self->debug($callerErrStr . $@);
            return;
          }
        }
        else
        {
          $self->{debugString} .= $callerErrStr . join(" ", @_);
        }
      }
    }
    $self->{debugString} .= "\n";
    $self->{debug} = 1;
  }

  return $self->{debug};
}

sub resetDebug
{
  my $self = shift;
  $self->{debugString} = "";
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Base(3)

=cut
