# ChartOfAccounts.pm - The Object Class that provides a ChartOfAccounts Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: ChartOfAccounts.pm,v 1.2 2005/01/06 16:41:38 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::ChartOfAccounts;
use strict;
use Portal::Accounting::Objects::Base;
use Date::Calc qw(Add_Delta_Days Today);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.2 $';

=head1 NAME

ChartOfAccounts - Object used to build a Chart Of Accounts.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::ChartOfAccounts;
  my $ChartOfAccountsObj = Portal::Accounting::Objects::ChartOfAccounts->new();
  if ($ChartOfAccountsObj->didErrorOccur())
  {
    die $ChartOfAccountsObj->errorMessage();
  }

=head1 DESCRIPTION

ChartOfAccounts is an object which holds, manages, displays, updates, edits account information and assignments.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(langObj => )

    Creates a new instance of Portal::Accounting::Objects::ChartOfAccounts.
    See Portal::Base(3) for a listing of required arguments.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( dbHandle => undef, ledgerCode => "", app => "Accounting", state => "Accounts", @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  $self->{ledgerCode} = $args{ledgerCode};
  $self->{dbHandle} = $args{dbHandle};
  $self->{app} = $args{app};
  $self->{state} = $args{state};
  # instantiate anything unique to this module
  my $sql = "SELECT name, number AS code, number, type, active, taxable";
  $sql .= ", (SELECT first_digit FROM acct_type_tb WHERE num = type) AS first_digit";
  $sql .= " FROM account_tb ORDER BY first_digit, type, name";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  while (my $data = $result->fetchrow_hashref)
  {
    my $assigned = "SELECT ledger_code FROM local_accounts_tb WHERE acct_num = '$data->{number}'";
    my $adb = $self->{dbHandle}->read($assigned);
    if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
    $data->{assigned} = [];
    if (my $a = $adb->fetchall_hashref("ledger_code"))
    {
      $data->{assigned} = $a;
    }
    push (@{$self->{data}}, $data);
  }
  my $sql = "SELECT name, num FROM acct_type_tb";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  while (my $data = $result->fetchrow_hashref)
  {
    $self->{types}->{$data->{num}} = $self->langObj->map($data->{name});
  }
  # do anything else you might need to do.
  $self->isValid;
  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (! defined $self->{dbHandle})
  {
    $self->missing("dbHandle");
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

#take the approach that this chart of accounts is mainly for assignement and to link to editing of accounts or new.
#if i do this on the client side i can simply
#
#1 send down raw data
#2 have client display data with markup etc.
#3 client can enable, disable assignment button as user changes ledger
sub printHTML
{
  my $self = shift;
  my $l = $self->langObj;
  my $click4new = $l->map("Click to Create New Account");
  my $type = "";
  my $Name = $l->map("Account");
  my $Code = $l->map("Code");
  my $Active = $l->map("Active");
  my $Taxable = $l->map("Taxable");
  my $Assigned = $l->map("Assigned");

  my $return = <<"ENDOF";
<style>
  .chartOfAccounts { width: 70%; }
  .heading  { text-align: left; padding: 5; }
  .name     { text-align: left; }
  .code     { text-align: right; }
  .active   { text-align: center; }
  .taxable  { text-align: center; }
  .assigned { text-align: center; }
</style>
ENDOF
  $return .= "<table class=\"edit\">\n";
  $return .= <<"ENDOF";
 <tr>
  <th class="name">$Name</th>
  <th class="code">$Code</th>
  <th class="active">$Active</th>
  <th class="taxable">$Taxable</th>
  <th class="assigned">$Assigned</th>
 </tr>
ENDOF
  foreach my $d (@{$self->{data}})
  {
    if ($type ne $d->{type})
    {
      $return .= <<"ENDOF";
 <tr class="heading">
  <td class="name" colspan="4"><br><a title="$click4new" href="$self->{baseURL}?app=$self->{app}&state=$self->{state}&command=new&type=$d->{type}">$self->{types}->{$d->{type}}</a></td>
  <td class="assigned">&nbsp;</td>
 </tr>
ENDOF
    }
    $type = $d->{type};
    $return .= " <tr>\n";
    $return .= "  <td class=\"name\">$d->{name}</td>\n";
    $return .= "  <td class=\"code\">$d->{code}</td>\n";
    if ($d->{active} == 0) { $return .= "  <td class=\"active\">[ &nbsp;]</td>\n" }
    else { $return .= "  <td class=\"active\">[X]</td>\n" }
    if ($d->{taxable} == 0) { $return .= "  <td class=\"taxable\">[ &nbsp;]</td>\n" }
    else { $return .= "  <td class=\"taxable\">[X]</td>\n" }
    #figure assigned
    if ($self->{ledgerCode} ne "")
    {
      if (exists $d->{assigned}->{$self->{ledgerCode}})
      {  #check the assigned button on
        $return .= "  <td class=\"assigned\"><input type=\"checkbox\" name=\"assign_$d->{number}\" checked=\"1\" /></td>\n";
      }
      else
      {  #check the assigned button off
        $return .= "  <td class=\"assigned\"><input type=\"checkbox\" name=\"assign_$d->{number}\" /></td>\n";
      }
    }
    else
    {
      #disalbe assign button
      $return .= "  <td class=\"assigned\"><input type=\"checkbox\" name=\"assign_$d->{number}\" disabled=\"1\" /></td>\n";
    }
    $return .= " </tr>\n";
  }
  $return .= "</table>\n";
  $return .= <<"ENDO";
  <p><form name="new">
  <input type="hidden" name="app" value="$self->{app}">
  <input type="hidden" name="state" value="$self->{state}">
  <input type="hidden" name="command" value="new">
  <radio name="asset" label="Asset">
  <radio name="liability" label="Liability">
  <input type="submit" name="submit" value="Create New Account">
ENDO
  return $return;
}

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

sub printXML
{
  my $self = shift;
  my $result = "";
  my %args = ( fullXML => 0, indent => 2, @_ );
  my $fullXML = $args{fullXML};
  my $indent = ($fullXML ? 0 : $args{indent});  # start at 0 indentation if we are making a full XML document.
  my $spaceStr = " " x $indent;

  if ($self->isValid())
  {
    if ($fullXML)
    {
      $result .= "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>";
    }
  }
  else
  {
    $result .= "Data not valid for XML!\n\n";
    $self->error($result);
  }
  return $result;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Base(3)

=cut
