# Register.pm - The Object Class that provides a Register Object
#$Id: Register.pm,v 1.11 2005/01/26 22:35:13 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Register;
use strict;
use Portal::Accounting::Objects::Base;
use Date::Calc qw (Today Today_and_Now Add_Delta_YMD);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

my $Revision = '$Revision: 1.11 $';
($VERSION = $Revision) =~ s/^Revision:\s//;

=head1 NAME

Register - Object used to build a Register Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Register;
  my $RegisterObj = Portal::Accounting::Objects::Register->new(   );
  if (! $RegisterObj->populate(%{$self->{input}}) || ! $RegisterObj->validate())
  {
    print $RegisterObj->printForm();
  }
  else
  {
    if ($RegisterObj->select)
    {
      print $RegisterObj->printHTML;
    }
    else
    {
      print $RegisterObj->errorMessage;
    }
  }

=head1 DESCRIPTION

Register is a Register class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::Register.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  my %args = (ledgerCode => "", id => "", @_ );
  my $l = $self->{langObj};
  my $DB = Portal::Accounting::DB::Ledger->new(langObj => $l, dbHandle => $self->{dbHandle});

  if ($args{ledgerCode} eq "") { $self->error("LedgerCode must be specified<br>\n"); return undef;  }
  if ($args{id} eq "") { $self->error("User id must be specified<br>\n"); return undef;  }
  $self->{vars} = { foo => "string" };
  $self->{detail} = [];

  my @date = Today();
  if ($date[1] == 1)
  {
    $date[1] = 12;
    $date[0]--;
  }
  else
  {  $date[1]--;  }
  $date[2] = "01";
  my $from = "$date[0]-$date[1]-$date[2]";
  @date = Add_Delta_YMD(@date, 0, 1, -1);
  my $to = "$date[0]-$date[1]-$date[2]";

  my $foo = $DB->getAccountsAsSth($args{ledgerCode});
  if ($DB->error) { $self->error("Failed to get accounts<br>".$DB->errorMessage."\n"); return undef;  }
  my $c = $self->getCommonAccounts($args{ledgerCode});
  if (! defined $c) { $c = {names => [], values => [] }; }
  my $a = $self->{formObj}->createSelectOptions($foo);
  if (! defined $a) { $a = {names => [], values => [] }; }
  $self->{data}->{main} = {
    "account" => { -Type => "hidden", -Value => ""},
    "common" => { -Label => $l->map("Most Used Accounts"),
      -Type => "select", size => 4,
      onchange => "this.form.accounts.selectedIndex = -1; this.form.account.value = this.options[this.selectedIndex].value; setDate(this.form);",
      -Options => $c
      },
    "accounts" => { -Label => $l->map("All Accounts"),
      -Type => "select",
      onchange => "this.form.common.selectedIndex = -1; this.form.account.value = this.options[this.selectedIndex].value;",
      -onload => "this.selectedIndex = -1;",
      -Options => $a
      },
    "from" => { -Label => $l->map("From"),
      -Type => "date-picker", year => $date[0],
      -Value => $from
      },
    "to" => { -Label => $l->map("To"),
      -Type => "date-picker", year => $date[0],
      -Value => $to
      },
    "reconcile" => { -Label => $l->map("Reconcile"), -Type => "checkbox", -Value => "1", checked => 1 },
    "command" => { -Type => "hidden", -Value => "displayRegister" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode} },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Register" },
  };

  $self->{javascript}->{main} = <<"ENDOF";
  var from = new Array();
  var to = new Array();

  function setDate(myForm)
  {
    var account = myForm.account.value;
    myForm.from.value = from[account];
    myForm.to.value = to[account];
  }

ENDOF
  my $submit = $l->map("Submit");
  $self->{template}->{main} = <<"ENDOF";
  <form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  #FIELD=account#
  <table border="0" cellspacing="0" cellpadding="4">
    <tbody>
    <tr>
      <td>
        #LABEL=common#<br>
        #FIELD=common#
      </td>
      <td align="right">
        #LABEL=from#: #FIELD=from#<br>
        #LABEL=to#: #FIELD=to#
      </td>
    </tr>
    <tr>
      <td>
        #LABEL=accounts#<br>
        #FIELD=accounts#
        </td>
        <td align="right">
          #FIELD=reconcile# #LABEL=reconcile#
      </td>
    </tr>
    <tr>
      <td align="right" colspan="2">
        <input type="submit" name="submit" value="$submit">
      </td>
    </tr>
    </tbody>
  </table>
  </form>
ENDOF

  $self->{profile}->{main} = {
    required => [ qw( account from to ) ],
    constraints  =>
        {
            from   => qr/^\d{4}-\d{2}-\d{2}$/,
            to   => qr/^\d{4}-\d{2}-\d{2}$/,
            account  => qr/^\d+$/,
        },
  };

  $self->{data}->{reconcile} = {
    "account" => { -Type => "hidden", -Value => ""},
    "rows" => { -Type => "hidden", -Value => ""},
    "from" => { -Label => $l->map("From"),
      -Type => "date-picker", year => $date[0],
      -Value => $from
      },
    "to" => { -Label => $l->map("To"),
      -Type => "date-picker", year => $date[0],
      -Value => $to
      },
    "reconcile" => { -Label => $l->map("Reconcile"), -Type => "checkbox", -Value => "1", checked => 1 },
    "command" => { -Type => "hidden", -Value => "saveReconcile" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode} },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Register" },
  };
  $self->{db2data}[0] = { date_of => "to" };
  $self->{data2db}[0] = reverse %{$self->{db2data}[0]};
  $self->{INSERTorder}[0] = [ qw( date_of ) ];
  $self->{UPDATE}[0] = "UPDATE debitcredit_tb SET cleared = 'r' WHERE cleared = 'x' AND trans_date <= ?";
  $self->{INSERTorder}[1] = [ qw( date_of account ledger_code ) ];
  $self->{db2data}[1] = { date_of => "to", ledger_code => "ledgerCode" };
  $self->{data2db}[1] = reverse %{$self->{db2data}[1]};
  $self->{UPDATE}[1] = "INSERT INTO reconcile_tb (id, ##COLUMNS##)";
  $self->{UPDATE}[1] .= " VALUES ('$args{id} '||NOW(), ##VALUES## )";
  return $self;
}

=item bool populate(%input)

  Checks for existence of the {rows} var.  If exists it dynamically creates
  a data->{reconcile} element for each row. so that when
  the parent populate method is called it will pull the data from
  the input hash

  If no rows exists then the form is main instead of reconcile and nothing extra is done.

=cut
sub populate
{
  my $self = shift;
  if (defined $self->{input}->{rows} && $self->{input}->{rows} =~ /^\d+$/)
  {
    for (my $n = 0; $n < $self->{input}->{rows}; $n++)
    {
      $self->{data}->{reconcile}->{ref_$n} = { type => "hidden", -Value => "" };
      $self->{data}->{reconcile}->{counter_$n} = { type => "hidden", -Value => "" };
      $self->{data}->{reconcile}->{cleared_$n} = { type => "button", -Value => "" };
    }
  }
  return $self->SUPER::populate(@_);
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
#   if ( $self->{data}->{main}->{ledgerCode}->{-Value} eq "")
#   {
#     $self->invalid("ledgerCode");
#   }
#   if ( $self->{data}->{main}->{accountName}{-Value} eq "")
#   {
#     $self->invalid("accountName");
#   }
#   if ( $self->{data}->{main}->{accountNumber}{-Value} =~ /^\d+$/)
#   {
#     $self->invalid("accountNumber");
#   }
#   if ( $self->{data}->{main}->{type}{-Value} =~ /^\d+$/)
#   {
#     $self->invalid("type");
#   }
#   if ( $self->{data}->{main}->{balance}{-Value} =~ /^\d(\.)?\d*$/)
#   {
#     $self->invalid("balance");
#   }
#   if ( $self->{data}->{main}->{from}{-Value} =~ /^(\d{4})-(\d{1,2})-(\d{1,2})$/)
#   {
#     $self->{data}->{main}->{from}{-Value} = $1 ."-". sprintf("%02d", $2) . "-" . sprintf("%02d", $3)
#   }
#   else
#   {
#     $self->invalid("from");
#   }
#   if ( $self->{data}->{main}->{to}{-Value} =~ /^(\d{4})-(\d{1,2})-(\d{1,2})$/)
#   {
#     $self->{data}->{main}->{to}{-Value} = $1 ."-". sprintf("%02d", $2) . "-" . sprintf("%02d", $3)
#   }
#   else
#   {
#     $self->invalid("to");
#   }
#   if (! defined $self->{detail})
#   {
#     $self->invalid("detail");
#   }

#   if ($self->numInvalid() > 0 || $self->numMissing() > 0)
#   {
#     $self->postfixError($self->genErrorString("all"));
#     return 0;
#   }

  return 1;
}

sub outstanding
{
  my $self = shift;
  my $total = 0;
  for (my $row = 0; $row < scalar @{$self->{detail}}; $row++)
  {
    $self->{cashFlow} += $self->{detail}->[$row]->[10];
    if ($self->{detail}->[$row]->[2] ne "")  {  $total += $self->{detail}->[$row]->[10];  }
  }
  return $total;
}

=item print

  This routine looks at the internal reconcile flag and calls the appropriate routine.

=cut
sub printHTML
{
  my $self = shift;
  if ($self->{data}->{main}->{reconcile}->{checked})
  {  return $self->printReconcile(@_);  }
  return $self->printPlain(@_);
}

=item string columnMarkup(text, row, column)

  This routine adds html that is appropriate to for each column in the register.
  Column is a integer starting from 0, text is the text to markup.
  The columns are based on the type of register
  Bank Acct: "ref_num", "counter", "$Outstanding", "$Date", "$Description", "$Comment", "$Chk", "$Dep", "$SubAccount", "$Cld", "$Amount"
  Invest Acct: "ref_num", "counter", "$Outstanding", "$Date", "$Description", "$Comment", "", "$SubAccountName", "$SubAccount", "$Cld", "$Amount"
  All others: "ref_num", "counter", "$Outstanding", "$Date", "$Description", "$Comment", "", "$SubAccountName", "$SubAccount", "$Cld", "$Amount"

=cut
sub columnMarkup
{
  my $self = shift;
  my $text = shift;
  my $row = shift;
  my $col = shift;
  my $return = "";
  if ($text eq "") { $text = "&nbsp;"; }
  if ($col == 9 && $self->{data}->{main}->{reconcile}->{checked})
  {
    #do button values get submitted?  is that why we have a hidden field?
    $return .= <<"ENDOF";
<input class="button" type="button" value="$text" name="button_$row" onclick="cyclebutton(this.form.cleared_$row, this); calculateBalance(this.form);">
<input type="hidden" name="cleared_$row" value="$text">
ENDOF
  }
  elsif ($col == 0 && $self->{data}->{main}->{reconcile}->{checked})
  {
    $return .= <<"ENDOF";
<input type="hidden" value="$text" name="ref_$row">$text
ENDOF
  }
  elsif ($col == 1 && $self->{data}->{main}->{reconcile}->{checked})
  {
    $return .= <<"ENDOF";
<input type="hidden" value="$text" name="counter_$row">$text
ENDOF
  }
  elsif ($col == 10)
  {
    $return .= "<input type=\"hidden\" name=\"amount_$row\" value=\"$text\">";
    if ($text < 0)
    {
      $return .= "<font class=\"negative\">$text</font>";
    }
    else
    {
      $return .= $text;
    }
  }
  elsif ($col == 11)
  {
    if ($text < 0)
    {
      $return .= "<font class=\"negative\">$text</font>";
    }
    else
    {
      $return .= $text;
    }
  }
  elsif ($col == 6) #check
  {
    my $value = $text;
    if ($text eq "&nbsp;") {  $value = "*"; }
    else
    {
      $return .= "<a style=\"text-decoration: none; color: #000000;\" href=\"?app=Accounting&state=JournalEntry&command=edit&ledgerCode=";
      $return .= $self->{ledgerCode}."&refNum=".$self->{detail}->[$row]->[0];
      $return .= "\" target=\"_new\">$text</a>";
    }
    $return .= "<input type=\"hidden\" name=\"check_$row\" value=\"$value\">&nbsp;";
  }
  elsif ($col == 7) #dep
  {
    my $value = $text;
    if ($text eq "&nbsp;") {  $value = "*"; }
    else
    {
      $return .= "<a style=\"text-decoration: none; color: #000000;\" href=\"?app=Accounting&state=JournalEntry&command=edit&ledgerCode=";
      $return .= $self->{ledgerCode}."&refNum=".$self->{detail}->[$row]->[0];
      $return .= "\" target=\"_new\">$text</a>";
    }
    $return .= "<input type=\"hidden\" name=\"deposit_$row\" value=\"$value\">&nbsp;";
  }
  elsif ($text ne "")
  {
    #/!!!! need to properly open a window.  use js code to do it?  does portal provide yet?
    $return .= "<a style=\"text-decoration: none; color: #000000;\" href=\"?app=Accounting&state=JournalEntry&command=edit&ledgerCode=";
    $return .= $self->{ledgerCode}."&refNum=".$self->{detail}->[$row]->[0];
    $return .= "\" target=\"_new\">$text</a>";
  }

  return $return;
}

sub setupStyles
{
  my $self = shift;
  my $l = $self->{langObj};

  my $Date = $l->map("Date");
  my $Comment = $l->map("Comment");
  my $Outstanding = $l->map("Outstanding");
  my $SubAccount = $l->map("Sub-Accounts");
  my $Debit = $l->map("Debit");
  my $Amount = $l->map("Amount");
  my $Credit = $l->map("Credit");
  my $Cleared = $l->map("Cleared");
  my $Cld = $l->map("Cld");
  my $Balance = $l->map("Balance");
  my $Description = $l->map("Description");
  my $Chk = $l->map("Chk");
  my $Dep = $l->map("Dep");
  my %return;
  if ($self->{type} eq "")
  {
    $return{css} .= "<!-- Error setting up styles.  No type given to setupStyles. -->\n";
    $return{headers} .= "<!-- Error setting up headings.  No type given. -->\n";
  }
  elsif ($self->{type} == 0)
  {
    $return{headers} .= <<"ENDOF";
      <th class="active-column-0">ref#</th>
      <th class="active-column-1">#</th>
      <th class="active-column-2"><a title="$Outstanding">O</a></th>
      <th class="active-column-3">$Date</th>
      <th class="active-column-4">$Description</th>
      <th class="active-column-5">$Comment</th>
      <th class="active-column-6">$Chk </th>
      <th class="active-column-7">$Dep </th>
      <th class="active-column-8">$SubAccount</th>
      <th class="active-column-9">$Cld</th>
      <th class="active-column-10">$Amount</th>
      <th class="active-column-11">$Balance</th>
ENDOF
    $return{css} .= <<"ENDOF";
  .active-column-6 {text-align: right;} /* chk */
  .active-column-7 {text-align: right;} /* dep */
  .active-column-8 {display: none;}    /* sub account */
ENDOF
  }
  elsif ($self->{type} == 3) #investment accounts
  {
    $return{headers} .= <<"ENDOF";
      <th class="active-column-0">ref#</th>
      <th class="active-column-1">#</th>
      <th class="active-column-2"><a title="$Outstanding">O</a></th>
      <th class="active-column-3">$Date</th>
      <th class="active-column-4">$Description</th>
      <th class="active-column-5">$Comment</th>
      <th class="active-column-6">&nbsp;</th>
      <th class="active-column-7">&nbsp;</th>
      <th class="active-column-8">$SubAccount</th>
      <th class="active-column-9">$Cld</th>
      <th class="active-column-10">$Amount</th>
      <th class="active-column-11">$Balance</th>
ENDOF
    $return{css} .= <<"ENDOF";
  .active-column-6 {display: none;} /* sub account */
  .active-column-7 {display: none;} /* sub account */
  .active-column-8 {display: none;} /* sub account */
ENDOF
  }
  else #all others
  {
    $return{headers} .= <<"ENDOF";
      <th class="active-column-0">ref#</th>
      <th class="active-column-1">#</th>
      <th class="active-column-2"><a title="$Outstanding">O</a></th>
      <th class="active-column-3">$Date</th>
      <th class="active-column-4">$Description</th>
      <th class="active-column-5">$Comment</th>
      <th class="active-column-6">&nbsp;</th>
      <th class="active-column-7">$SubAccount</th>
      <th class="active-column-8">$SubAccount</th>
      <th class="active-column-9">$Cld</th>
      <th class="active-column-10">$Amount</th>
      <th class="active-column-11">$Balance</th>
ENDOF
    $return{css} = <<"ENDOF";
  .active-column-6 {display: none;}     /* sub account */
  .active-column-7 {text-align: left;}  /* sub account */
  .active-column-8 {text-align: right;} /* sub account */
ENDOF
  }
  my $outstanding = "display: none;";
  my $balance = "display: none;";
  if ($self->{data}->{main}->{reconcile}->{checked})  { $outstanding = "width: 20px;"; }
  else { $balance = "text-align: right;"; }
  $return{css} = <<"ENDOF";
  body, html {margin:0px; padding: 0px; }
  input.button { font-size: 9; border-style:none;}

  .active-controls-grid {font: menu;}
  .active-grid-column {border-right: 1px solid threedlightshadow;}
  .active-grid-row {border-bottom: 1px solid threedlightshadow;}
  .negative { color: #ff0000; }  /* !!!! should be coming from system wide styles */

  .active-column-0 {display: none; text-align: right;}      /* ref */
  .active-column-1 {display: none; text-align: left;}      /* counter */
  .active-column-2 {$outstanding}      /* outstanding */
  .active-column-3 {text-align: left;}   /* date */
  .active-column-4 {text-align: left;}   /* payee */
  .active-column-5 {text-align: left;}   /* comment */
$return{css}
  .active-column-9 {width: 40px; text-align: center;}  /* cleared */
  .active-column-10 {text-align: right;} /* amount */
  .active-column-11 {$balance}  /* balance / / */
ENDOF
  return %return;
}

=item %hash printPlain()

  This is a register view with a running balance.  But no reconciliation forms.

=cut
sub printPlain
{
  my $self = shift;
  my %return;
  my %setup = $self->setupStyles();
  my $balance = $self->{balance};
  $return{css} = $setup{css};
  $return{body} .= <<"ENDOF";
  <center>$self->{accountName} - $self->{accountNumber} $self->{from} - $self->{to}</center><br>
  <table class="register" border="1" width="100%" cellpadding="0" cellspacing="0">
  $setup{headers}
  <tr>
    <td class="active-column-3">$self->{from}</td>
    <td class="active-column-4" colspan="5">Balance Forward</td>
    <td class="active-column-10">$balance</td>
    <td class="active-column-11">$balance</td></tr>
ENDOF
  for (my $row = 0; $row < scalar @{$self->{detail}}; $row++)
  {
    $return{body} .= "  <tr>\n ";
    my $col;
    for ($col = 3; $col < scalar @{$self->{detail}->[$row]}; $col++)
    {
      $return{body} .= "    <td class=\"active-column-$col\">".$self->columnMarkup($self->{detail}->[$row]->[$col], $row, $col)."</td>\n";
    }
    $balance += $self->{detail}->[$row]->[$col - 1];
    $return{body} .= "    <td class=\"active-column-$col\">".$self->columnMarkup(sprintf("%.02f",$balance))."</td>\n";
    $return{body} .= "  </tr>\n ";
  }
  $return{body} .= "</table>\n ";
  return %return;
}

=item %hash printReconcile()

  This register view contains forms for editing line items and marking entries as cleared.

=cut
sub printReconcile
{
  my $self = shift;
  my %args = (app => "Accounting", state => "Register", @_);
  $self->{app} = $args{app};
  $self->{state} = $args{state};
  my %return;
  my $l = $self->{langObj};
  my %setup = $self->setupStyles();
  my $headers;
  my $styles;
  my $rows = scalar @{$self->{detail}};
  my $columns = 0;
  if (defined $self->{detail}->[0])
  {
    $columns = scalar @{$self->{detail}->[0]};
  }
  my $Mark = $l->map("Mark");
  my $from = $l->map("From");
  my $to = $l->map("To");
  my $Dep = $l->map("Dep");
  my $Chk = $l->map("Chk");
  my $usethis = $l->map("usethis");

  $return{css} = $setup{css};
  $return{javascript} = <<"ENDOF";
function checkSubmit(myform)
{
  mark(myform);
  myform.submit();
}

function cyclebutton(button, cleared)
{
  if (button.value == "_")
  {
    button.value = "x";
    cleared.value = "x";
  }
  else if (button.value == "x")
  {
    button.value = "v";
    cleared.value = "v";
  }
  else if (button.value == "v")
  {
    button.value = "^";
    cleared.value = "^";
  }
  else if (button.value == "^")
  {
    button.value = "r";
    cleared.value = "r";
  }
  else
  {
    button.value = "_";
    cleared.value = "_";
  }
}

function mark(myform)
{
  markChecks(myform,'check');
  markChecks(myform,'deposit');
  markButtons(myform);
}

//takes: form, field
// form - form to work with
// field -  name of field to check for check or deposit number
function markChecks(myform, CorD)
{
  var count;
  var from;
  var to;
  if (CorD == "check")
  {
    from = myform.chkFrom.value;
    to = myform.chkTo.value;
    myform.chkFrom.value = '';
    myform.chkTo.value = '';
  }
  else if(CorD == "deposit")
  {
    from = myform.depFrom.value;
    to = myform.depTo.value;
    myform.depFrom.value = '';
    myform.depTo.value = '';
  }
  else
  {
    alert("Invalid field name for markChecks");
    return false;
  }
  if ((from != "" && isNaN(from)) || (to != "" && isNaN(to)) )
  { //either is set but not a number then blow error
    alert("FromToNotAnum");
    return false;
  }
  if (from != "" && to != "")
  {
    if (from > to)
    {
     alert("FromGTTo");
     return false;
    }
   //check each row for being a check by form.check_count is NAN
    for (count = 0; count < myform.rows.value; count++)
    {
      var check;
      eval('check = myform.' + CorD + '_' + count + '.value');
      if (! isNaN(check) )
      {
        var cleared;
        eval('cleared = myform.cleared_' + count);
        var button;
        eval('button = myform.button_' + count);
        if (cleared.value != "r" && check >= from && check <= to)
        {
          cleared.value = "x";
          button.value = "x";
        }
      }
    }
  }
  else if (from != "")
  {
    for (count = 0; count < myform.rows.value; count++)
    {
      var check;
      eval('check = myform.' + CorD + '_' + count + '.value');
      if (! isNaN(check) )
      {
        var cleared;
        eval('cleared = myform.cleared_' + count);
        var button;
        eval('button = myform.button_' + count);
        if (cleared.value != "r" && check == from)
        {
          cleared.value = "x";
          button.value = "x";
        }
      }
    }
  }
  else if (to != "")
  {
    for (count = 0; count < myform.rows.value; count++)
    {
      var check;
      eval('check = myform.' + CorD + '_' + count + '.value');
      if (! isNaN(check) )
      {
        var cleared;
        eval('cleared = myform.cleared_' + count);
        var button;
        eval('button = myform.button_' + count);
        if (cleared.value != "r" && check == to)
        {
          cleared.value = "x";
          button.value = "x";
        }
      }
    }
  }
  return true;
}

function markButtons(myform)
{
  var count;
  var markEntryX = 0;
  for (count = 0; count < myform.rows.value; count++)
  {
    eval('if (myform.cleared_' + count + '.value == "^") { markEntryX = 0; myform.cleared_' + count + '.value = "x"; myform.button_' + count + '.value = "x"; }' );
    eval('if (myform.cleared_' + count + '.value == "v") { markEntryX = 1; }');
    if (markEntryX == 1)
    {
      eval('myform.cleared_' + count + '.value = "x"; myform.button_' + count + '.value = "x";');
    }
  }
  calculateBalance(myform);
}

function calculateDiff(myform)
{
  if (myform.manualEndStatement.value == "")  { return true; }
  if (isNaN(myform.manualEndStatement.value))  { myform.manualEndStatement.value = ""; }
  else
  {
    var diff;
    diff = parseFloat(myform.manualEndStatement.value) - parseFloat(myform.endStatement.value);
    floatify(diff, 2);
    myform.difference.value = diff;
  }
  return true;
}

function calculateBalance(myform)
{
  //start with original balance
  //walk each row
  //check for x or r
  // if so, add to balance
  var balance = parseFloat(myform.beginStatement.value);
  var count;
  for (count = 0; count < myform.rows.value; count++)
  {
    var cleared;
    var amount = 0.0;
    eval('cleared = myform.cleared_' + count);
    if (cleared.value == "x" || cleared.value == "r")
    {
      eval('amount = myform.amount_' + count + '.value');
      balance += parseFloat(amount);
    }
  }
  myform.endStatement.value = balance;
  calculateDiff(myform);
}
ENDOF
  my $balance = $self->{balance} + $self->{outstanding};
  my $book = $self->{balance} + $self->{cashFlow};
  $return{body} .= <<"ENDOF";
<center>$self->{accountName} - $self->{accountNumber} $self->{from} - $self->{to}</center><br>
<form name="mainForm" method="post">
  <input type="hidden" name="app" value="$self->{app}">
  <input type="hidden" name="state" value="$self->{state}">
  <input type="hidden" name="command" value="reconcile">
  <table border="0" width="100%" cellpadding="0" cellspacing="0">
    <tr>
      <td style="text-align: left;">
      <input class="active-column-10" type="text" size="9" name="book" onfocus="this.form.chkFrom.focus();" style="border: medium none ;" tabIndex="-1" value="$book"> $self->{to} Book Balance<br>
      <input class="active-column-10" type="text" size="9" name="beginStatement" onfocus="this.form.chkFrom.focus();" style="border: medium none ;" value="$balance" tabIndex="-1"> <a title="If this does not match your statement correct it first with a non reconcile view.">Beginning Statement Balance</a><br>
      <input class="active-column-10" type="text" size="9" name="endStatement" onfocus="this.form.chkFrom.focus();" style="border: medium none ;"> <a title="When finished marking entries this should match your statement.">Ending Statement Balance</a><br>
      <input class="active-column-10" type="text" size="9" name="difference" onfocus="this.form.chkFrom.focus();" style="border: medium none ;" tabIndex="-1"> Difference between statement and calculations<br>
      <input class="active-column-10" type="text" size="9" name="manualEndStatement" onchange="calculateDiff(this.form); return true;"> Enter the ending balance from your statement
      </td>
      <td style="text-align: right;">
      <table border="0" align="right">
        <tr><th><font style="color: blue; font-size: large;"><input type="button" name="checks" value="$Mark" onclick="mark(this.form);" title="$usethis"></font></th><th>$from</th><th>$to</th></tr>
        <tr><th>$Chk</th><td><input type="text" size="5" name="chkFrom"></td> <td><input type="text" size="5" name="chkTo"></td></tr>
        <tr><th>$Dep</th><td><input type="text" size="5" name="depFrom"></td> <td><input type="text" size="5" name="depTo"></td></tr>
      </table>
      </td>
    </tr>
  </table>

  <input type="hidden" name="reconcile" value="1">
  <input type="hidden" name="rows" value="$rows">
  <input type="hidden" name="ledgerCode" value="$self->{ledgerCode}">
  <input type="hidden" name="accountName" value="$self->{accountName}">
  <input type="hidden" name="account" value="$self->{accountNumber}">
  <input type="hidden" name="accountType" value="$self->{type}">
  <input type="hidden" name="from" value="$self->{from}">
  <input type="hidden" name="to" value="$self->{to}">

  <table border="0" width="100%" cellspacing="0" cellpadding="0" bgcolor="white">
    <tbody>
    <tr>
    <td align="left"><input type="submit" name="Submit" value="Finished" onclick="checkSubmit(this.form);"></td>
    <td style="text-align: center;"><input type="submit" name="save" value="Save" onclick="this.form.command.value = 'saveReconcile'; checkSubmit(this.form);"><br>
    </td>
    <td align="right"><input type="reset" name="reset" value="Reset"></td>
    </tr>
    </tbody>
  </table>
ENDOF
  $return{body} .= <<"ENDOF";
  <table class="register" border="1" width="100%" cellpadding="0" cellspacing="0">
  $setup{headers}
ENDOF
  for (my $row = 0; $row < scalar @{$self->{detail}}; $row++)
  {
    $return{body} .= "  <tr>\n ";
    my $col;
    for ($col = 0; $col < scalar @{$self->{detail}->[$row]}; $col++)
    {
      $return{body} .= "    <td class=\"active-column-$col\">".$self->columnMarkup($self->{detail}->[$row]->[$col], $row, $col)."</td>\n";
    }
    $return{body} .= "  </tr>\n ";
  }
  $return{body} .= "</table>\n ";

  return %return;
}

sub printOutstanding
{
  my $self = shift;
  my $l = $self->{langObj};
  my $debits;
  my $credits;
  my $sub = $l->map("Sub Account");
  my $Date = $l->map("Date");
  my $desc = $l->map("Description");
  my $amount = $l->map("Amount");
  my $Total = $l->map("SubTotal");
  my $Balance = $l->map("SubBalance");
  my $Outstanding = $l->map("Outstanding");
  my $cBalance = 0;
  my $dBalance = 0;

  if ($self->{type} == 0) { $sub = $l->map("Chk");  }
  $debits .= <<"ENDOF";
<table border="1" class="register">
  <tr>
    <th>$Date</th>
    <th>$desc</th>
    <th>$sub</th>
    <th>$amount</th>
  </tr>
ENDOF
  if ($self->{type} == 0) { $sub = $l->map("Dep");  }
  $credits .= <<"ENDOF";
<table border="1" class="register">
  <tr>
    <th>$Date</th>
    <th>$desc</th>
    <th>$sub</th>
    <th>$amount</th>
  </tr>
ENDOF
  for (my $i = 0; $i < scalar @{$self->{detail}}; $i++)
  {
    if ($self->{detail}->[$i]->[2] == "*")
    { #entry is outstanding
      my @detail = @{$self->{detail}->[$i]};
      if ($detail[10] < 0)
      {
        $credits .= <<"ENDOF";
  <tr><td>$detail[3]</td><td>$detail[4]</td><td>$detail[6]</td><td>$detail[10]</td>
ENDOF
        $cBalance += $detail[10];
      }
      else
      {
        $debits .= <<"ENDOF";
  <tr><td>$detail[3]</td><td>$detail[4]</td><td>$detail[7]</td><td>$detail[10]</td>
ENDOF
        $dBalance += $detail[10];
      }
    }
  }
  my $o = sprintf ("%.02d", $dBalance + $cBalance);
  $dBalance = sprintf ("%.02d", $dBalance);
  $debits .= <<"ENDOF";
  <tr><td align="right">$Total</td><td align="right" colspan="3">$dBalance</td></tr>
</table>
ENDOF
  $cBalance = sprintf ("%.02d", $cBalance);
  $credits .= <<"ENDOF";
  <tr><td align="right">$Total</td><td align="right" colspan="3">$cBalance</td></tr>
</table>
<p>Summary<br>
$Balance:$self->{balance}<br>
$Outstanding:$o
</p>
ENDOF
  return $debits.$credits;
}

sub select
{
  my $self = shift;
  my @account;
  my $sql;
  my $accountSql = "SELECT name, type, ";
  $accountSql .= "  (SELECT SUM(d_amount) FROM debitcredit_tb";
  $accountSql .= "    WHERE account_num = '$self->{data}->{main}->{account}->{-Value}'";
  $accountSql .= "    AND trans_date < '$self->{data}->{main}->{from}->{-Value}'";
  $accountSql .= "  ) AS balance";
  $accountSql .= "  FROM account_tb WHERE number = '$self->{data}->{main}->{account}->{-Value}'";
  my $sth = $self->{dbHandle}->read(sql => $accountSql);
  if ($self->{dbHandle}->error)
  {
    $self->error("$sql".$self->{dbHandle}->errorMessage);
    return undef;
  }
  if (my @a = $sth->fetchrow_array)
  {
    $self->{accountName} = $a[0];
    $self->{Type} = $a[1];
    $self->{accountBalance} = $a[2];
  }
  else
  {
    $self->error("No account found for '$self->{data}->{main}->{account}->{-Value}'");
    return undef;
  }

  if ($self->{Type} == 0)  #if the type is a bank account do not select sub account names
  {
    $sql  = "SELECT parent_num, counter, '' AS outstanding, trans_date ";
    $sql .= ", (SELECT payee_payer FROM journal_entry_tb WHERE ref_num = parent_num) AS payee";
    $sql .= ", SUBSTRING(comment, 1, 15)";
    $sql .= ", checknum(dorc, sub_account_num), depositnum(dorc, sub_account_num), sub_account_num, cleared, d_amount";
    $sql .= " FROM debitcredit_tb";
    $sql .= " WHERE ledger_code = '$self->{data}->{main}->{ledgerCode}->{-Value}' AND account_num = '$self->{data}->{main}->{account}->{-Value}'";
    $sql .= " AND trans_date >= '$self->{data}->{main}->{from}->{-Value}' AND trans_date <= '$self->{data}->{main}->{to}->{-Value}'";
    if ($self->{data}->{main}->{reconcile}->{-Value})
    {
      $sql .= " UNION ";
      $sql .= "SELECT parent_num, counter, '*' AS oustanding, trans_date ";
      $sql .= ", (SELECT payee_payer FROM journal_entry_tb WHERE ref_num = parent_num) AS payee";
      $sql .= ", SUBSTRING(comment, 1, 15)";
      $sql .= ", checknum(dorc, sub_account_num), depositnum(dorc, sub_account_num), sub_account_num, cleared, d_amount";
      $sql .= " FROM debitcredit_tb";
      $sql .= " WHERE ledger_code = '$self->{data}->{main}->{ledgerCode}->{-Value}' AND account_num = '$self->{data}->{main}->{account}->{-Value}'";
      $sql .= " AND trans_date < '$self->{data}->{main}->{from}->{-Value}' AND cleared <> 'r'";
    }
    $sql .= " ORDER BY trans_date, payee";
  }
  elsif ($self->{Type} == 3)  #if the type is a stock account do not select sub accounts.  Only show cash transactions
  {
    $sql  = "SELECT parent_num, counter, '' AS outstanding, trans_date ";
    $sql .= ", (SELECT payee_payer FROM journal_entry_tb WHERE ref_num = parent_num) AS payee";
    $sql .= ", SUBSTRING(comment, 1, 15)";
    $sql .= ", '', '', sub_account_num, cleared, d_amount";
    $sql .= " WHERE ledger_code = '$self->{data}->{main}->{ledgerCode}->{-Value}' AND account_num = '$self->{data}->{main}->{account}->{-Value}'";
    $sql .= " FROM debitcredit_tb";
    $sql .= " AND trans_date >= '$self->{data}->{main}->{from}->{-Value}' AND trans_date <= '$self->{data}->{main}->{to}->{-Value}' AND sub_account_num = '-1'";
    if ($self->{data}->{main}->{reconcile}->{-Value})
    {
      $sql .= " UNION ";
      $sql .= "SELECT parent_num, counter, '*' AS oustanding, trans_date ";
      $sql .= ", (SELECT payee_payer FROM journal_entry_tb WHERE ref_num = parent_num) AS payee";
      $sql .= ", SUBSTRING(comment, 1, 15)";
      $sql .= ", '', '', sub_account_num, cleared, d_amount";
      $sql .= " FROM debitcredit_tb";
      $sql .= " WHERE ledger_code = '$self->{data}->{main}->{ledgerCode}->{-Value}' AND account_num = '$self->{data}->{main}->{account}->{-Value}'";
      $sql .= " AND trans_date < '$self->{data}->{main}->{from}->{-Value}' AND cleared <> 'r' AND sub_account_num = '-1'";
    }
    $sql .= " ORDER BY trans_date, payee";
  }
  else
  {
    $sql  = "SELECT parent_num, counter, '' AS outstanding, trans_date ";
    $sql .= ", (SELECT payee_payer FROM journal_entry_tb WHERE ref_num = parent_num)";
    $sql .= ", SUBSTRING(comment, 1, 15)";
    $sql .= ", '', (SELECT sub_acc_name from sub_acc_tb WHERE sub_acc_number = sub_account_num) AS sub_account, sub_account_num, cleared, d_amount";
    $sql .= " FROM debitcredit_tb";
    $sql .= " WHERE ledger_code = '$self->{data}->{main}->{ledgerCode}->{-Value}' AND account_num = '$self->{data}->{main}->{account}->{-Value}'";
    $sql .= " AND trans_date >= '$self->{data}->{main}->{from}->{-Value}' AND trans_date <= '$self->{data}->{main}->{to}->{-Value}'";
    if ($self->{data}->{main}->{reconcile}->{-Value})
    {
      $sql .= " UNION ";
      $sql .= "SELECT parent_num, counter, '*' AS oustanding, trans_date ";
      $sql .= ", (SELECT payee_payer FROM journal_entry_tb WHERE ref_num = parent_num)";
      $sql .= ", SUBSTRING(comment, 1, 15)";
      $sql .= ", '', (SELECT sub_acc_name from sub_acc_tb WHERE sub_acc_number = sub_account_num) AS sub_account, sub_account_num, cleared, d_amount";
      $sql .= " FROM debitcredit_tb";
      $sql .= " WHERE ledger_code = '$self->{data}->{main}->{ledgerCode}->{-Value}' AND account_num = '$self->{data}->{main}->{account}->{-Value}'";
      $sql .= " AND trans_date < '$self->{data}->{main}->{from}->{-Value}' AND cleared <> 'r'";
    }
    $sql .= " ORDER BY trans_date, sub_account_num";
  }
  my $sthData = $self->{dbHandle}->read(sql => $sql);
  if ($self->{dbHandle}->error)
  {
    $self->error("Error occured calling read<br>$sql".$self->{dbHandle}->errorMessage);
    return undef;
  }
  if (my $data = $sthData->fetchall_arrayref)
  {
   ############!~~~~~~ uncomment to run over data set and output with error
#     $self->debug("Debug on <br>$sql<br>".scalar @$data."<hr>\n");
#     for (my $i = 0; $i < scalar @$data ; $i++)
#     {
#       $self->debug("start:");
#       for (my $j = 0; $j < scalar @{@$data[$i]} ; $j++)
#       {
#         $self->debug("$i,$j:".@$data[$i]->[$j]."\t");
#       }
#       $self->debug("<br>");
#     }
#     return undef;
    ##########################
    $self->{detail} = $data;
  }
  return 1;
}

=item boolean insert

  This routine inserts the internal data reprenting a reconciliation.

=cut
sub insert
{
  my $self = shift;
  $self->error("insert is a stub");
  return undef;
}

=item @ getCommonAccounts(ledgerCode)

  this returns a 2D array with values for
  name, number, from, to

=cut
sub getCommonAccounts
{
  my $self = shift;
  my $l = shift;
  my %data;
  my $sql = <<"SQL";
SELECT account,
  (SELECT name FROM account_tb WHERE number = account) AS name,
  MAX(date_of) + 1 AS f,
  MAX(date_of) + 30 AS t
FROM reconcile_tb
WHERE ledger_code = '$l' AND (SELECT active FROM account_tb WHERE number = account) = true
GROUP BY account ORDER BY f ASC, account
SQL
  if ($self->debug) { $self->debug($sql."<br>");  }
  my $sth = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->didErrorOccur)
  {
    $self->prefixError($self->{dbHandle}->errorMessage);
    return undef;
  }
  my $index = 0;
  while (my $d = $sth->fetchrow_hashref)
  {
    push @{$data{names}}, $d->{name};
    push @{$data{values}}, $d->{number};
    #!!!!!! THESE LINES are NOT showing up in the js part of the doc
    $self->{javascript}->{main} .= "from[$index] = ".$d->{f}.";\n";
    $self->{javascript}->{main} .= "to[".$index++."] = ".$d->{t}.";\n";
  }
  return \%data;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
