# StockObject.pm - The Object Class that provides a Stock Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: StockObject.pm,v 1.9 2005/01/24 22:27:03 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::StockObject;
use strict;
use Portal::Accounting::Objects::SubAccount;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::SubAccount Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.9 $';

=head1 NAME

StockObject - Object used to build a Stock Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::StockObject;
  my $obj = Portal::Accounting::Objects::StockObject->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

StockObject is a Stock class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(ticker, name, number, type, market)

 Creates a new instance of the Portal::Accounting::Objects::StockObject module.
 See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

 requires:
  ticker
  name
  number  - The SubAccount number this stock is associated with
  type
  market

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  my $l = $self->{langObj};
  $self->{markets} = [ qw(None Auto NYSE Nasdaq Amex) ] ;
  my @markets;
  foreach my $m (@{$self->{markets}})
  {
    push @markets, $l->map($m);
  }
  $self->{data}->{main}->{market} = { -Label => $l->map("Market"), -Type => "select", -Options => {values => $self->{markets}, names => \@markets } };
  $self->{data}->{main}->{ticker} = { -Label => $l->map("Ticker"), -Type => "text", size => 7 };
  $self->{data}->{main}->{company} = { -Label => $l->map("Company"), -Type => "text", size => 20 };
  $self->{data}->{main}->{type} = { -Label => $l->map("Type"), -Value => 70, -Type => "hidden" };

  # do validation
#   if (!$self->isValid)
#   {
#     # the error is set in the isValid() method.
#     return $self;
#   }

  # do anything else you might need to do.
  my $Submit = $l->map("Save");
  my $title = $l->map("Public/Private Stocks");
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  <h4>$title</h4>
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=id#
  #FIELD=new#
  <table class="main">
    <tr>
      <td>#LABEL=name#</td><td>#FIELD=name#</td>
    </tr>
    <tr>
      <td>#LABEL=code#</td><td>#FIELD=code#</td>
    </tr>
    <tr>
      <td>#LABEL=type#</td><td>#LABEL=typeName# #FIELD=type#</td>
    </tr>
    <tr>
      <td>#LABEL=company#</td><td>#FIELD=company#</td>
    </tr>
    <tr>
      <td>#LABEL=ticker#</td><td>#FIELD=ticker#</td>
    </tr>
    <tr>
      <td>#LABEL=market#</td><td>#FIELD=market#</td>
    </tr>
  </table>
  <input type="submit" name="Submit" value="$Submit">
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( name code id ) ],
    optional => [  ],
    constraints => {
      id => qr/^\d\d+$/,
    },
  };
  $self->{defaultForm} = "main";
  ##db infrastructure
  $self->{INSERTorder}[@{$self->{INSERTorder}}] = [ qw( name ticker market ) ] ;  #insert order for db column names
  $self->{newOnly}[@{$self->{newOnly}}] = { sub_acct => "" };
  $self->{db2data}[@{$self->{db2data}}] = { name => "company", sub_acct => 'id' };  #map database columns to ->data hash, not specified assumes same name
  my $index = scalar @{$self->{data2db}};
  $self->{data2db}[$index] = reverse %{$self->{db2data}[$index]};

  $self->{SELECT}[@{$self->{SELECT}}] = "SELECT ##COLUMNS## FROM stocks_tb WHERE stocks_tb.sub_acct = #FIELD=$self->{id}#";
  $self->{INSERT}[@{$self->{INSERT}}] = "INSERT INTO stocks_tb (sub_acct, ##COLUMNS##) VALUES ((SELECT sub_acc_number FROM sub_acc_tb WHERE sub_acc_name = '#FIELD=name#' AND code = '#FIELD=code#'), ##VALUES##)";
  $self->{UPDATE}[@{$self->{UPDATE}}] = "UPDATE stocks_tb SET ##COLUMNS## WHERE sub_acct = '#FIELD=$self->{id}#'";
  #remember to use single quotes around ##foo## if needed
  $self->{DELETE}[@{$self->{DELETE}}] = "DELETE FROM stocks_tb WHERE sub_acct = #FIELD=$self->{id}#";  #is 'id' a hardcoded form element?
  #find out ifis this being referenced from other tables before deleting
  my $cannot = $l->map("Cannot Delete") ." ". $l->map("Stocks");
  $self->{preDELETE}[@{$self->{preDELETE}}] = "SELECT '$cannot'";
  #what should we output as the value of the thing just deleted?

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Accounting::Objects::Base(3)

=cut
