# Portfolio.pm - The Object Class that provides a Portfolio Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: Portfolio.pm,v 1.31 2004/12/10 22:01:48 pcxuser Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Portfolio;
use strict;
use Portal::AppState;
use Portal::Accounting::DB::Main;
use Portal::Accounting::DB::Security;
use Portal::Accounting::DisplayItems;
use Portal::Data::Config;
use Portal::Accounting::Objects::PortfolioObject;
use Portal::Accounting::Objects::PortfolioAccountObject;
use Portal::Accounting::Objects::PortfolioLotObject;
use Portal::Accounting::Objects::PortfolioStockObject;
use POSIX qw(tmpnam);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::AppState Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.02';

# global variables.
# modify to represent the commands this state provides.
my %commands;
%commands = ( display => "Customization Screen", execute => "Validate Input and display results", update => "Redisplay portfolio",
                 print => "Display portfolio for printing", executeFile => "Re-run a portfolio with a new date" );

=head1 NAME

Portfolio - Object used to build a Portfolio Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Portfolio;
  my $obj = Portal::Accounting::Portfolio->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Portfolio is a Portfolio class.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::Accounting::Portfolio module.
 See Portal::AppState(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

# run
# takes: command
# returns: The HTMLObject document to display or undef if an error.
sub run
{
  my $self = shift;
  my %args = ( command => "", @_ );
  my $command = $args{command};
  my $doc;
  my $ledgerCode;
  my $ledger;
  my $dateYear = $self->{methods}->getCurrentDate(format => "%Y");

  if (!exists $commands{$command})
  {
    $self->setError(errorString => sprintf($self->langObj->phraseObj->phrases->{commandNotKnown}, $command));
    return undef;
  }

  if (!exists $self->{input}->{ledgerCode})
  {
    $self->setError(errorString => "LedgerCode is required");
    return undef;
  }
  else
  {
    $ledgerCode = $self->{input}->{ledgerCode};
  }
  if (!exists $self->{input}->{ledger})
  {
    $self->setError(errorString => $self->langObj->phraseObj->phrases->{ledgerRequired});
    return undef;
  }
  else
  {
    $ledger = $self->{input}->{ledger};
  }
  if (!exists $self->{sessionObj}->{store}->{ledgers}->{$ledger})
  {
    $self->setError(errorString => sprintf($self->langObj->phraseObj->phrases->{invalidLedger}, $ledger));
    return undef;
  }

  $doc = HTMLObject::Normal->new;
  if ($command eq "display")
  {
    $doc = $self->displayCustomizationScreen(doc => $doc, firstTime => 1);
    if ($self->didErrorOccur)
    {
      $self->prefixError();
      return undef;
    }
  }
  elsif ($command eq "execute")
  {
    # validate input
    my $invAccounts = $self->{input}->{invAccounts};
    my $stocks = $self->{input}->{stocks};
    my $date = $self->{input}->{date};
    my $negativeFormat = $self->{input}->{negativeFormat};
    my $displayZeroShares = (exists $self->{input}->{displayZeroShares} ? 1 : 0);
    my $portfolioPhrase = $self->{langObj}->map("Portfolio");

    my @invAccounts = split /\0/, $invAccounts;
    my @stocks = split /\0/, $stocks;

    my $errorString = "";
    if (scalar @invAccounts == 0)
    {
      $errorString .= "You must select an Investment Account!<br>\n";
    }
    if ($date !~ /^\d{4}(-|\/)\d{2}(-|\/)\d{2}$/)
    {
      $errorString .= "date = '$date' is invalid!<br>\n";
    }
    if ($negativeFormat < 0 || $negativeFormat > 2)
    {
      $errorString .= "negativeFormat = '$negativeFormat' is invalid!<br>\n";
    }

    if (length $errorString > 0)
    {
      $doc = $self->displayCustomizationScreen(doc => $doc, error => $errorString);
      if ($self->didErrorOccur)
      {
        $self->prefixError();
        return undef;
      }
      return $doc;
    }
    else
    {
      my %times = ();
      $times{start} = time();
      my $dbObj = Portal::Accounting::DB::Main->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
      if ($dbObj->didErrorOccur)
      {
        $self->setError(errorString => $dbObj->errorMessage);
        return undef;
      }

      # gather the specified data and store in a storable object.

      # first we need to get the info for the specified accounts and sub accounts to pass into the data gathering routine.
      my @invAccountObjs = ();
      foreach my $account (@invAccounts)
      {
        my $accountObj = $dbObj->getAccountInfo(number => $account);
        if (!defined $accountObj)
        {
          if ($dbObj->didErrorOccur)
          {
            $self->setError(errorString => $dbObj->errorMessage);
            return undef;
          }
          $doc = $self->displayCustomizationScreen(doc => $doc, error => "Account = '$account' does not exist!");
          if ($self->didErrorOccur)
          {
            $self->prefixError();
            return undef;
          }
          return $doc;
        }
        else
        {
          if ($accountObj->didErrorOccur)
          {
            $self->setError(errorString => $accountObj->errorMessage);
            return undef;
          }
          $invAccountObjs[++$#invAccountObjs] = $accountObj;  # store for later use.
        }
      }
      $times{accounts} = time();
      my @stockObjs = ();
      foreach my $stock (@stocks)
      {
        my $stockObj = $dbObj->getSubAccountInfo(number => $stock);
        if ($dbObj->didErrorOccur)
        {
          $self->setError(errorString => $dbObj->errorMessage);
          return undef;
        }
        if (!defined $stockObj)
        {
          $doc = $self->displayCustomizationScreen(doc => $doc, error => "Stock = '$stock' does not exist!");
          if ($self->didErrorOccur)
          {
            $self->prefixError();
            return undef;
          }
          return $doc;
        }
        else
        {
          if ($stockObj->didErrorOccur)
          {
            $self->setError(errorString => $stockObj->errorMessage);
            return undef;
          }
          $stockObjs[++$#stockObjs] = $stockObj;  # store for later use.
        }
      }
      $times{subAccounts} = time();

      my $securitiesObj = Portal::Accounting::DB::Security->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
      if ($securitiesObj->didErrorOccur)
      {
        $self->setError(errorString => $securitiesObj->errorMessage);
        return undef;
      }
      my $portfolioObj = $securitiesObj->getPortfolioData(accounts => \@invAccountObjs, stocks => \@stockObjs, date => $date, ledgerCode => $ledgerCode,
                                         displayZeroShares => $displayZeroShares, year => $dateYear);
      $times{getPortfolio} = time();
      if ($securitiesObj->didErrorOccur)
      {
        $self->setError(errorString => $securitiesObj->errorMessage);
        return undef;
      }
      if (!defined $portfolioObj)
      {
        if ($securitiesObj->{errorCode} == -1)
        {
          $doc = $self->displayCustomizationScreen(doc => $doc, error => "No Portfolio Data returned for the selected Accounts and Stocks!");
        }
        else
        {
          $doc = $self->displayCustomizationScreen(doc => $doc, error => "Portfolio Object not created!");
        }
        if ($self->didErrorOccur)
        {
          $self->prefixError();
          return undef;
        }
        return $doc;
      }
      else
      {
        if ($portfolioObj->didErrorOccur)
        {
          $self->setError(errorString => $portfolioObj->errorMessage);
          return undef;
        }
      }

      my %data = ( portfolioObj => $portfolioObj, date => $date, invAccountObjs => \@invAccountObjs, stockObjs => \@stockObjs, displayZeroShares => $displayZeroShares, times => \%times );
      my $filename = tmpnam();
      $self->{sessionObj}->{store}->{$filename} = \%data;

      # call the display routine giving it the name of the file to work with.
      $doc = $self->displayPortfolio(doc => $doc, filename => $filename, negativeFormat => $negativeFormat);
      if ($self->didErrorOccur)
      {
        $self->prefixError();
        return undef;
      }
    }
  }
  elsif ($command eq "executeFile")
  {
    my %times = ();
    my $filename = $self->{input}->{filename};
    my $date = $self->{input}->{date};
    my $negativeFormat = $self->{input}->{negativeFormat};

    if ($date !~ /^\d{4}(-|\/)\d{2}(-|\/)\d{2}$/)
    {
      $self->setError(errorString => "date = '$date' is invalid!<br>\n");
      return $doc;
    }
    if ($negativeFormat < 0 || $negativeFormat > 2)
    {
      $self->setError(errorString => "negativeFormat = '$negativeFormat' is invalid!<br>\n");
      return $doc;
    }
    if (length $filename == 0)
    {
      $self->setError(errorString => "filename = '$filename' is invalid or does not exist!<br>\n");
      return $doc;
    }

    my %data = %{$self->{sessionObj}->{store}->{$filename}};

    $times{start} = $times{accounts} = $times{subAccounts} = time();
    my $securitiesObj = Portal::Accounting::DB::Security->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
    if ($securitiesObj->didErrorOccur)
    {
      $self->setError(errorString => $securitiesObj->errorMessage);
      return undef;
    }
    my $portfolioObj = $securitiesObj->getPortfolioData(accounts => $data{invAccountObjs}, stocks => $data{stockObjs}, date => $date, ledgerCode => $ledgerCode,
                                      displayZeroShares => $data{displayZeroShares}, year => $dateYear);
    $times{getPortfolio} = time();
    if (!defined $portfolioObj)
    {
      if ($securitiesObj->didErrorOccur)
      {
        $self->setError(errorString => $securitiesObj->errorMessage);
        return undef;
      }
      if ($securitiesObj->{errorCode} == -1)
      {
        $doc = $self->displayCustomizationScreen(doc => $doc, error => "No Portfolio Data returned for the selected Accounts and Stocks!");
      }
      else
      {
        $doc = $self->displayCustomizationScreen(doc => $doc, error => "Portfolio Object not created!");
      }
      if ($self->didErrorOccur)
      {
        $self->prefixError();
        return undef;
      }
      return $doc;
    }
    else
    {
      if ($portfolioObj->didErrorOccur)
      {
        $self->setError(errorString => $portfolioObj->errorMessage);
        return undef;
      }
    }

    $data{portfolioObj} = $portfolioObj;
    $data{date} = $date;
    $data{times} = \%times;
    $self->{sessionObj}->{store}->{$filename} = \%data;

    # display the Portfolio
    $doc = $self->displayPortfolio(doc => $doc, filename => $filename, negativeFormat => $negativeFormat);
    if ($self->didErrorOccur)
    {
      $self->prefixError();
      return undef;
    }
  }
  elsif ($command eq "update")
  {
    my $filename = $self->{input}->{filename};
    my $updateWhat = $self->{input}->{updateWhat};
    my $number = $self->{input}->{number};
    my $negativeFormat = $self->{input}->{negativeFormat};

    if (length $filename == 0)
    {
      $self->setError(errorString => "filename = '$filename' is invalid or does not exist!<br>\n");
      return $doc;
    }

    if ($updateWhat !~ /^(account|stock|refresh)$/)
    {
      $self->setError(errorString => "updateWhat = '$updateWhat' is invalid!<br>\n");
      return $doc;
    }
    if (length $number == 0 && $updateWhat ne "refresh")
    {
      $self->setError(errorString => "number = '$number' is invalid!<br>\n");
      return $doc;
    }
    if ($updateWhat eq "stock" && ($number != -1 && $number != -2 && scalar (split /\|/, $number) != 2))
    {
      $self->setError(errorString => "number = '$number' is invalid for Toggling a Stock!<br>\n");
      return $doc;
    }
    if ($negativeFormat < 0 || $negativeFormat > 2)
    {
      $self->setError(errorString => "negativeFormat = '$negativeFormat' is invalid!<br>\n");
      return $doc;
    }

    if ($updateWhat ne "refresh")
    {
      my %data = %{$self->{sessionObj}->{store}->{$filename}};

      my $portfolioObj = $data{portfolioObj};
      if (!defined $portfolioObj || !$portfolioObj->isValid)
      {
        $self->setError(errorString => "stored portfolioObj is invalid or does not exist!<br>\n");
        return $doc;
      }

      # find the account or stock to toggle
      if ($updateWhat eq "account")
      {
        if ($number == -1 || $number == -2)
        {
          for (my $count=0; $count < scalar @{$portfolioObj->{accounts}}; $count++)
          {
            $portfolioObj->{accounts}->[$count]->status($number == -1 ? "o" : "c");
          }
        }
        else  # normal behaviour
        {
          my $account = $number;
          my $found = 0;
          for (my $count=0; $count < scalar @{$portfolioObj->{accounts}} && !$found; $count++)
          {
            if ($portfolioObj->{accounts}[$count]->{accountNum} == $account)
            {
              $portfolioObj->{accounts}->[$count]->status($portfolioObj->{accounts}->[$count]->{status} eq "c" ? "o" : "c");
              $found = 1;
            }
          }

          if (!$found)
          {
            $self->setError(errorString => "Account = '$account' not found!<br>\n");
            return $doc;
          }
        }
      }
      elsif ($updateWhat eq "stock")
      {
        if ($number == -1 || $number == -2)
        {
          # loop through all Accounts (open or closed) and toggle the flag of all stocks to either be open or closed.
          for (my $count=0; $count < scalar @{$portfolioObj->{accounts}}; $count++)
          {
            $portfolioObj->{accounts}->[$count]->status("o");  # force all Accounts to be open when we deal with all Stocks.
            for (my $count2=0; $count2 < scalar @{$portfolioObj->{accounts}->[$count]->{stocks}}; $count2++)
            {
              $portfolioObj->{accounts}->[$count]->{stocks}->[$count2]->status($number == -1 ? "o" : "c");
            }
          }
        }
        else  # normal behaviour
        {
          (my $account, my $stock) = split /\|/, $number;
          my $found = 0;
          for (my $count=0; $count < scalar @{$portfolioObj->{accounts}} && !$found; $count++)
          {
            if ($portfolioObj->{accounts}->[$count]->{accountNum} == $account)
            {
              # now find the Stock associated with this account.
              my $stockFound = 0;
              for (my $count2=0; $count2 < scalar @{$portfolioObj->{accounts}->[$count]->{stocks}} && !$stockFound; $count2++)
              {
                if ($portfolioObj->{accounts}->[$count]->{stocks}->[$count2]->{subAccountNum} == $stock)
                {
                  $portfolioObj->{accounts}->[$count]->{stocks}->[$count2]->status($portfolioObj->{accounts}->[$count]->{stocks}->[$count2]->{status} eq "c" ? "o" : "c");
                  $stockFound = 1;
                }
              }
              if (!$stockFound)
              {
                $self->setError(errorString => "Account = '$account' found but Stock = '$stock' not found!<br>\n");
                return $doc;
              }
              $found = 1;
            }
          }

          if (!$found)
          {
            $self->setError(errorString => "Account = '$account' not found!<br>\n");
            return $doc;
          }
        }
      }

      $data{portfolioObj} = $portfolioObj;  # make sure it is back in the data hash.
      $self->{sessionObj}->{store}->{$filename} = \%data;
    }

    # redisplay the Portfolio
    $doc = $self->displayPortfolio(doc => $doc, filename => $filename, negativeFormat => $negativeFormat);
    if ($self->didErrorOccur)
    {
      $self->prefixError();
      return undef;
    }
  }
  elsif ($command eq "print")
  {
    my $filename = $self->{input}->{filename};
    my $negativeFormat = $self->{input}->{negativeFormat};

    if (length $filename == 0) # || ! -e $filename)
    {
      $self->setError(errorString => "filename = '$filename' is invalid or does not exist!<br>\n");
      return $doc;
    }
    if ($negativeFormat < 0 || $negativeFormat > 2)
    {
      $self->setError(errorString => "negativeFormat = '$negativeFormat' is invalid!<br>\n");
      return $doc;
    }

    $doc = $self->displayPortfolio(doc => $doc, filename => $filename, negativeFormat => $negativeFormat, printable => "true");
    if ($self->didErrorOccur)
    {
      $self->prefixError();
      return undef;
    }
  }

  return $doc;
}

# Helper functions

# displayCustomizationScreen
# takes: doc, error
# optional: firstTime
# returns: HTMLObject::Normal
sub displayCustomizationScreen
{
  my $self = shift;
  my %args = ( doc => undef, error => "", firstTime => 0, @_ );
  my $doc = $args{doc};
  my $error = $args{error};
  my $firstTime = $args{firstTime};
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $invAccounts = $self->{input}->{invAccounts};
  my $tmpDate = $self->{methods}->getCurrentDate();
  my $dateYear = $self->{methods}->getCurrentDate(format => "%Y");
  my $date = (exists $self->{input}->{date} ? $self->{input}->{date} : "$tmpDate");
  my $stocks = $self->{input}->{stocks};
  my $negativeFormat = (exists $self->{input}->{negativeFormat} ? $self->{input}->{negativeFormat} : 0);
  my $displayZeroShares = (exists $self->{input}->{displayZeroShares} ? 1 : 0);
  my $dbObj = Portal::Accounting::DB::Main->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
  if ($dbObj->didErrorOccur)
  {
    $self->setError(errorString => $dbObj->errorMessage);
    return undef;
  }

  if (! defined $doc)
  {
    $self->setError(errorString => "doc must be defined!<br>\n");
    return undef;
  }

  if ($negativeFormat < 0 || $negativeFormat > 2)
  {
    $self->setError(errorString => "negativeFormat = '$negativeFormat' is invalid!<br>\n");
    return undef;
  }

  my $ledgerURI = $doc->encodeString(string => $ledger);

  # get the users colorScheme to work with.
  my $colorScheme = $self->{applicationObj}->getCurrentColorScheme(userId => $self->{userObj}->{id});
  if ($self->{applicationObj}->didErrorOccur)
  {
    $self->prefixError($self->{applicationObj}->errorMessage);
    return undef;
  }
  if (!defined $colorScheme)
  {
    $self->setError("No valid colorScheme returned!<br>\n");
    return undef;
  }

  # create the needed css entries
  my %cssEntries = $self->{methods}->generateColorSchemeCSS(colorScheme => $colorScheme, appName => "Accounting");
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError($self->{methods}->errorMessage);
    return undef;
  }
  $doc->print(%cssEntries);

  my $title = $self->{langObj}->phraseObj->{phrases}->{portfolioCustomizationTitle};
  my $execute = $self->{langObj}->phraseObj->{phrases}->{execute};
  my $investmentAccounts = $self->{langObj}->phraseObj->{phrases}->{investmentAccounts};
  my $stocksPhrase = $self->{langObj}->phraseObj->{phrases}->{stocks};
  my $datePhrase = $self->{langObj}->phraseObj->{phrases}->{date};
  my $selectAll = $self->{langObj}->phraseObj->{phrases}->{selectAll};
  my $unSelectAll = $self->{langObj}->phraseObj->{phrases}->{unSelectAll};
  my $refresh = $self->{langObj}->phraseObj->{phrases}->{refresh};
  my $negativeNumberFormatPhrase = $self->{langObj}->phraseObj->{phrases}->{negativeNumberFormatPhrase};
  my $stockPricesPhrase = $self->{langObj}->phraseObj->{phrases}->{stockPrices};
  my $investmentJournalPhrase = $self->{langObj}->phraseObj->{phrases}->{investmentJournal};
  my $displayZeroSharesPhrase = $self->{langObj}->phraseObj->{phrases}->{displayZeroShares};

  my $type0Checked = ($negativeFormat == 0 ? " checked" : "");
  my $type1Checked = ($negativeFormat == 1 ? " checked" : "");
  my $type2Checked = ($negativeFormat == 2 ? " checked" : "");

  my $zeroSharesSelected = ($displayZeroShares == 1 ? " checked" : "");

  my $displayItemObj = Portal::Accounting::DisplayItems->new(langObj => $self->{langObj});

  $doc->setTitle($title);
  $doc->setLinkDecorations;

  # I should lookup the Investment Account Type and Stock Sub Account Type values instead of hard coding them.

  # gather all Investment Accounts
  my @InvAccounts = $dbObj->getListOfAccounts(location => "local", type => 3, ledgerCode => $ledgerCode);
  if ($dbObj->didErrorOccur)
  {
    $self->setError(errorString => $dbObj->errorMessage);
    return undef;
  }

  # gather all Stocks
  my @Stocks = $dbObj->getListOfSubAccounts(type => 70, orderBy => "sub_acc_name, sub_acc_number");
  if ($dbObj->didErrorOccur)
  {
    $self->setError(errorString => $dbObj->errorMessage);
    return undef;
  }

  my %datePicker = $self->{methods}->generatePickerCode(type => "date", doc => $doc, form => "mainForm",
                          itemName => "date", itemValue => $date, year => $dateYear,
                          phrase => $datePhrase, class => "menu", linkClass => "menu", width => 300, height => 250);
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError($self->{methods}->errorMessage);
    return undef;
  }

  my $siteDir = $self->{configObj}->{siteDir};

  # display input form with multiselect boxes for Accounts and Stocks and a Date Selection Widget.
  $doc->setFocus("body");
  $doc->print(<<"END_OF_CODE");
<a href="$siteDir/cgi-bin/index.cgi?app=Accounting&state=InvJournal&command=display&ledgerCode=$ledgerCode&ledger=$ledgerURI" onMouseOver="window.status='$investmentJournalPhrase'; return true;">$investmentJournalPhrase</a>&nbsp;&nbsp;|&nbsp;
<a href="$siteDir/cgi-bin/index.cgi?app=Accounting&state=StockPrices&command=display&ledgerCode=$ledgerCode&ledger=$ledgerURI" onMouseOver="window.status='$stockPricesPhrase'; return true;">$stockPricesPhrase</a>&nbsp;&nbsp;|&nbsp;
<a href="$siteDir/cgi-bin/index.cgi?app=Accounting&state=Portfolio&command=display&ledgerCode=$ledgerCode&ledger=$ledgerURI" onMouseOver="window.status='$refresh'; return true;">$refresh</a>
<br>
<hr>
<center>
<h2>$title</h2>
<br>
<form name="mainForm" method="post" action="$siteDir/cgi-bin/index.cgi">
  <input type=hidden name="app" value="Accounting">
  <input type=hidden name="state" value="Portfolio">
  <input type=hidden name="command" value="execute">
  <input type=hidden name="ledger" value="$ledger">
  <input type=hidden name="ledgerCode" value="$ledgerCode">
  <table border=0 cellpadding=1 cellspacing=1 class="menu">
    <tr>
      <td align="center" colspan="3">
END_OF_CODE
  $doc->print(%datePicker);
  $doc->print(<<"END_OF_CODE");
      </td>
    </tr>
    <tr>
      <th align="center"><span class="menu">$investmentAccounts</span></th>
      <th align="center"><span class="menu">$stocksPhrase</span></th>
      <th align="center"><span class="menu">$negativeNumberFormatPhrase</span></th>
    </tr>
    <tr>
      <td rowspan="3" align="center">
        <select name="invAccounts" size="10" multiple>
END_OF_CODE

  my @tmpInvAccounts = split /\0/, $invAccounts;
  foreach my $acctObj (@InvAccounts)
  {
    my $selected = (grep(/$acctObj->{number}/, @tmpInvAccounts) > 0 ? " selected" : ($firstTime ? " selected" : ""));
    $doc->print("          <option value=\"$acctObj->{number}\"$selected>$acctObj->{name}\n");
  }

  $doc->print(<<"END_OF_CODE");
        </select>
        <br>
        <input type="button" name="allAccounts" value="$selectAll" onclick="selectAll(document.mainForm.invAccounts)">
        <br>
        <input type="button" name="noAccounts" value="$unSelectAll" onclick="unSelectAll(document.mainForm.invAccounts)">
      </td>
      <td rowspan="3" align="center">
        <select name="stocks" size="10" multiple>
END_OF_CODE

  my @tmpStocks = split /\0/, $stocks;
  foreach my $stockObj (@Stocks)
  {
    my $selected = (grep(/$stockObj->{number}/, @tmpStocks) > 0 ? " selected" : "");
    $doc->print("          <option value=\"$stockObj->{number}\"$selected>$stockObj->{name}\n");
  }

  $doc->print(<<"END_OF_CODE");
        </select>
        <br>
        <input type="button" name="allStocks" value="$selectAll" onclick="selectAll(document.mainForm.stocks)">
        <br>
        <input type="button" name="noStocks" value="$unSelectAll" onclick="unSelectAll(document.mainForm.stocks)">
      </td>
      <td>&nbsp;</td>
    </tr>
    <tr>
      <td align="center" valign="top">
        <table border="0" cellpadding="2" cellspacing="0">
          <tr>
            <td align="center">
              <input type="radio" name="negativeFormat" value="0"$type0Checked>
            </td>
            <td align="left">
              <font color="red">(</font>xxx.xx<font color="red">)</font>
            </td>
          </tr>
          <tr>
            <td align="center">
              <input type="radio" name="negativeFormat" value="1"$type1Checked>
            </td>
            <td align="left">
              <font color="red">-</font>xxx.xx
            </td>
          </tr>
          <tr>
            <td align="center">
              <input type="radio" name="negativeFormat" value="2"$type2Checked>
            </td>
            <td align="left">
              xxx.xx<font color="red">-</font>
            </td>
          </tr>
        </table>
      </td>
    </tr>
    <tr>
      <td><span class="menu">$displayZeroSharesPhrase</span> <input type="checkbox" name="displayZeroShares" value="1"$zeroSharesSelected></td>
    </tr>
  </table>
  <input type="submit" name="execute" value="$execute">
</form>
</center>
END_OF_CODE

  if (length $error > 0)
  {
    $doc->print("<br><br><center><font color=\"red\">Error:&nbsp;</font>$error</center>\n");
  }

  return $doc;
}

# displayPortfolio
# requires: doc, filename, negativeFormat
# optional: printable
# returns: doc or undef if error
# This routine uses the storable object defined by filename to know what data it needs to display.  It then displays the Portfolio from the
# stored data and provides the user with an interface to selectively browse through the displayed data.
sub displayPortfolio
{
  my $startTime = time();
  my $self = shift;
  my %args = ( doc => undef, filename => "", negativeFormat => 0, printable => "false", @_ );
  my $doc = $args{doc};
  my $filename = $args{filename};
  my $printable = $args{printable};
  my $negativeFormat = $args{negativeFormat};
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $ledgerColor = $self->{sessionObj}->{store}->{ledgers}->{$ledger}->{color};
  my $displayItemObj = Portal::Accounting::DisplayItems->new(langObj => $self->{langObj});
  my $dateYear = $self->{methods}->getCurrentDate(format => "%Y");

  if (! defined $doc)
  {
    $self->setError(errorString => "doc must be defined!<br>\n");
    return $doc;
  }
  if (length $filename == 0)
  {
    $self->setError(errorString => "filename = '$filename' is invalid or does not exist!<br>\n");
    return $doc;
  }
  if ($ledgerCode eq "")
  {
    $self->setError(errorString => "ledgerCode must be defined!<br>\n");
    return $doc;
  }

  if ($printable !~ /^(true|false)$/)
  {
    $self->setError(errorString => "printable = '$printable' is invalid!<br>\n");
    return $doc;
  }
  if ($negativeFormat < 0 || $negativeFormat > 2)
  {
    $self->setError(errorString => "negativeFormat = '$negativeFormat' is invalid!<br>\n");
    return $doc;
  }

  my %data = %{$self->{sessionObj}->{store}->{$filename}};

  my %times = %{$data{times}};
  my $portfolioObj = $data{portfolioObj};
  if (!defined $portfolioObj || !$portfolioObj->isValid)
  {
    $self->setError(errorString => "stored portfolioObj is invalid or does not exist!<br>\n");
    return $doc;
  }
  my $date = $data{date};

  my $phrases = $self->{langObj};

  my $refresh = $phrases->map("refresh");
  my $printPhrase = $phrases->map("print");
  my $portfolioPhrase = $phrases->map("Portfolio");
  my $datePhrase = $self->{langObj}->phraseObj->{phrases}->{date};

  my $ledgerURI = $doc->encodeString(string => $ledger);
  my $filenameURI = $doc->encodeString(string => $filename);

  my $siteDir = $self->{configObj}->{siteDir};

  my $url = $self->{configObj}->{siteDir} . "/cgi-bin/index.cgi?app=Accounting&state=Portfolio&command=\%s&ledger=%s&ledgerCode=$ledgerCode&filename=%s&negativeFormat=$negativeFormat";
  my $refreshUrl = sprintf($url, "update", $ledgerURI, $filenameURI);
  $refreshUrl .= "&updateWhat=refresh&number=";
  my $printUrl = sprintf($url, "print", $ledgerURI, $filenameURI);
  my $doAnotherUrl = sprintf($url, "display", $ledgerURI, $filenameURI);
  my $investmentJournalURL = $self->{configObj}->{siteDir} . "/cgi-bin/index.cgi?app=Accounting&state=InvJournal&command=display&ledgerCode=$ledgerCode&ledger=$ledgerURI";

  $doc->setTitle("Portfolio: Ledger = '$ledger', Date = '$date'");
  $doc->setLinkDecorations;

  my %datePicker = $self->{methods}->generatePickerCode(type => "date", doc => $doc, form => "doAnother",
                          itemName => "date", itemValue => $date, year => $dateYear,
                          phrase => $datePhrase, width => 300, height => 250);
  if ($self->{methods}->didErrorOccur)
  {
    $self->setError($self->{methods}->errorMessage);
    return undef;
  }

  # generate any needed JavaScript functions
  if ($printable eq "false")
  {
    $doc->setFocus("javascript");
    $doc->print(<<"END_OF_CODE");

function toggleAccount(account)
{
  var d = document.portfolio_form;

  d.updateWhat.value = "account";
  d.number.value = account;

  d.submit();
}

function toggleStock(stock)
{
  var d = document.portfolio_form;

  d.updateWhat.value = "stock";
  d.number.value = stock;

  d.submit();
}

END_OF_CODE

  }

  # now display the body.
  $doc->setFocus("body");
  #$doc->print("Account Time: " . ($times{accounts} - $times{start}) . " SubAccount Time: " . ($times{subAccounts} - $times{accounts}) . " getPortfolio Time: " . ($times{getPortfolio} - $times{subAccounts}) . " (SQL: " . ($portfolioObj->{times}->{sql} - $portfolioObj->{times}->{start}) . " Cast: " . ($portfolioObj->{times}->{cast} - $portfolioObj->{times}->{sql}) . " Process: " . ($portfolioObj->{times}->{end} - $portfolioObj->{times}->{cast}) . ") Total Time: <b>" . ($times{getPortfolio} - $times{start}) . "</b> Display Time: #DISPLAYTIME#<br>\n");
  my $actionUrl = $self->{configObj}->{siteDir} . "/cgi-bin/index.cgi";
  if ($printable eq "false")
  {
    $doc->print(<<"END_OF_CODE");
<a href="$refreshUrl" onMouseOver="window.status='$refresh'">$refresh</a>&nbsp;&nbsp;|&nbsp;
<a href="$doAnotherUrl" onMouseOver="window.status='New View'">New View</a>&nbsp;&nbsp;|&nbsp;
<a href="$investmentJournalURL" onMouseOver="window.status='Investment Journal'">Investment Journal</a>&nbsp;&nbsp;|&nbsp;
<a href="$printUrl" onMouseOver="window.status='$printPhrase'">$printPhrase</a>
<hr>
<table border="0" cellpadding="0" cellspacing="0" width="100%">
  <tr>
    <td align="left" width="33%">
      <a href="#" onMouseOver="window.status='Open All Accounts'" onClick="javascript: toggleAccount(-1); return false;">Open</a>&nbsp;/&nbsp;
      <a href="#" onMouseOver="window.status='Close All Accounts'" onClick="javascript: toggleAccount(-2); return false;">Close</a> All Accounts
    </td>
    <td align="center" width="34%">
      <form name="doAnother" method="post" action="$actionUrl">
        <input type="hidden" name="app" value="Accounting">
        <input type="hidden" name="state" value="Portfolio">
        <input type="hidden" name="command" value="executeFile">
        <input type="hidden" name="ledgerCode" value="$ledgerCode">
        <input type="hidden" name="ledger" value="$ledger">
        <input type="hidden" name="filename" value="$filename">
        <input type="hidden" name="negativeFormat" value="$negativeFormat">
END_OF_CODE
    $doc->print(%datePicker);
    $doc->print(<<"END_OF_CODE");
        <input type="submit" name="execute" value="Go">
      </form>
    </td>
    <td align="right" width="33%">
      <a href="#" onMouseOver="window.status='Open All Stocks'" onClick="javascript: toggleStock(-1); return false;">Open</a>&nbsp;/&nbsp;
      <a href="#" onMouseOver="window.status='Close All Stocks'" onClick="javascript: toggleStock(-2); return false;">Close</a> All Stocks
    </td>
  </tr>
</table>
END_OF_CODE
  }

  my $headerString = ($printable eq "true" ? "Return" : "Refresh");
  $doc->print(<<"END_OF_CODE");
<center>
<a href="$siteDir/cgi-bin/index.cgi?app=Accounting&state=Portfolio&command=update&updateWhat=refresh&number=&ledgerCode=$ledgerCode&ledger=$ledgerURI&filename=$filenameURI&negativeFormat=$negativeFormat" onMouseOver="window.status='$headerString'"><font color="black"><h2>Portfolio for Ledger <font color="$ledgerColor">$ledger</font> - Date = $date</h2></font></a>
</center>
<form name="portfolio_form" method="post" action="$actionUrl">
  <input type="hidden" name="app" value="Accounting">
  <input type="hidden" name="state" value="Portfolio">
  <input type="hidden" name="command" value="update">
  <input type="hidden" name="ledger" value="$ledger">
  <input type="hidden" name="ledgerCode" value="$ledgerCode">
  <input type="hidden" name="filename" value="$filename">
  <input type="hidden" name="negativeFormat" value="$negativeFormat">
  <input type="hidden" name="updateWhat" value="">
  <input type="hidden" name="number" value="">
</form>
<table border="0" cellspacing="2" cellpadding="0" width="100%">
  <tr>
    <td width="2%">&nbsp;</td>
    <td width="23%">&nbsp;</td>
    <td width="10%">&nbsp;</td>
    <td width="10%">&nbsp;</td>
    <td width="15%">&nbsp;</td>
    <td width="10%">&nbsp;</td>
    <td width="10%">&nbsp;</td>
    <td width="10%">&nbsp;</td>
    <td width="10%">&nbsp;</td>
  </tr>
  <tr>
    <th>&nbsp;</th><th align="left" valign="bottom" nowrap>Account / Stock / Lot</th><th align="center" valign="bottom">Buy<br>Date</th><th align="right" valign="bottom">Shares</th><th align="right" valign="bottom">Cost&nbsp;&nbsp;</th><th align="right" valign="bottom">Cost&nbsp;/<br>Share</th><th align="right" valign="bottom">Price&nbsp;/<br>Share</th><th align="right" valign="bottom">Market<br>Value&nbsp;</th><th align="right" valign="bottom">Gain&nbsp;/<br>Loss&nbsp;&nbsp;</th>
  </tr>
  <tr>
    <td align="center" colspan="9"><hr></td>
  </tr>
END_OF_CODE

  my $portfolioObj = $data{portfolioObj};

  my $output = "";
  my @paddingValue;
  my %grandTotals = ( "totalCost" => "0.00", "totalMarketValue" => "0.00", "totalGainLoss" => "0.00" );
  # now we need to walk over the PortfolioObject and display all Accounts and Stocks that are stored in it.
  foreach my $accountObj (@{$portfolioObj->{accounts}})
  {
    # increment the grand totals.
    $grandTotals{totalCost} += $accountObj->totalCost;
    $grandTotals{totalMarketValue} += $accountObj->totalMarketValue;
    $grandTotals{totalGainLoss} += $accountObj->totalGainLoss;

    # display the state of the Account (+ or -) but only if there are stocks to display for it.
    $output .= "  <tr>\n    <td align=\"right\">";
    if (scalar @{$accountObj->{stocks}} > 0 && $printable eq "false")
    {
      my $image = ($accountObj->status eq "c" ? "plus.gif" : "minus.gif");
      my $phrase = ($accountObj->status eq "c" ? "Expand" : "Contract");
      $output .= "<a href=\"#\" onMouseOver=\"window.status='$phrase'\" onClick=\"javascript: toggleAccount($accountObj->{accountNum}); return false;\"><img border=\"0\" src=\"$siteDir/js/Accounting/accounting/images/$image\" alt=\"$phrase\"></a>&nbsp;";
    }
    else
    {
      $output .= "&nbsp;";
    }
    $output .= "</td>\n";

    # now display the Account info
    if ($accountObj->status eq "c")
    {
      (my $accountName = $accountObj->accountName) =~ s/&/&amp;/g;
      $output .= "    <td align=\"left\" colspan=\"2\" nowrap>$accountName ($accountObj->{accountNum})</td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      @paddingValue = $displayItemObj->displayMoney(amount => $accountObj->{totalCost}, negativeFormat => $negativeFormat);
      $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      @paddingValue = $displayItemObj->displayMoney(amount => $accountObj->totalMarketValue, negativeFormat => $negativeFormat);
      $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
      @paddingValue = $displayItemObj->displayMoney(amount => $accountObj->totalGainLoss, negativeFormat => $negativeFormat);
      $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
      $output .= "  </tr>\n";
    }
    else
    {
      (my $accountName = $accountObj->accountName) =~ s/&/&amp;/g;
      $output .= "    <td align=\"left\" colspan=\"2\" nowrap>$accountName ($accountObj->{accountNum})</td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";  # the totals go at the bottom of the list of Stocks.
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      $output .= "  </tr>\n";

      # we also have to display all Stocks and their status's here.
      foreach my $stockObj (@{$accountObj->{stocks}})
      {
        $output .= "  <tr>\n";
        $output .= "    <td align=\"right\">&nbsp;</td>\n";
        $output .= "    <td align=\"left\" nowrap>";
        if (scalar @{$stockObj->{lots}} > 0 && $printable eq "false")
        {
          my $image = ($stockObj->status eq "c" ? "plus.gif" : "minus.gif");
          my $phrase = ($stockObj->status eq "c" ? "Expand" : "Contract");
          $output .= "<a href=\"#\" onMouseOver=\"window.status='$phrase'\" onClick=\"javascript: toggleStock('$accountObj->{accountNum}|$stockObj->{subAccountNum}'); return false;\"><img border=\"0\" src=\"$siteDir/js/Accounting/accounting/images/$image\" alt=\"$phrase\"></a>&nbsp;";
        }
        else
        {
          $output .= "&nbsp;&nbsp;&nbsp;&nbsp;";
        }
        (my $stockName = $stockObj->{subAccountName}) =~ s/&/&amp;/g;
        $output .= "$stockName</td>\n";

        # now output the Stock info
        if ($stockObj->status eq "c")
        {
          $output .= "    <td align=\"center\">&nbsp;</td>\n";
          $output .= "    <td align=\"right\"><b>$stockObj->{totalShares}</b></td>\n";
          @paddingValue = $displayItemObj->displayMoney(amount => $stockObj->totalCost, negativeFormat => $negativeFormat);
          $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
          @paddingValue = $displayItemObj->displayMoney(amount => $stockObj->averageCostPerShare, negativeFormat => $negativeFormat);
          $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
          @paddingValue = $displayItemObj->displayMoney(amount => $stockObj->newPricePerShare, negativeFormat => $negativeFormat);
          $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
          @paddingValue = $displayItemObj->displayMoney(amount => $stockObj->totalMarketValue, negativeFormat => $negativeFormat);
          $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
          @paddingValue = $displayItemObj->displayMoney(amount => $stockObj->totalGainLoss, negativeFormat => $negativeFormat);
          $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
          $output .= "  </tr>\n";
        }
        else
        {
          # display the header info and all lots for this stock
          $output .= "    <td align=\"right\">&nbsp;</td>\n";
          $output .= "    <td align=\"right\">&nbsp;</td>\n";
          $output .= "    <td align=\"right\">&nbsp;</td>\n";
          $output .= "    <td align=\"right\">&nbsp;</td>\n";
          $output .= "    <td align=\"right\">&nbsp;</td>\n";
          $output .= "    <td align=\"right\">&nbsp;</td>\n";
          $output .= "    <td align=\"right\">&nbsp;</td>\n";
          $output .= "  </tr>\n";

          # now display all lots for this account/stock pair.
          foreach my $lotObj (@{$stockObj->{lots}})
          {
            $output .= "  <tr>\n";
            $output .= "    <td>&nbsp;</td>\n";
            my $spaceString = "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;";
            $spaceString .= "&nbsp;&nbsp;" if ($printable eq "true");
            $output .= "    <td align=\"left\">$spaceString$lotObj->{lotRefNum}</td>\n";  # we need to make this link somewhere, don't know where yet.
            $output .= "    <td align=\"center\">$lotObj->{buyDate}</td>\n";
            $output .= "    <td align=\"right\">$lotObj->{shares}</td>\n";
            @paddingValue = $displayItemObj->displayMoney(amount => $lotObj->cost, negativeFormat => $negativeFormat);
            $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\">" . $paddingValue[1] . "</td>\n";
            @paddingValue = $displayItemObj->displayMoney(amount => $lotObj->costPerShare, negativeFormat => $negativeFormat);
            $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\">" . $paddingValue[1] . "</td>\n";
            @paddingValue = $displayItemObj->displayMoney(amount => $lotObj->newPricePerShare, negativeFormat => $negativeFormat);
            $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\">" . $paddingValue[1] . "</td>\n";
            @paddingValue = $displayItemObj->displayMoney(amount => $lotObj->marketValue, negativeFormat => $negativeFormat);
            $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\">" . $paddingValue[1] . "</td>\n";
            @paddingValue = $displayItemObj->displayMoney(amount => "$lotObj->{gainLoss}", negativeFormat => $negativeFormat);
            $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\">" . $paddingValue[1] . "</td>\n";
            $output .= "  </tr>\n";
          }

          # display the totals for the stock after all lots.
          $output .= "  <tr>\n";
          $output .= "    <td>&nbsp;</td>\n";
          $output .= "    <td>&nbsp;</td>\n";
          $output .= "    <td>&nbsp;</td>\n";
          $output .= "    <td align=\"right\"><hr align=\"right\" width=\"75%\"></td>\n";
          $output .= "    <td align=\"right\"><hr align=\"right\" width=\"75%\"></td>\n";
          $output .= "    <td align=\"right\"><hr align=\"right\" width=\"75%\"></td>\n";
          $output .= "    <td align=\"right\">&nbsp;</td>\n";
          $output .= "    <td align=\"right\"><hr align=\"right\" width=\"75%\"></td>\n";
          $output .= "    <td align=\"right\"><hr align=\"right\" width=\"75%\"></td>\n";
          $output .= "  </tr>\n";

          $output .= "  <tr>\n";
          $output .= "    <td>&nbsp;</td>\n";
          my $spaceString = "&nbsp;&nbsp;&nbsp;";
          $spaceString .= "&nbsp;" if ($printable eq "true");
          $output .= "    <td><b>$spaceString$stockName Total</b></td>\n";
          $output .= "    <td align=\"center\">&nbsp;</td>\n";
          $output .= "    <td align=\"right\"><b>$stockObj->{totalShares}</b></td>\n";
          @paddingValue = $displayItemObj->displayMoney(amount => $stockObj->totalCost, negativeFormat => $negativeFormat);
          $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
          @paddingValue = $displayItemObj->displayMoney(amount => $stockObj->averageCostPerShare, negativeFormat => $negativeFormat);
          $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
          @paddingValue = $displayItemObj->displayMoney(amount => $stockObj->newPricePerShare, negativeFormat => $negativeFormat);
          $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
          @paddingValue = $displayItemObj->displayMoney(amount => $stockObj->totalMarketValue, negativeFormat => $negativeFormat);
          $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
          @paddingValue = $displayItemObj->displayMoney(amount => $stockObj->totalGainLoss, negativeFormat => $negativeFormat);
          $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
          $output .= "  </tr>\n";
        }
      }

      # display the Totals for the Account.
      $output .= "  <tr>\n";
      $output .= "    <td>&nbsp;</td>\n";
      $output .= "    <td>&nbsp;</td>\n";
      $output .= "    <td>&nbsp;</td>\n";
      $output .= "    <td>&nbsp;</td>\n";
      $output .= "    <td align=\"right\"><hr align=\"right\" width=\"75%\"></td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      $output .= "    <td align=\"right\"><hr align=\"right\" width=\"75%\"></td>\n";
      $output .= "    <td align=\"right\"><hr align=\"right\" width=\"75%\"></td>\n";
      $output .= "  </tr>\n";

      $output .= "  <tr>\n";
      $output .= "    <td>&nbsp;</td>\n";
      my $spaceString = "";
      $output .= "    <td align=\"left\" colspan=\"2\"><b>$spaceString$accountName Total</b></td>\n";
      $output .= "    <td>&nbsp;</td>\n";
      @paddingValue = $displayItemObj->displayMoney(amount => $accountObj->totalCost, negativeFormat => $negativeFormat);
      $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      $output .= "    <td align=\"right\">&nbsp;</td>\n";
      @paddingValue = $displayItemObj->displayMoney(amount => $accountObj->totalMarketValue, negativeFormat => $negativeFormat);
      $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
      @paddingValue = $displayItemObj->displayMoney(amount => $accountObj->totalGainLoss, negativeFormat => $negativeFormat);
      $output .= "    <td align=\"right\" style=\"padding-right: $paddingValue[0];\"><b>" . $paddingValue[1] . "</b></td>\n";
      $output .= "  </tr>\n";
    }
  }

  # this row will be the calibration row that causes the table to properly proportion the cells.
  $output .= "  <tr>\n";
  $output .= "    <td width=\"2%\" align=\"right\">&nbsp;</td>\n";
  $output .= "    <td width=\"23%\" align=\"right\">&nbsp;</td>\n";
  $output .= "    <td width=\"10%\" align=\"right\">&nbsp;</td>\n";
  $output .= "    <td width=\"10%\" align=\"right\">&nbsp;</td>\n";
  $output .= "    <td width=\"15%\" align=\"right\">&nbsp;</td>\n";
  $output .= "    <td width=\"10%\" align=\"right\">&nbsp;</td>\n";
  $output .= "    <td width=\"10%\" align=\"right\">&nbsp;</td>\n";
  $output .= "    <td width=\"10%\" align=\"right\">&nbsp;</td>\n";
  $output .= "    <td width=\"10%\" align=\"right\">&nbsp;</td>\n";
  $output .= "  </tr>\n";

  if (scalar @{$portfolioObj->{accounts}} == 0)  # This should never happen!
  {
    $output .= "  <tr>\n";
    $output .= "    <td align=\"right\">&nbsp;</td>\n";
    $output .= "    <td colspan=\"8\" align=\"left\">No Accounts returned!</td>\n";
    $output .= "  </tr>\n";
  }
  else
  {
    # display the grand totals.
    $output .= "</table>\n";
    $output .= "<br>\n";
    $output .= "<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\">\n";
    $output .= "  <tr>\n";
    $output .= "    <td align=\"right\" width=\"25%\"><b>Grand Totals:</b></td>\n";
    $output .= "    <td align=\"left\" width=\"75%\">&nbsp;</td>\n";
    $output .= "  </tr>\n";
    $output .= "  <tr>\n";
    $output .= "    <td colspan=\"2\">&nbsp;</td>\n";
    $output .= "  </tr>\n";

    $grandTotals{totalCost} = sprintf("%.2f", $grandTotals{totalCost});
    $grandTotals{totalMarketValue} = sprintf("%.2f", $grandTotals{totalMarketValue});
    $grandTotals{totalGainLoss} = sprintf("%.2f", $grandTotals{totalGainLoss});

    $output .= "  <tr>\n";
    $output .= "    <td align=\"right\" width=\"25%\"><b>Total Cost</b></td>\n";
    @paddingValue = $displayItemObj->displayMoney(amount => $grandTotals{totalCost}, negativeFormat => $negativeFormat);
    $output .= "    <td align=\"left\" width=\"75%\">&nbsp; = &nbsp;<b>\$" . $paddingValue[1] . "</b></td>\n";
    $output .= "  </tr>\n";
    $output .= "  <tr>\n";
    $output .= "    <td align=\"right\" width=\"25%\"><b>Total Market Value</b></td>\n";
    @paddingValue = $displayItemObj->displayMoney(amount => $grandTotals{totalMarketValue}, negativeFormat => $negativeFormat);
    $output .= "    <td align=\"left\" width=\"75%\">&nbsp; = &nbsp;<b>\$" . $paddingValue[1] . "</b></td>\n";
    $output .= "  </tr>\n";
    $output .= "  <tr>\n";
    $output .= "    <td align=\"right\" width=\"25%\"><b>Total Gain / Loss</b></td>\n";
    @paddingValue = $displayItemObj->displayMoney(amount => $grandTotals{totalGainLoss}, negativeFormat => $negativeFormat);
    $output .= "    <td align=\"left\" width=\"75%\">&nbsp; = &nbsp;<b>\$" . $paddingValue[1] . "</b></td>\n";
    $output .= "  </tr>\n";
  }

  $doc->print($output);  # output the portfolio data into the HTMLObject.

  $doc->print("</table>\n");  # close the table out.

  if ($printable eq "true")
  {
    # generate the onLoad Code that will trigger a print of the page.
    my $onLoadCode = "window.print();\n";
    $doc->setOnload(code => $onLoadCode);
  }

  my $endTime = time();

  # now substitute into the Document.
  $doc->printTag(tag => "#DISPLAYTIME#", value => ($endTime - $startTime));

  return $doc;
}

=back

=cut

1;
__END__

=head1 Exported FUNCTIONS (non-Inline POD)

  HTMLObject run(command)
    Runs the specified command and returns the resulting HTMLObject.


=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
