# SubAccounts.pm - The Object Class that provides a SubAccounts Object
#$Id: SubAccounts.pm,v 1.3 2005/01/24 22:26:37 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.

package Portal::Accounting::SubAccounts;
use strict;
use Portal::Accounting::Main;
use Portal::Accounting::Objects::SubAccount;
use Portal::Accounting::Objects::StockObject;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.3 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Sub Accounts",
  new => "New Sub Account",
  save => "Save Sub Account",
  edit => "Edit Sub Account",
  delete => "Delete Sub Account",
  list => "Manage Sub Accounts",
  );

=head1 NAME

SubAccounts - Object used to work with SubAccounts

=head1 SYNOPSIS

  use Portal::Accounting::SubAccounts;
  my $obj = Portal::Accounting::SubAccounts->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

SubAccounts displays the SubAccounts Menu from which sub accounts
can be edited, created, and deleted.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::SubAccounts module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the SubAccounts State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{commands} = \%commands;

  # do validation
  if (!$self->Portal::Accounting::SubAccounts::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.
  my $s = Portal::Accounting::Objects::SubAccount->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if (! defined $s) { $self->error("SubAccount->new did not succeed<br>\n"); return undef;  }
  $self->{metaTypes} = $s->{metaTypes};
  return $self;
}

=item % getCommands(void)

  This method returns the commands hash and is only to be
  used by the registerApp method for creating the permissions,
  if using the AppStateCommandSecurity implementation.

=cut
sub getCommands
{
  return %commands;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject c_display()

=cut
sub c_display
{
  my $self = shift;
  my %args = ( command => "display", @_ );
  my $command = $args{command};
  my $doc = $self->prepDoc;
  my $siteDir = $self->{configObj}->{siteDir};
  my $url = $self->{baseURL};
  my $refreshUrl = $url . "?$self->{getInfo}&command=display";
  my $l = $self->{langObj};
  my $submit = $l->map("Manage");

  $doc->setFocus("body");
  $doc->print("<p>".$l->map("Select Sub Account Type to Manage")."</p><p>\n");
  #foreach metaType output into form select
  $doc->print(<<"ENDOF");
<form name="mainForm">
  <input type="hidden" name="app" value="$self->{app}">
  <input type="hidden" name="state" value="$self->{state}">
  <input type="hidden" name="command" value="list">
  <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
  <select name="type">
ENDOF
  my %reverse = reverse %{$self->{metaTypes}};
  foreach my $type (sort keys %reverse)
  {
    $doc->print("<option name=\"$type\" value=\"$reverse{$type}\">$type</option>\n");
  }
  $doc->print(<<"ENDOF");
  </select>
  <input type="submit" name="submit" value="$submit">
</form>
</p>
ENDOF
  return $doc;
}

sub c_new
{
  my $self = shift;
  return $self->c_edit(@_);
}

sub c_save
{
  my $self = shift;
  return $self->c_edit(@_);
}

=item HTMLObject c_edit()

  this routine is to create/edit a new account

=cut
sub c_edit
{
  my $self = shift;
  my $doc = $self->prepDoc;
  my %args = ( @_ );
  my $command = $self->{input}->{command};
  my $l = $self->{langObj};
  my $Manage = $l->map("Manage");
  my $subAccount;
  my $type = $self->{input}->{type};
#  $doc->print($type."<hr>"); #!!!should probably populate(id => ) then ->select to get type
  if ($type == 70)
  {
    $subAccount = Portal::Accounting::Objects::StockObject->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  }
  else
  {
    $subAccount = Portal::Accounting::Objects::SubAccount->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, type => $self->{input}->{type});
  }
  if ($subAccount->error) { $doc->print($subAccount->errorMessage); return $doc;  }
  #how do i know if this is edit or new?
  if ($self->{input}->{command} eq "save")
  { #validate user input then react
    if ($subAccount->validateForm(input => $self->{input}), formName => "main")
    {
      my $result = $subAccount->populate(%{$self->{input}});
      if (not defined $result) { $self->error($subAccount->errorMessage);  return undef;  }
      if ($result) #populate worked
      {
        if ($subAccount->isNew) #insert subAccount obj
        {
          $subAccount->insert();
          $doc->print($subAccount->debugString);
        }
        else
        {
          $subAccount->update();
        }
        if ($subAccount->error)
        {
          $doc->print("Error".$subAccount->errorMessage());
        }
        else
        {
          $doc->setFocus("body");
          $doc->print("Success");
          $doc->print(<<"ENDOF");
  <form name="list">
    <input type="submit" name="submit" value="$Manage $self->{metaTypes}->{$self->{input}->{type}}">
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="command" value="list">
    <input type="hidden" name="type" value="$self->{input}->{type}">
  </form>
ENDOF
          $doc->print($self->mainButton);
          return $doc;
        }
      }
      else { $doc->print($subAccount->genErrorString("all"));  }
    }
    #invalid so let form redisplay
    else
    {
      $doc->print($subAccount->errorMessage);
    }
  }
  elsif ($command eq "edit")
  {
    if (! $subAccount->populate(id => $self->{input}->{id}) || ! $subAccount->select() )
    {
      $doc->print("Error".$subAccount->errorMessage());
    }
  }
  #display form
  if ($subAccount->debug){$doc->print($subAccount->{debugString});}
  $subAccount->resetError;
  my %form = $subAccount->printForm(formName => "main");
  if ($subAccount->error) {  $doc->print($subAccount->errorMessage); }
  else {  $doc->print(%form);  }

  $doc->print(<<"ENDOF");
  <form name="list">
    <input type="submit" name="submit" value="$Manage $self->{metaTypes}->{$self->{input}->{type}}">
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="command" value="list">
    <input type="hidden" name="type" value="$self->{input}->{type}">
  </form>
ENDOF
  $doc->print($self->mainButton);
  return $doc;
}

sub c_list
{
  my $self = shift;
  my $doc = $self->prepDoc();
  my $type = $self->{metaTypes}->{$self->{input}->{type}};
  my $systemType = 0;
  if ($self->{input}->{type} == 1 || $self->{input}->{type} == 15 || $self->{input}->{type} == 90)
  {
    $systemType = 1;
  }
  my $l = $self->{langObj};
  my $New = $l->map("New");
  my $Edit = $l->map("Edit");
  my $Name = $l->map("Name");
  my $Code = $l->map("Code");
  my $Delete = $l->map("Delete");
  my $Manage = $l->map("Manage");
  my $SubAccounts = $l->map("Sub Accounts");
  my $confirm = $l->map("confirmDeleteSubAccount");
  my %foo;
  my $message = "";
  my $sql = "SELECT sub_acc_name AS name, sub_acc_number AS id, code FROM sub_acc_tb WHERE type = '$self->{input}->{type}' ORDER BY sub_acc_name";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }

  $foo{css} .= <<"ENDOF";
  .name { text-align: left; }
  .code { text-align: left; }
  .delete { text-align: center; }
ENDOF
  $doc->print("<h4>$Manage $type $SubAccounts</h4>");
  $doc->print(<<"ENDOF");
  <table width="50%" class="main">
    <tr>
      <th class="name">$Name</th>
      <th class="code">$Code</th>
      <th class="delete">$Delete</th>
    </tr>
ENDOF
  while (my $data = $result->fetchrow_hashref)
  {
    if ($systemType)
    {
      $doc->print(<<"ENDOF");
  <tr>
    <td class="name">$data->{name}</td>
    <td class="code">$data->{code}</td>
    <td class="delete">&nbsp;</td>
  </tr>
ENDOF
    }
    else
    {
      $doc->print(<<"ENDOF");
  <tr>
    <td class="name"><a href="?app=$self->{app}&state=$self->{state}&command=edit&id=$data->{id}&type=$self->{input}->{type}">$data->{name}</a></td>
    <td class="code">$data->{code}</td>
    <td class="delete">
      <form name="delete_$data->{number}">
        <input type="hidden" name="app" value="$self->{app}">
        <input type="hidden" name="state" value="$self->{state}">
        <input type="hidden" name="command" value="delete">
        <input type="hidden" name="type" value="$self->{input}->{type}">
        <input type="hidden" name="id" value="$data->{id}">
        <input type="hidden" name="confirmDelete" value="0">
        <input type="submit" name="delete" value="X" onclick="return deleteConfirm(this.form); ">
      </form>
    </td>
  </tr>
ENDOF
    }
  }
  $doc->print("</table>");
  if (defined $self->{input}->{message})
  {
    $message = '<p class="message">'.$self->{input}->{message}.'</p>';
  }
  $foo{javascript} = <<"ENDOF";
function deleteConfirm(myForm)
{
  if (confirm("$confirm"))
  {
    myForm.confirmDelete.value = 1;
    return true;
  }
  return false;
}
ENDOF
  $foo{body} .= $message;
  if (!$systemType)
  {
    $foo{body} .= <<"ENDOF";
    <form name="new">
      <input type="hidden" name="app" value="$self->{app}">
      <input type="hidden" name="state" value="$self->{state}">
      <input type="hidden" name="command" value="new">
      <input type="hidden" name="type" value="$self->{input}->{type}">
      <input type="submit" name="new" value="$New">
    </form>
    <br>
ENDOF
  }
  $foo{body} .= $self->mainButton;
  $doc->print(%foo);
  return $doc;
}

=item HTMLObject c_delete()


=cut
sub c_delete
{
  my $self = shift;
  my %args = (doc => undef, @_);
  my $l = $self->{langObj};
  my $Manage = $l->map("Manage");
  my $Failed = $l->map("Failed");
  my $Success = $l->map("Success");
  my $confirm = $l->map("confirmDeleteSubAccount");
  my $doc;
  if (defined $args{doc}) { $doc = $args{doc}; }
  else { $doc = $self->prepDoc();  }
  my $ChartOfSubsubAccounts = $l->map("Sub Accounts");
  my $type = $self->{input}->{type};
  my $subAccount;
  if ($type == 70)
  {
    $subAccount = Portal::Accounting::Objects::StockObject->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  }
  else
  {
    $subAccount = Portal::Accounting::Objects::SubAccount->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, type => $self->{input}->{type});
  }
  if (! $subAccount->populate(id => $self->{input}->{id}) || ! $subAccount->select() )
  {
    $doc->print($subAccount->errorMessage);
    return $doc;
  }
  my $typeName = $subAccount->typeName;
  $subAccount->resetDebug;
  if (defined $self->{input}->{confirmDelete} && $self->{input}->{confirmDelete} eq "1")
  {#do the actual delete
    my @result = $subAccount->delete();
    if (! defined @result)
    {
      $doc->print($subAccount->errorMessage);
    }
    else
    {
      my $out = $self->{langObj}->map("Delete") . ": ";
      if ($result[0] == -1)
      {
        $out .= "$Failed<br>".$result[1];
      }
      else {  $out .= "$Success:".$result[0]."<br>".$result[1];  }
      $doc->print($out."<br>". $subAccount->debugString);
    $doc->print(<<"ENDOF");
<table width="50%"><tr>
  <td>
  <form name="type">
    <input type="submit" name="submit" value="$Manage $typeName">
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="command" value="list">
    <input type="hidden" name="type" value="$type">
  </form>
  </td><td align="right">
ENDOF
    $doc->print($self->mainButton);
    $doc->print(<<"ENDOF");
  </td>
</tr></table>
ENDOF
    }
    return $doc;
  }
  else
  { #output confirm d
    my $yes = $self->{langObj}->map("Yes");
    $doc->print(<<"ENDOF");
<form name="mainForm">
  <p>$confirm<br>
  <input type="submit" name="yes" value="$yes"></p>
  <input type="hidden" name="app" value="$self->{app}">
  <input type="hidden" name="state" value="$self->{state}">
  <input type="hidden" name="command" value="delete">
  <input type="hidden" name="confirmDelete" value="1">
  <input type="hidden" name="id" value="$self->{input}->{id}">
</form>
<br>
<table width="50%"><tr>
  <td>
  <form name="type">
    <input type="submit" name="submit" value="$Manage $self->{metaTypes}->{$self->{input}->{type}}">
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="command" value="list">
    <input type="hidden" name="type" value="$self->{input}->{type}">
  </form>
  </td><td align="right">
ENDOF
    $doc->print($self->mainButton);
    $doc->print(<<"ENDOF");
  </td>
</tr></table>
ENDOF
  }
  return $doc;
}

sub mainButton
{
  my $self = shift;
  my $return;
  my $display = $self->{langObj}->map($self->{commands}->{display});
  $return = << "ENDOF";
  <form name="display">
    <input type="submit" name="submit" value="$display">
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="command" value="display">
  </form>
ENDOF
  return $return;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
