# TransactionParser.pm - Will parse the XML file/string and return the data object.
# Created by James A. Pattie.  Copyright (c) 2003, Xperience, Inc.
=head1 NAME

TransactionParser - The XML Transaction Import/Export Parser Module.

=head1 SYNOPSIS

  use Portal::Accounting::XML::TransactionParser;
  my $obj;
  eval { $obj = Portal::Accounting::XML::TransactionParser->new(langObj => $self->{langObj},
          appConfigObj => $appConfigObj); };
  if ($@)
  {
    # error occured!
    $self->setError("$@");
    return undef;   # signal an error to your caller however you need to.
  }
  my $importObj;
  eval { $importObj = $obj->parse(file => "import.xml"); };
  if ($@)
  {
    # error occured!
    $self->setError("$@");
    return undef;
  }
  # this is a TransactionImport Data object.
  # you can now process the incoming transaction.

=head1 DESCRIPTION

TransactionParser will parse XML files that have been generated to the XIWA
Transaction Import/Export specification.  See the
Portal::Accounting::Objects::TransactionImport and
Portal::Accounting::Objects::TransactionExport man pages for the
structure of the returned data.

=cut
package Portal::Accounting::XML::TransactionParser;

use strict;
use XML::LibXML;
use Portal::Base;
use Portal::Accounting::Objects::TransactionImport;
use Portal::Accounting::Objects::TransactionExport;
use Portal::Accounting::Objects::AccountObject;
use Portal::Accounting::Objects::SubAccountObject;
use Portal::Accounting::Objects::StockObject;
use Portal::Accounting::Objects::Ledger;
use Portal::Accounting::Objects::LedgerContainer;
use Portal::Accounting::Objects::SubEntity;
use Portal::Accounting::Objects::Journal;
use Portal::Accounting::Objects::JournalEntry;
use Portal::Accounting::Objects::JournalEntryRow;
use vars qw ($AUTOLOAD @ISA @EXPORT $VERSION);

require Exporter;
@ISA = qw(Portal::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = "1.0";

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(appConfigObj, debug)

 Creates a new instance of the Portal::Accounting::XML::TransactionParser module.
 See Portal::Base(3) for a listing of required arguments.

 If an error occurs, it will die, so eval it.

 requires:
   appConfigObj - Portal::Data::Config Accounting Config Object.
   debug - boolean (causes debug output to be generated if on)

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( appConfigObj => undef, debug => 0, @_ );
  my $errStr = "Portal::Accounting::XML::TransactionParser->new()  - Error:";

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{importVersion} = "1.0";  # version of transactionImport file
  $self->{exportVersion} = "1.0";  # version of transactionExport file

  $self->{dataObj} = undef;
  $self->{dataFile} = "";
  $self->{appConfigObj} = $args{appConfigObj};
  $self->{debug} = $args{debug};

  eval { $self->{xmlObj} = XML::LibXML->new(); };
  if ($@)
  {
    die "$errStr $@\n";
  }

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if (!defined $self->{appConfigObj})
  {
    $self->missing("appConfigObj");
  }
  if ($self->{debug} !~ /^(0|1)$/)
  {
    $self->invalid("debug", $self->{debug});
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item Portal::Accounting::Objects::TransactionImport/Export parse(file, string)
    Does the actual parsing of the XML file and generates the
    resulting data object and returns it.

    file points to the XML Config file to use.

    If you don't specify a file to work with then you must specify the
    xml via the string argument.  If you specify both, then the string
    will take precedence.  The file must still point to a valid file.

    returns: Portal::Accounting::Objects::TransactionImport/Ex instance with parsed info.

    If an error occurs, it will die, so eval it.

=cut
sub parse
{
  my $self = shift;
  my %args = ( "file" => "", string => "", @_ );
  my $nodes = undef;
  my $errStr = "Portal::Accounting::XML::TransactionParser->parse()  - Error:";

  if (length $args{file} > 0)
  {
    if ($args{file} !~ /^(-|.*\.xml)$/)
    {
      die "$errStr file = '$args{file}' is not a valid file!\n";
    }
    if ($args{file} ne "-" && ! -e $args{file})
    {
      die "$errStr Can not find config file = '$args{file}'!  $!\n";
    }
  }
  elsif (length $args{string} == 0)
  {
    die "$errStr You must specify either 'file' or 'string'!\n";
  }

  $self->{dataFile} = (length $args{string} > 0 ? $args{string} : $args{file});

  if (length $args{file} > 0 && length $args{string} == 0)
  {
    eval { $self->{xmlDoc} = $self->{xmlObj}->parse_file($self->{dataFile}); };
  }
  else
  {
    eval { $self->{xmlDoc} = $self->{xmlObj}->parse_string($self->{dataFile}); };
  }
  if ($@)
  {
    die "$errStr $@\n";
  }

  # get the type
  $self->{type} = $self->getType;

  if ($self->{type} =~ /^(transactionImport)$/)
  {
    eval { $self->{dataObj} = Portal::Accounting::Objects::TransactionImport->new(langObj => $self->{langObj}); };
    if ($@)
    {
      die "$errStr $@\n";
    }

    # start by validating the version of the XML file.
    $self->validateVersion;

    # initiate the data structure.  Fill in any default values possible.
    $self->{dataObj}->{version} = $self->{importVersion};
    $self->{dataObj}->{dataFile} = $self->{dataFile};
  }
  elsif ($self->{type} =~ /^(transactionExport)$/)
  {
    eval { $self->{dataObj} = Portal::Accounting::Objects::TransactionExport->new(langObj => $self->{langObj}); };
    if ($@)
    {
      die "$errStr $@\n";
    }

    # start by validating the version of the XML file.
    $self->validateVersion;

    # initiate the data structure.  Fill in any default values possible.
    $self->{dataObj}->{version} = $self->{exportVersion};
    $self->{dataObj}->{dataFile} = $self->{dataFile};
  }
  else
  {
    die "$errStr  Unknown transaction type = '$self->{type}'!\n";
  }

  # gather the timestamp, genSysId, LedgerCode, LedgerName info
  $self->getGlobalInfo;

  # gather the <accounts> values
  $self->getAccounts;

  # gather the <subAccounts> values
  $self->getSubAccounts;

  # gather the <stocks> values
  $self->getStocks;

  # gather the <ledgers> values
  $self->getLedgers;

  return $self->{dataObj};
}

=item hash getAttributes(node)
 requires: node - XPath Node
 returns:  hash of attributes for the specified node.
=cut
sub getAttributes
{
  my $self = shift;
  my %args = ( node => undef, @_ );
  my $node = $args{node};
  my %attributes = ();
  my $errStr = "Portal::Accounting::XML::TransactionParser->getAttributes()  - Error:";

  if (!defined $node)
  {
    die "$errStr  You must specify the XPath Node to work with!\n";
  }
  if ($node->getType() != XML_ELEMENT_NODE)
  {
    die "$errStr  You did not specify an XPath Node: " . $node->getType() . "\n";
  }
  foreach my $attribute ($node->getAttributes)
  {
    my $name = $attribute->getName;
    $attributes{$name} = $attribute->getValue;
  }

  return %attributes;
}

=item array getNodes(path, context)
 required: path - XPath to search for
 optional: context - the XPath object to base the search from.  Make sure your
path is relative to it!
 returns:  array - array of nodes returned.  These are the XPath objects
representing each node.
=cut
sub getNodes
{
  my $self = shift;
  my %args = ( path => "*", context => undef, @_ );
  my $path = $args{path};
  my $context = $args{context};
  my @nodes = ( );
  my $nodes = undef;
  my $errStr = "Portal::Accounting::XML::TransactionParser->getNodes()  - Error:";

  if (length $path == 0)
  {
    die "$errStr  You must specify a path!\n";
  }

  if (! defined $context)
  {
    $nodes = $self->{xmlDoc}->findnodes($path);
  }
  else
  {
    $nodes = $context->findnodes($path);
  }
  if (!$nodes->isa('XML::LibXML::NodeList'))
  {
    die "$errStr  Query '$path' didn't return a nodelist: " . $nodes->getType()
      . "\n";
  }
  if ($nodes->size)
  {
    #print "Found " . $nodes->size . " nodes...\n";
    foreach my $node ($nodes->get_nodelist)
    {
      push @nodes, $node;
    }
  }

  return @nodes;
}

=item string getType(void)
    returns the top level tag name (transactionImport, transactionExport).
=cut
sub getType
{
  my $self = shift;
  my $errStr = "Portal::Accounting::XML::TransactionParser->getType()  - Error:";
  my $type = "";

  my @nodes = $self->getNodes(path => "/transactionImport");
  if (scalar @nodes == 0)
  {
    @nodes = $self->getNodes(path => "/transactionExport");
    if (scalar @nodes == 0)
    {
      die "$errStr  Your XML file doesn't contain a <transactionImport> or <transactionExport> tag!\n";
    }
    if (scalar @nodes > 1)
    {
    die "$errStr  You have too many <transactionExport> tags!  You should only have one!\n";
    }
    $type = "transactionExport";

    return $type;
  }
  if (scalar @nodes > 1)
  {
    die "$errStr  You have too many <transactionImport> tags!  You should only have one!\n";
  }
  $type = "transactionImport";

  return $type;
}

=item  string getVersion(void)
    returns the version value from the parent <transactionImport/Export> tag.

=cut
sub getVersion
{
  my $self = shift;
  my $errStr = "Portal::Accounting::XML::TransactionParser->getVersion()  - Error:";

  my @nodes = $self->getNodes(path => "/$self->{type}");
  if (scalar @nodes == 0)
  {
    die "$errStr  Your XML file doesn't contain a <$self->{type}> tag!\n";
  }
  if (scalar @nodes > 1)
  {
    die "$errStr  You have too many <$self->{type}> tags!  You should only have one!\n";
  }
  my %attributes = $self->getAttributes(node => $nodes[0]);
  if (!exists $attributes{version})
  {
    die "$errStr  You do not have the version defined!\n";
  }

  return $attributes{version};
}

=item void validateVersion(void)

 This routine looks up the <transactionImport/Export version=""> tag and validates that the
 version specified is the same as what we know how to work with.

=cut

sub validateVersion
{
  my $self = shift;
  my $errStr = "Portal::Accounting::XML::TransactionParser->validateVersion()  - Error:";

  my $version = $self->getVersion;
  my $curVersion = ($self->{type} eq "transactionImport" ? $self->{importVersion} : $self->{exportVersion});
  if ($version !~ /^($curVersion)$/)
  {
    die "$errStr  '$version' is not equal to Version '$curVersion'!\n";
  }
}

=item void getGlobalInfo(void)

 gathers the timestamp, genSysId, ledgerCode, and ledgerName values
 and sets them in the dataObj we are building up.

=cut

sub getGlobalInfo
{
  my $self = shift;
  my $errStr = "Portal::Accounting::XML::TransactionParser->getGlobalInfo()  - Error:";

  my @nodes = $self->getNodes(path => "/$self->{type}");
  if (scalar @nodes == 0)
  {
    die "$errStr  Your XML file doesn't contain a <$self->{type}> tag!\n";
  }
  if (scalar @nodes > 1)
  {
    die "$errStr  You have too many <$self->{type}> tags!  You should only have one!\n";
  }
  my %attributes = $self->getAttributes(node => $nodes[0]);
  if (!exists $attributes{timestamp})
  {
    die "$errStr  You do not have the timestamp defined!\n";
  }
  if (!exists $attributes{genSysId})
  {
    die "$errStr  You do not have the genSysId defined!\n";
  }

  # validate the timestamp
  if ($attributes{timestamp} !~ /^(\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2})$/)
  {
    die "$errStr  timestamp = '$attributes{timestamp}' does not appear to be valid!\n";
  }

  if (length $attributes{genSysId} == 0)
  {
    die "$errStr  genSysId = '$attributes{genSysId}' is not valid!\n";
  }

  $self->{dataObj}->{timestamp} = $attributes{timestamp};
  $self->{dataObj}->{genSysId} = $attributes{genSysId};
}

=item void getAccounts(void)

 Gathers all <accounts> entries and stores in accounts[]

=cut

sub getAccounts
{
  my $self = shift;
  my $parent = "accounts";
  my $tag = "account";
  my $errStr = "Portal::Accounting::XML::TransactionParser->getAccounts()  - Error:";

  my @nodes = $self->getNodes(path => "/$self->{type}/$parent");
  if (scalar @nodes == 0)
  {
    return;  # this is a valid scenario.
  }
  if (scalar @nodes > 1)
  {
    die "$errStr  You have too many <$parent> tags!  You should only have one!\n";
  }

  # now get all the $tag tags under $parent
  @nodes = $self->getNodes(path => "/$self->{type}/$parent/$tag");
  if (@nodes == 0)
  {
    return;  # this is a valid scenario.
  }
  # now foreach tag, process and create it's object.
  foreach my $node (@nodes)
  {
    my $nodeName = $node->getName;
    if ($nodeName ne $tag)
    {
      die "$errStr  <$nodeName> is invalid inside the <$parent> tag!\n";
    }
    # for this node, get the following tags
    my %tagInfo = ();
    foreach my $tagName (qw(name code type taxable))
    {
      my @tagNodes = $self->getNodes(path => $tagName, context => $node);
      if (@tagNodes == 0)
      {
        die "$errStr  <$tagName> is not defined in <$parent><$tag>!\n";
      }
      if (@tagNodes > 1)
      {
        die "$errStr  You have too many <$tagName> tags in <$parent><$tag>!  You should only have one!\n";
      }
      # get the $tagName tags contents and store in the tagInfo hash.
      my $value = $tagNodes[0]->textContent;
      $tagInfo{$tagName} = $value;
      print "$tagName = '$value'\n" if ($self->{debug});
    }
    # now instantiate the object and store it.
    my $obj = Portal::Accounting::Objects::AccountObject->new(langObj => $self->{langObj}, name => $tagInfo{name},
              number => $tagInfo{code}, type => $tagInfo{type}, active => "true", taxable => $tagInfo{taxable});
    if ($obj->didErrorOccur)
    {
      die "$errStr\n" . $obj->errorMessage;
    }
    # first make sure we don't have another object with the same name or number defined.
    foreach my $tmpObj (@{$self->{dataObj}->{$parent}})
    {
      if ($tmpObj->{name} eq $obj->{name} || $tmpObj->{number} eq $obj->{number})
      {
        die "$errStr  You have already defined an $tag with name = '$obj->{name}', code = '$obj->{number}'!\n";
      }
    }
    push @{$self->{dataObj}->{$parent}}, $obj;
  }
}

=item void getSubAccounts(void)

 Gathers all <subAccounts> entries and stores in subAccounts[]

=cut

sub getSubAccounts
{
  my $self = shift;
  my $errStr = "Portal::Accounting::XML::TransactionParser->getSubAccounts()  - Error:";
  my $parent = "subAccounts";
  my $tag = "subAccount";

  my @nodes = $self->getNodes(path => "/$self->{type}/$parent");
  if (scalar @nodes == 0)
  {
    return;  # this is a valid scenario.
  }
  if (scalar @nodes > 1)
  {
    die "$errStr  You have too many <$parent> tags!  You should only have one!\n";
  }

  # now get all the $tag tags under $parent
  @nodes = $self->getNodes(path => "/$self->{type}/$parent/$tag");
  if (@nodes == 0)
  {
    return;  # this is a valid scenario.
  }
  # now foreach tag, process and create it's object.
  foreach my $node (@nodes)
  {
    my $nodeName = $node->getName;
    if ($nodeName ne $tag)
    {
      die "$errStr  <$nodeName> is invalid inside the <$parent> tag!\n";
    }
    # for this node, get the following tags
    my %tagInfo = ();
    foreach my $tagName (qw(name code type))
    {
      my @tagNodes = $self->getNodes(path => $tagName, context => $node);
      if (@tagNodes == 0)
      {
        die "$errStr  <$tagName> is not defined in <$parent><$tag>!\n";
      }
      if (@tagNodes > 1)
      {
        die "$errStr  You have too many <$tagName> tags in <$parent><$tag>!  You should only have one!\n";
      }
      # get the $tagName tags contents and store in the tagInfo hash.
      my $value = $tagNodes[0]->textContent;
      $tagInfo{$tagName} = $value;
      print "$tagName = '$value'\n" if ($self->{debug});
    }
    # now instantiate the object and store it.
    my $obj = Portal::Accounting::Objects::SubAccountObject->new(langObj => $self->{langObj}, name => $tagInfo{name},
              number => $tagInfo{code}, type => $tagInfo{type});
    if ($obj->didErrorOccur)
    {
      die "$errStr\n" . $obj->errorMessage;
    }
    # first make sure we don't have another object with the same name or number defined.
    foreach my $tmpObj (@{$self->{dataObj}->{$parent}})
    {
      if ($tmpObj->{name} eq $obj->{name} || $tmpObj->{number} eq $obj->{number})
      {
        die "$errStr  You have already defined an $tag with name = '$obj->{name}', code = '$obj->{number}'!\n";
      }
    }
    push @{$self->{dataObj}->{$parent}}, $obj;
  }
}

=item void getStocks(void)

 Gathers all <stocks> entries and stores in stocks[]

=cut

sub getStocks
{
  my $self = shift;
  my $errStr = "Portal::Accounting::XML::TransactionParser->getStocks()  - Error:";
  my $parent = "stocks";
  my $tag = "stock";

  my @nodes = $self->getNodes(path => "/$self->{type}/$parent");
  if (scalar @nodes == 0)
  {
    return;  # this is a valid scenario.
  }
  if (scalar @nodes > 1)
  {
    die "$errStr  You have too many <$parent> tags!  You should only have one!\n";
  }

  # now get all the $tag tags under $parent
  @nodes = $self->getNodes(path => "/$self->{type}/$parent/$tag");
  if (@nodes == 0)
  {
    return;  # this is a valid scenario.
  }
  # now foreach tag, process and create it's object.
  foreach my $node (@nodes)
  {
    my $nodeName = $node->getName;
    if ($nodeName ne $tag)
    {
      die "$errStr  <$nodeName> is invalid inside the <$parent> tag!\n";
    }
    # for this node, get the following tags
    my %tagInfo = ();
    foreach my $tagName (qw(ticker name code type market class))
    {
      my @tagNodes = $self->getNodes(path => $tagName, context => $node);
      if (@tagNodes == 0)
      {
        die "$errStr  <$tagName> is not defined in <$parent><$tag>!\n";
      }
      if (@tagNodes > 1)
      {
        die "$errStr  You have too many <$tagName> tags in <$parent><$tag>!  You should only have one!\n";
      }
      # get the $tagName tags contents and store in the tagInfo hash.
      my $value = $tagNodes[0]->textContent;
      $tagInfo{$tagName} = $value;
      print "$tagName = '$value'\n" if ($self->{debug});
    }
    # now instantiate the object and store it.
    my $obj = Portal::Accounting::Objects::StockObject->new(langObj => $self->{langObj}, name => $tagInfo{name},
              number => $tagInfo{code}, type => $tagInfo{type}, ticker => $tagInfo{ticker},
              market => $tagInfo{market}, class => $tagInfo{class});
    if ($obj->didErrorOccur)
    {
      die "$errStr\n" . $obj->errorMessage;
    }
    # first make sure we don't have another object with the same name, number or ticker defined.
    foreach my $tmpObj (@{$self->{dataObj}->{$parent}})
    {
      if ($tmpObj->{name} eq $obj->{name} || $tmpObj->{number} eq $obj->{number} || $tmpObj->{ticker} eq $obj->{ticker})
      {
        die "$errStr  You have already defined an $tag with name = '$obj->{name}', code = '$obj->{number}', ticker = '$obj->{ticker}'!\n";
      }
    }
    push @{$self->{dataObj}->{$parent}}, $obj;
  }
}

=item void getLedgers(void)

 Gathers all <ledgers> entries and stores in ledgers[]

=cut

sub getLedgers
{
  my $self = shift;
  my @ledgers = ();
  my $errStr = "Portal::Accounting::XML::TransactionParser->getLedgers()  - Error:";
  my $parent = "ledgers";
  my $tag = "ledger";

  my @nodes = $self->getNodes(path => "/$self->{type}/$parent");
  if (scalar @nodes == 0)
  {
    return;  # this is a valid scenario.
  }
  if (scalar @nodes > 1)
  {
    die "$errStr  You have too many <$parent> tags!  You should only have one!\n";
  }

  # now get all the $tag tags under $parent
  @nodes = $self->getNodes(path => "/$self->{type}/$parent/$tag");
  if (@nodes == 0)
  {
    return;  # this is a valid scenario.
  }
  # now foreach tag, process and create it's object.
  foreach my $node (@nodes)
  {
    my $obj = $self->getLedger(node => $node);
    push @{$self->{dataObj}->{$parent}}, $obj;
  }
}

=item XMLLedger getLedger(node)

 Gathers the information about the ledger at node and
 returns a Portal::Accounting::Objects::XMLLedger
 Object which represents all the information about that
 Ledger.

=cut
sub getLedger
{
  my $self = shift;
  my %args = ( node => undef, @_ );
  my $node = $args{node};
  my $ledger = undef;
  my $errStr = "Portal::Accounting::XML::TransactionParser->getLedger()  - Error:";
  my $parent = "ledger";
  my $tag = "ledgerInfo";
  my %tagInfo = ();

  # instantiate the ledgerContainer object.
  $ledger = Portal::Accounting::Objects::LedgerContainer->new(langObj => $self->{langObj});
  if ($ledger->didErrorOccur)
  {
    die "$errStr\n" . $ledger->errorMessage;
  }

  # get the attributes for the <ledger> tag.
  my %attributes = $self->getAttributes(node => $node);
  foreach my $tagName (qw(name code))
  {
    if (!exists $attributes{$tagName})
    {
      die "$errStr  <$tagName> is not defined in <$parent>!\n";
    }
    my $value = $attributes{$tagName};
    $tagInfo{$tagName} = $value;
    print "$tagName = '$value'\n" if ($self->{debug});
  }

  # gather the <ledgerInfo> data.
  # now get all the $tag tags under $parent
  my @nodes = $self->getNodes(path => "$tag", context => $node);
  if (@nodes == 0)
  {
    die "$errStr  <$tag> is not defined in <ledgers><$parent>!\n";
  }
  if (@nodes > 1)
  {
    die "$errStr  You have too many <$tag> tags in <ledgers><$parent>!  You should only have one!\n";
  }

  # for this node, get the following tags
  foreach my $tagName (qw(color cashOrAccrual businessType businessTypeName))
  {
    my @tagNodes = $self->getNodes(path => $tagName, context => $nodes[0]);
    if (@tagNodes == 0)
    {
      die "$errStr  <$tagName> is not defined in <$parent><$tag>!\n";
    }
    if (@tagNodes > 1)
    {
      die "$errStr  You have too many <$tagName> tags in <$parent><$tag>!  You should only have one!\n";
    }
    # get the $tagName tags contents and store in the tagInfo hash.
    my $value = $tagNodes[0]->textContent;
    $tagInfo{$tagName} = $value;
    print "$tagName = '$value'\n" if ($self->{debug});
  }
  # gather the knownYears entries
  my %knownYears = $self->getKnownYears(node => $nodes[0], name => $tagInfo{name}, code => $tagInfo{code});

  # gather the dates entries
  my @startInfo = $self->getDates(node => $nodes[0], name => $tagInfo{name}, code => $tagInfo{code});

  # now instantiate the ledger object and store it.
  my $obj = Portal::Accounting::Objects::Ledger->new(langObj => $self->{langObj}, name => $tagInfo{name},
            code => $tagInfo{code}, color => $tagInfo{color}, cashOrAccrual => $tagInfo{cashOrAccrual},
            businessType => $tagInfo{businessType}, businessTypeName => $tagInfo{businessTypeName},
            knownYears => \%knownYears, start => \@startInfo);
  if ($obj->didErrorOccur)
  {
    die "$errStr\n" . $obj->errorMessage;
  }
  $ledger->{ledger} = $obj;

  # gather the <subEntities> values
  my @subEntities = $self->getSubEntities(node => $node, name => $tagInfo{name}, code => $tagInfo{code});

  $ledger->{subEntities} = \@subEntities;

  # gather the <journals> values
  my @journals = $self->getJournals(node => $node, name => $tagInfo{name}, code => $tagInfo{code});

  $ledger->{journals} = \@journals;

  # gather the <journalEntry> values (the heart of the document)
  my @journalEntries = $self->getJournalEntries(node => $node, name => $tagInfo{name}, code => $tagInfo{code});

  $ledger->{journalEntries} = \@journalEntries;

  return $ledger;
}

=item % getKnownYears(node, name, code)

 Gets all <knownYears> entries at node and returns to the caller

 Takes the current ledgers name and code.

=cut

sub getKnownYears
{
  my $self = shift;
  my %args = ( node => undef, name => "", code => "", @_ );
  my $node = $args{node};
  my $name = $args{name};
  my $code = $args{code};
  my %years = ();
  my $errStr = "Portal::Accounting::XML::TransactionParser->getKnownYears()  - Error:";

  my $parent = "knownYears";
  my $parentTag = "<ledgers><ledger name=\"$name\" code=\"$code\">";
  my $tag = "info";

  # verify we have our parent tag.
  my @nodes = $self->getNodes(path => "$parent", context => $node);
  if (@nodes == 0)
  {
    die "$errStr  <$parent> is not defined in $parentTag!\n";
  }
  if (@nodes > 1)
  {
    die "$errStr  You have too many <$parent> tags in $parentTag!  You can only have one.\n";
  }

  # now get all the $tag tags under $parent
  @nodes = $self->getNodes(path => "$tag", context => $nodes[0]);
  if (@nodes == 0)
  {
    die "$errStr  <$tag> is not defined in $parentTag<$parent>!\n";
  }
  # now foreach tag, process and create it's object.
  foreach my $infoNode (@nodes)
  {
    my $nodeName = $infoNode->getName;
    if ($nodeName ne $tag)
    {
      die "$errStr  <$nodeName> is invalid inside the $parentTag<$parent> tag!\n";
    }
    # for this node, get the following attributes
    my %tagInfo = ();
    my %attributes = $self->getAttributes(node => $infoNode);
    foreach my $tagName (qw(year endDate))
    {
      if (!exists $attributes{$tagName})
      {
        die "$errStr  <$tagName> is not defined in $parentTag<$parent><$tag>!\n";
      }
      my $value = $attributes{$tagName};
      $tagInfo{$tagName} = $value;
      print "$tagName = '$value'\n" if ($self->{debug});
    }

    # first make sure we don't have another date defined that conflicts.
    if (exists $years{$tagInfo{year}})
    {
      die "$errStr  year = '$tagInfo{year}' already defined in $parentTag<$parent><$tag>!\n";
    }
    elsif ($years{$tagInfo{year}} eq $tagInfo{endDate})
    {
      die "$errStr  endDate = '$tagInfo{endDate}' already defined in $parentTag<$parent><$tag>!\n";
    }
    else
    {
      $years{$tagInfo{year}} = $tagInfo{endDate};
    }
  }

  return %years;
}

=item @ getDates(node, name, code)

 Gets all <dates> entries at node and returns to the caller

 Takes the current ledgers name and code.

=cut

sub getDates
{
  my $self = shift;
  my %args = ( node => undef, name => "", code => "", @_ );
  my $node = $args{node};
  my $name = $args{name};
  my $code = $args{code};
  my @dates = ();
  my $offset = 0;
  my $errStr = "Portal::Accounting::XML::TransactionParser->getDates()  - Error:";

  my $parent = "dates";
  my $parentTag = "<ledgers><ledger name=\"$name\" code=\"$code\">";
  my $tag = "info";

  # verify we have our parent tag.
  my @nodes = $self->getNodes(path => "$parent", context => $node);
  if (@nodes == 0)
  {
    die "$errStr  <$parent> is not defined in $parentTag!\n";
  }
  if (@nodes > 1)
  {
    die "$errStr  You have too many <$parent> tags in $parentTag!  You can only have one.\n";
  }

  # now get all the $tag tags under $parent
  @nodes = $self->getNodes(path => "$tag", context => $nodes[0]);
  if (@nodes == 0)
  {
    die "$errStr  <$tag> is not defined in $parentTag<$parent>!\n";
  }
  # now foreach tag, process and create it's object.
  foreach my $infoNode (@nodes)
  {
    my $nodeName = $infoNode->getName;
    if ($nodeName ne $tag)
    {
      die "$errStr  <$nodeName> is invalid inside the $parentTag<$parent> tag!\n";
    }
    # for this node, get the following attributes
    my %tagInfo = ();
    my %attributes = $self->getAttributes(node => $infoNode);
    foreach my $tagName (qw(index start offset))
    {
      if (!exists $attributes{$tagName})
      {
        die "$errStr  <$tagName> is not defined in $parentTag<$parent><$tag>!\n";
      }
      my $value = $attributes{$tagName};
      $tagInfo{$tagName} = $value;
      print "$tagName = '$value'\n" if ($self->{debug});
    }

    if ($tagInfo{index} <= 0 || $tagInfo{index} > 12)
    {
      die "$errStr  index = '$tagInfo{index}' is invalid in $parentTag<$parent><$tag>!\n";
    }

    # first make sure we don't have another date defined that conflicts.
    if (defined $dates[$tagInfo{index}])
    {
      die "$errStr  index = '$tagInfo{index}' already defined in $parentTag<$parent><$tag>!\n";
    }
    if ($tagInfo{index} == 1)
    {
      $offset = $tagInfo{offset};
    }
    foreach my $date (@dates)
    {
      if ($date eq $tagInfo{start})
      {
        die "$errStr  start = '$tagInfo{start}' already defined in $parentTag<$parent><$tag> at index = '$tagInfo{index}'!\n";
      }
    }
    $dates[$tagInfo{index}] = $tagInfo{start};
  }

  # put the offset in index 0.
  $dates[0] = $offset;

  return @dates;
}

=item @SubEntity getSubEntities(node, name, code)

 Gets all <subEntities> entries at node and returns to the caller

 Takes the current ledgers name and code.

=cut

sub getSubEntities
{
  my $self = shift;
  my %args = ( node => undef, name => "", code => "", @_ );
  my $node = $args{node};
  my $name = $args{name};
  my $code = $args{code};
  my @subEntities = ();
  my $errStr = "Portal::Accounting::XML::TransactionParser->getSubEntities()  - Error:";

  my $parent = "subEntities";
  my $parentTag = "<ledgers><ledger name=\"$name\" code=\"$code\">";
  my $tag = "subEntity";

  # verify we have our parent tag.
  my @nodes = $self->getNodes(path => "$parent", context => $node);
  if (@nodes == 0)
  {
    return;  # this is a valid scenario.
  }
  if (@nodes > 1)
  {
    die "$errStr  You have too many <$parent> tags in $parentTag!  You can only have one.\n";
  }

  # now get all the $tag tags under $parent
  @nodes = $self->getNodes(path => "$tag", context => $nodes[0]);
  if (@nodes == 0)
  {
    return;  # This is a valid scenario.
  }
  foreach my $node (@nodes)
  {
    my %tagInfo = ();
    # for this node, get the following tags
    foreach my $tagName (qw(name code))
    {
      my @tagNodes = $self->getNodes(path => $tagName, context => $node);
      if (@tagNodes == 0)
      {
        die "$errStr  <$tagName> is not defined in $parentTag<$parent><$tag>!\n";
      }
      if (@tagNodes > 1)
      {
        die "$errStr  You have too many <$tagName> tags in $parentTag<$parent><$tag>!  You should only have one!\n";
      }
      # get the $tagName tags contents and store in the tagInfo hash.
      my $value = $tagNodes[0]->textContent;
      $tagInfo{$tagName} = $value;
      print "$tagName = '$value'\n" if ($self->{debug});
    }

    # verify we don't have any duplicates.
    foreach my $subEntity (@subEntities)
    {
      if ($subEntity->{name} eq $tagInfo{name})
      {
        die "$errStr  name = '$tagInfo{name}' already defined in $parentTag<$parent><$tag>!\n";
      }
      if ($subEntity->{code} eq $tagInfo{code})
      {
        die "$errStr  code = '$tagInfo{code}' already defined in $parentTag<$parent><$tag>!\n";
      }
    }

    # now create the SubEntity object.
    #!!! this may require a DB handle.  hmmm.  will have to think about that for objects
    my $obj = Portal::Accounting::Objects::SubEntity->new(langObj => $self->{langObj}, ledgerCode => $code);
    $obj->populate(name => $tagInfo{name}, code => $tagInfo{code}, ledgerCode => $code);
    if ($obj->error)
    {
      die "$errStr\n" . $obj->errorMessage;
    }
    push @subEntities, $obj;
  }

  return @subEntities;
}

=item @Journal getJournals(node, name, code)

 Gets all <journals> entries at node and returns to the caller

 Takes the current ledgers name and code.

=cut

sub getJournals
{
  my $self = shift;
  my %args = ( node => undef, name => "", code => "", @_ );
  my $node = $args{node};
  my $name = $args{name};
  my $code = $args{code};
  my @journals = ();
  my $errStr = "Portal::Accounting::XML::TransactionParser->getJournals()  - Error:";

  my $parent = "journals";
  my $parentTag = "<ledgers><ledger name=\"$name\" code=\"$code\">";
  my $tag = "journal";

  # verify we have our parent tag.
  my @nodes = $self->getNodes(path => "$parent", context => $node);
  if (@nodes == 0)
  {
    return;  # this is a valid scenario.
  }
  if (@nodes > 1)
  {
    die "$errStr  You have too many <$parent> tags in $parentTag!  You can only have one.\n";
  }

  # now get all the $tag tags under $parent
  @nodes = $self->getNodes(path => "$tag", context => $nodes[0]);
  if (@nodes == 0)
  {
    return;  # This is a valid scenario.
  }
  foreach my $node (@nodes)
  {
    my %tagInfo = ();
    # for this node, get the following tags
    foreach my $tagName (qw(name code))
    {
      my @tagNodes = $self->getNodes(path => $tagName, context => $node);
      if (@tagNodes == 0)
      {
        die "$errStr  <$tagName> is not defined in $parentTag<$parent><$tag>!\n";
      }
      if (@tagNodes > 1)
      {
        die "$errStr  You have too many <$tagName> tags in $parentTag<$parent><$tag>!  You should only have one!\n";
      }
      # get the $tagName tags contents and store in the tagInfo hash.
      my $value = $tagNodes[0]->textContent;
      $tagInfo{$tagName} = $value;
      print "$tagName = '$value'\n" if ($self->{debug});
    }

    # verify we don't have any duplicates.
    foreach my $journal (@journals)
    {
      if ($journal->{name} eq $tagInfo{name})
      {
        die "$errStr  name = '$tagInfo{name}' already defined in $parentTag<$parent><$tag>!\n";
      }
      if ($journal->{code} eq $tagInfo{code})
      {
        die "$errStr  code = '$tagInfo{code}' already defined in $parentTag<$parent><$tag>!\n";
      }
    }

    # now create the Journal object.
    my $obj = Portal::Accounting::Objects::Journal->new(langObj => $self->{langObj}, name => $tagInfo{name}, code => $tagInfo{code});
    if ($obj->didErrorOccur)
    {
      die "$errStr\n" . $obj->errorMessage;
    }
    push @journals, $obj;
  }

  return @journals;
}

=item @JournalEntry getJournalEntries(node, name, code)

 Gets all <journalEntry> entries at node and returns to the caller

 Takes the current ledgers name and code.

=cut

sub getJournalEntries
{
  my $self = shift;
  my %args = ( node => undef, name => "", code => "", @_ );
  my $node = $args{node};
  my $name = $args{name};
  my $code = $args{code};
  my @journalEntries = ();
  my $errStr = "Portal::Accounting::XML::TransactionParser->getJournalEntries()  - Error:";

  my $parent = "journalEntries";
  my $parentTag = "<ledgers><ledger name=\"$name\" code=\"$code\">";
  my $tag = "journalEntry";

  # verify we have our parent tag.
  my @nodes = $self->getNodes(path => "$parent", context => $node);
  if (@nodes == 0)
  {
    return;  # this is a valid scenario.
  }
  if (@nodes > 1)
  {
    die "$errStr  You have too many <$parent> tags in $parentTag!  You can only have one.\n";
  }

  # now get all the $tag tags under $parent
  @nodes = $self->getNodes(path => "$tag", context => $nodes[0]);
  if (@nodes == 0)
  {
    return;  # This is a valid scenario.
  }
  foreach my $node (@nodes)
  {
    my %tagInfo = ();
    # for this node, get the following tags
    my @requiredTagNames = qw(date journalCode);
    push @requiredTagNames, "origId" if ($self->{type} eq "transactionImport");
    push @requiredTagNames, "id" if ($self->{type} eq "transactionExport");

    my @optionalTagNames = qw(remembered memo payeePayer);

    foreach my $tagName (@requiredTagNames)
    {
      my @tagNodes = $self->getNodes(path => $tagName, context => $node);
      if (@tagNodes == 0)
      {
        die "$errStr  <$tagName> is not defined in $parentTag<$parent><$tag>!\n";
      }
      if (@tagNodes > 1)
      {
        die "$errStr  You have too many <$tagName> tags in $parentTag<$parent><$tag>!  You should only have one!\n";
      }
      # get the $tagName tags contents and store in the tagInfo hash.
      my $value = $tagNodes[0]->textContent;
      $tagInfo{$tagName} = $value;
      print "$tagName = '$value'\n" if ($self->{debug});
    }

    foreach my $tagName (@optionalTagNames)
    {
      my @tagNodes = $self->getNodes(path => $tagName, context => $node);
      if (@tagNodes == 0)
      {
        $tagInfo{$tagName} = "";  # default to being empty.
        $tagInfo{$tagName} = "false" if ($tagName eq "remembered");
        next;  # skip processing the rest.
      }
      if (@tagNodes > 1)
      {
        die "$errStr  You have too many <$tagName> tags in $parentTag<$parent><$tag>!  You should only have one!\n";
      }
      # get the $tagName tags contents and store in the tagInfo hash.
      my $value = $tagNodes[0]->textContent;
      $tagInfo{$tagName} = $value;
      print "$tagName = '$value'\n" if ($self->{debug});
    }

    # now parse the <lines> entries.
    my @rows = $self->getJournalEntryLines(node => $node, name => $name, code => $code, journalCode => $tagInfo{journalCode}, date => $tagInfo{date});

    # now create the Journal object.
    my $obj = Portal::Accounting::Objects::JournalEntry->new(langObj => $self->{langObj},
              referenceNum => ($self->{type} eq "transactionExport" ? $tagInfo{id} : -1), ledgerCode => $code,
              journalCode => $tagInfo{journalCode}, transDate => $tagInfo{date},
              payeePayer => $tagInfo{payeePayer}, memo => $tagInfo{memo},
              remember => $tagInfo{remembered}, rowObjects => \@rows,
              origId => ($self->{type} eq "transactionImport" ? $tagInfo{origId} : ""));
    if ($obj->didErrorOccur)
    {
      die "$errStr\n" . $obj->errorMessage;
    }
    push @journalEntries, $obj;
  }

  return @journalEntries;
}

=item @JournalEntryRow getJournalEntryLines(node, name, code, journalCode)

 Gets all <lines> entries at node and returns to the caller

 Takes the current ledgers name, code and journalCode.

=cut

sub getJournalEntryLines
{
  my $self = shift;
  my %args = ( node => undef, name => "", code => "", journalCode => "", date => "", @_ );
  my $node = $args{node};
  my $name = $args{name};
  my $code = $args{code};
  my $journalCode = $args{journalCode};
  my $date = $args{date};
  my @rows = ();
  my $errStr = "Portal::Accounting::XML::TransactionParser->getJournalEntryLines()  - Error:";

  my $parent = "lines";
  my $parentTag = "<ledgers><ledger name=\"$name\" code=\"$code\"><journalEntry journalCode=\"$journalCode\" date=\"$date\">";
  my $tag = "line";

  # verify we have our parent tag.
  my @nodes = $self->getNodes(path => "$parent", context => $node);
  if (@nodes == 0)
  {
    die "$errStr  You do not have any <$parent> tags in $parentTag!  You must have one.\n";
  }
  if (@nodes > 1)
  {
    die "$errStr  You have too many <$parent> tags in $parentTag!  You can only have one.\n";
  }

  # now get all the $tag tags under $parent
  @nodes = $self->getNodes(path => "$tag", context => $nodes[0]);
  if (@nodes == 0)
  {
    die "$errStr  You do not have any <$tag> tags in $parentTag<$parent>!  You must have at least two (2).\n";
  }
  if (@nodes < 2)
  {
    die "$errStr  You do not have enough <$tag> tags in $parentTag<$parent>!  You must have at least two (2).\n";
  }
  foreach my $node (@nodes)
  {
    my %tagInfo = ();
    # for this node, get the following tags
    my @requiredTagNames = qw(dorc accountCode amount);

    my @optionalTagNames = qw(memo subAccountCode comment subEntityCode cleared);

    foreach my $tagName (@requiredTagNames)
    {
      my @tagNodes = $self->getNodes(path => $tagName, context => $node);
      if (@tagNodes == 0)
      {
        die "$errStr  <$tagName> is not defined in $parentTag<$parent><$tag>!\n";
      }
      if (@tagNodes > 1)
      {
        die "$errStr  You have too many <$tagName> tags in $parentTag<$parent><$tag>!  You should only have one!\n";
      }
      # get the $tagName tags contents and store in the tagInfo hash.
      my $value = $tagNodes[0]->textContent;
      $tagInfo{$tagName} = $value;
      print "$tagName = '$value'\n" if ($self->{debug});
    }

    foreach my $tagName (@optionalTagNames)
    {
      my @tagNodes = $self->getNodes(path => $tagName, context => $node);
      if (@tagNodes == 0)
      {
        $tagInfo{$tagName} = "";  # default to being empty.
        $tagInfo{$tagName} = "-1" if ($tagName eq "subAccountCode");
        $tagInfo{$tagName} = " " if ($tagName eq "cleared");
        next;  # skip processing the rest.
      }
      if (@tagNodes > 1)
      {
        die "$errStr  You have too many <$tagName> tags in $parentTag<$parent><$tag>!  You should only have one!\n";
      }
      # get the $tagName tags contents and store in the tagInfo hash.
      my $value = $tagNodes[0]->textContent;
      $tagInfo{$tagName} = $value;
      print "$tagName = '$value'\n" if ($self->{debug});
    }

    # fixup anything that needs fixing.
    $tagInfo{dorc} = "debit" if ($tagInfo{dorc} eq "d");
    $tagInfo{dorc} = "credit" if ($tagInfo{dorc} eq "c");

    # now create the TransactionRow object.
    my $obj = Portal::Accounting::Objects::JournalEntryRow->new(langObj => $self->{langObj}, rowDate => $date,
              memo => $tagInfo{memo}, subEntity => $tagInfo{subEntityCode},
              accountNum => $tagInfo{accountCode}, subAccountNum => $tagInfo{subAccountCode},
              dorc => $tagInfo{dorc}, amount => $tagInfo{amount}, cleared => $tagInfo{cleared});
    if ($obj->didErrorOccur)
    {
      die "$errStr\n" . $obj->errorMessage;
    }
    push @rows, $obj;
  }

  return @rows;
}

=back

=cut

1;
__END__

=head1 VARIABLES

  dataFile - The xml file name we are working with or the contents
               of the string of xml passed in.

  importVersion - The version of the transactionImport XML file we require.

  exportVersion - The version of the transactionExport XML file we require.

  dataObj - Data object that represents the xml file.

  xmlObj - The XML::LibXML object being used to parse the XML File.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin at pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::TransactionImport(3), Portal::Accounting::Objects::TransactionExport(3)

=cut
